searchData={"items":[{"type":"module","title":"Commanded","doc":"Use Commanded to build your own Elixir applications following the CQRS/ES pattern.\n\nProvides support for:\n\n- [Defining applications](application.html).\n- [Command registration and dispatch](commands.html).\n- [Hosting and delegation to aggregates](aggregates.html).\n- [Event handling](events.html).\n- [Long running process managers](process-managers.html).\n\nUse Commanded with one of the following event stores for persistence:\n\n- Elixir [EventStore](https://hex.pm/packages/eventstore) using Postgres for persistence\n- [EventStoreDB](https://www.eventstore.com/)\n\nPlease check the [Getting Started](getting-started.html) and [Usage](usage.html) guides to learn more.","ref":"Commanded.html"},{"type":"function","title":"Commanded.aggregate_state/4","doc":"Retrieve aggregate state of an aggregate.\n\nRetrieving aggregate state is done by calling to the opened aggregate,\nor querying the event store for an optional state snapshot\nand then replaying the aggregate's event stream.","ref":"Commanded.html#aggregate_state/4"},{"type":"behaviour","title":"Commanded.Application","doc":"Defines a Commanded application.\n\nThe application expects at least an `:otp_app` option to be specified. It\nshould point to an OTP application that has the application configuration.\n\nFor example, the application:\n\n    defmodule MyApp.Application do\n      use Commanded.Application, otp_app: :my_app\n\n      router(MyApp.Router)\n    end\n\nCould be configured with:\n\n    # config/config.exs\n    config :my_app, MyApp.Application\n      event_store: [\n        adapter: Commanded.EventStore.Adapters.EventStore,\n        event_store: MyApp.EventStore\n      ],\n      pubsub: :local,\n      registry: :local\n\nAlternatively, you can include the configuration when defining the\napplication:\n\n    defmodule MyApp.Application do\n      use Commanded.Application,\n        otp_app: :my_app,\n        event_store: [\n          adapter: Commanded.EventStore.Adapters.EventStore,\n          event_store: MyApp.EventStore\n        ],\n        pubsub: :local,\n        registry: :local\n\n      router(MyApp.Router)\n    end\n\nA Commanded application must be started before it can be used:\n\n    {:ok, _pid} = MyApp.Application.start_link()\n\nInstead of starting the application manually, you should use a\n[Supervisor](supervision.html).","ref":"Commanded.Application.html"},{"type":"behaviour","title":"Supervision - Commanded.Application","doc":"Use a supervisor to start your Commanded application:\n\n    Supervisor.start_link([\n      MyApp.Application\n    ], strategy: :one_for_one)","ref":"Commanded.Application.html#module-supervision"},{"type":"behaviour","title":"Command routing - Commanded.Application","doc":"Commanded applications are also composite routers allowing you to include\none or more routers within an application.","ref":"Commanded.Application.html#module-command-routing"},{"type":"behaviour","title":"Example - Commanded.Application","doc":"defmodule MyApp.Application do\n      use Commanded.Application, otp_app: :my_app\n\n      router(MyApp.Accounts.Router)\n      router(MyApp.Billing.Router)\n      router(MyApp.Notifications.Router)\n    end\n\nSee `Commanded.Commands.CompositeRouter` for details.","ref":"Commanded.Application.html#module-example"},{"type":"behaviour","title":"Command dispatch - Commanded.Application","doc":"Once a router has been configured you can dispatch a command via the\napplication:\n\n    :ok = MyApp.dispatch(command, opts)\n\nSee `c:dispatch/1` and `c:dispatch/2` for details.","ref":"Commanded.Application.html#module-command-dispatch"},{"type":"behaviour","title":"Dynamic named applications - Commanded.Application","doc":"An application can be provided with a name as an option to `start_link/1`.\nThis can be used to start the same application multiple times, each using its\nown separately configured and isolated event store. Each application must be\nstarted with a unique name.\n\nMultiple instances of the same event handler or process manager can be\nstarted by referring to a started application by its name. The event store\noperations can also be scoped to an application by referring to its name.","ref":"Commanded.Application.html#module-dynamic-named-applications"},{"type":"behaviour","title":"Example - Commanded.Application","doc":"Start an application process for each tenant in a multi-tenanted app,\nguaranteeing that the data and processing remains isolated between tenants.\n\n    for tenant <- [:tenant1, :tenant2, :tenant3] do\n      {:ok, _app} = MyApp.Application.start_link(name: tenant)\n    end\n\nTypically you would start the applications using a supervisor:\n\n    children =\n      for tenant <- [:tenant1, :tenant2, :tenant3] do\n        {MyApp.Application, name: tenant}\n      end\n\n    Supervisor.start_link(children, strategy: :one_for_one)\n\nTo dispatch a command you must provide the application name:\n\n    :ok = MyApp.Application.dispatch(command, application: :tenant1)","ref":"Commanded.Application.html#module-example"},{"type":"behaviour","title":"Default dispatch options - Commanded.Application","doc":"An application can be configured with default command dispatch options such as\n`:consistency`, `:timeout`, and `:returning`. Any defaults will be used\nunless overridden by options provided to the dispatch function.\n\n    defmodule MyApp.Application do\n      use Commanded.Application,\n        otp_app: :my_app,\n        default_dispatch_opts: [\n          consistency: :eventual,\n          returning: :aggregate_version\n        ]\n    end\n\nSee the `Commanded.Commands.Router` module for more details about the\nsupported options.","ref":"Commanded.Application.html#module-default-dispatch-options"},{"type":"behaviour","title":"Telemetry - Commanded.Application","doc":"* `[:commanded, :application, :dispatch, :start]`\n  * Description: Emitted when an application starts dispatching a command\n  * Measurements: `%{system_time: integer()}`\n  * Metadata: `%{application: Commanded.Application.t(),\n  execution_context: Commanded.Aggregates.ExecutionContext.t()}\n`\n\n* `[:commanded, :application, :dispatch, :stop]`\n  * Description: Emitted when an application stops dispatching a command\n  * Measurements: `%{duration: non_neg_integer()}`\n  * Metadata: `%{application: Commanded.Application.t(),\n  execution_context: Commanded.Aggregates.ExecutionContext.t(),\n  error: nil | any()}\n`","ref":"Commanded.Application.html#module-telemetry"},{"type":"callback","title":"Commanded.Application.config/0","doc":"Returns the application configuration stored in the `:otp_app` environment.","ref":"Commanded.Application.html#c:config/0"},{"type":"callback","title":"Commanded.Application.dispatch/1","doc":"Dispatch a registered command.\n\n  - `command` is a command struct which must be registered with a\n    `Commanded.Commands.Router` and included in the application.","ref":"Commanded.Application.html#c:dispatch/1"},{"type":"callback","title":"Commanded.Application.dispatch/2","doc":"Dispatch a registered command.\n\n  - `command` is a command struct which must be registered with a\n    `Commanded.Commands.Router` and included in the application.\n\n  - `timeout_or_opts` is either an integer timeout or a keyword list of\n    options.\n\n    The timeout must be an integer greater than zero which specifies how many\n    milliseconds to allow the command to be handled, or the atom `:infinity`\n    to wait indefinitely. The default timeout value is five seconds.\n\n    Alternatively, an options keyword list can be provided, it supports the\n    following options.\n\n    Options:\n\n      - `causation_id` - an optional UUID used to identify the cause of the\n        command being dispatched.\n\n      - `correlation_id` - an optional UUID used to correlate related\n        commands/events together.\n\n      - `consistency` - to choose the consistency guarantee of the command dispatch.\n\n        The available options are:\n\n        - `:eventual` (default) - a successful command dispatch will return immediately.\n\n        - `:strong` - a successful command dispatch will block until all strongly\n          consistent event handlers and process managers have handled all events created by the command.\n\n        - An explicit list of event handler and process manager modules (or their configured names),\n          containing only those handlers you'd like to wait for. No other handlers will be awaited on,\n          regardless of their own configured consistency setting.\n          e.g. `[ExampleHandler, AnotherHandler]` or `[\"ExampleHandler\", \"AnotherHandler\"]`\n\n      - `metadata` - an optional map containing key/value pairs comprising\n        the metadata to be associated with all events created by the\n        command.\n\n      - `returning` - to choose what response is returned from a successful\n          command dispatch. The default is to return an `:ok`.\n\n          The available options are:\n\n          - `:aggregate_state` - to return the update aggregate state in the\n            successful response: `{:ok, aggregate_state}`.\n\n          - `:aggregate_version` - to include the aggregate stream version\n            in the successful response: `{:ok, aggregate_version}`.\n\n          - `:execution_result` - to return a `Commanded.Commands.ExecutionResult`\n            struct containing the aggregate's identity, version, and any\n            events produced from the command along with their associated\n            metadata.\n\n          - `false` - don't return anything except an `:ok`.\n\n      - `timeout` - as described above.\n\nReturns `:ok` on success unless the `:returning` option is specified where\nit returns one of `{:ok, aggregate_state}`, `{:ok, aggregate_version}`, or\n`{:ok, %Commanded.Commands.ExecutionResult{}}`.\n\nReturns `{:error, reason}` on failure.","ref":"Commanded.Application.html#c:dispatch/2"},{"type":"callback","title":"Example - Commanded.Application.dispatch/2","doc":"command = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n    :ok = BankApp.dispatch(command, timeout: 30_000)","ref":"Commanded.Application.html#c:dispatch/2-example"},{"type":"callback","title":"Commanded.Application.init/1","doc":"A callback executed when the application starts.\n\nIt must return `{:ok, keyword}` with the updated list of configuration.","ref":"Commanded.Application.html#c:init/1"},{"type":"callback","title":"Commanded.Application.start_link/1","doc":"Starts the application supervisor.\n\nReturns `{:ok, pid}` on success, `{:error, {:already_started, pid}}` if the\napplication is already started, or `{:error, term}` in case anything else goes\nwrong.","ref":"Commanded.Application.html#c:start_link/1"},{"type":"callback","title":"Commanded.Application.stop/2","doc":"Shuts down the application.","ref":"Commanded.Application.html#c:stop/2"},{"type":"type","title":"Commanded.Application.options/0","doc":"","ref":"Commanded.Application.html#t:options/0"},{"type":"type","title":"Commanded.Application.t/0","doc":"","ref":"Commanded.Application.html#t:t/0"},{"type":"module","title":"Commanded.UUID","doc":"Generate and format random, version 4 UUIDs.","ref":"Commanded.UUID.html"},{"type":"function","title":"Commanded.UUID.binary_to_string!/1","doc":"","ref":"Commanded.UUID.html#binary_to_string!/1"},{"type":"function","title":"Commanded.UUID.string_to_binary!/1","doc":"Converts a string representing a UUID into a raw binary.","ref":"Commanded.UUID.html#string_to_binary!/1"},{"type":"function","title":"Commanded.UUID.uuid4/0","doc":"Generates a random, version 4 UUID.","ref":"Commanded.UUID.html#uuid4/0"},{"type":"type","title":"Commanded.UUID.hex/0","doc":"A hex-encoded UUID string.","ref":"Commanded.UUID.html#t:hex/0"},{"type":"type","title":"Commanded.UUID.raw/0","doc":"A raw binary representation of a UUID.","ref":"Commanded.UUID.html#t:raw/0"},{"type":"module","title":"Commanded.Event.ErrorHandler","doc":"","ref":"Commanded.Event.ErrorHandler.html"},{"type":"function","title":"Commanded.Event.ErrorHandler.backoff/4","doc":"Backoff exponentially when an error is encountered.\n  * Adds up to 1 second of jitter.\n  * Minimum of 1 second delay\n  * Maximum of 24 hour delay","ref":"Commanded.Event.ErrorHandler.html#backoff/4"},{"type":"function","title":"Commanded.Event.ErrorHandler.stop_on_error/3","doc":"Stop the `Commanded.Event.Handler` with the reason given.","ref":"Commanded.Event.ErrorHandler.html#stop_on_error/3"},{"type":"module","title":"Commanded.Aggregate.Multi","doc":"Use `Commanded.Aggregate.Multi` to generate multiple events from a single\ncommand.\n\nThis can be useful when you want to emit multiple events that depend upon the\naggregate state being updated.","ref":"Commanded.Aggregate.Multi.html"},{"type":"module","title":"Example - Commanded.Aggregate.Multi","doc":"In the example below, money is withdrawn from the bank account and the\nupdated balance is used to check whether the account is overdrawn.\n\n    defmodule BankAccount do\n      alias Commanded.Aggregate.Multi\n\n      defstruct [:account_number, :state, balance: 0]\n\n      def withdraw(\n        %BankAccount{state: :active} = account,\n        %WithdrawMoney{amount: amount})\n        when is_number(amount) and amount > 0\n      do\n        account\n        |> Multi.new()\n        |> Multi.execute(&withdraw_money(&1, amount))\n        |> Multi.execute(&check_balance/1)\n      end\n\n      defp withdraw_money(%BankAccount{account_number: account_number, balance: balance}, amount) do\n        %MoneyWithdrawn{\n          account_number: account_number,\n          amount: amount,\n          balance: balance - amount\n        }\n      end\n\n      defp check_balance(%BankAccount{account_number: account_number, balance: balance})\n        when balance < 0\n      do\n        %AccountOverdrawn{account_number: account_number, balance: balance}\n      end\n      defp check_balance(%BankAccount{}), do: []\n    end","ref":"Commanded.Aggregate.Multi.html#module-example"},{"type":"function","title":"Commanded.Aggregate.Multi.execute/3","doc":"Adds a command execute function to the multi.\n\nIf `step_name` is provided, the aggregate state after that step is\nstored under that name. That can be useful in a long multi step multi\nin which one needs to know what was the agg state while procesisng\nthe multi. It's possible, then, to pattern match the step name in the\nsecond parameter of the anonymous function to be executed.","ref":"Commanded.Aggregate.Multi.html#execute/3"},{"type":"function","title":"Example - Commanded.Aggregate.Multi.execute/3","doc":"alias Commanded.Aggregate.Multi\n\n    aggregate\n    |> Multi.new()\n    |> Multi.execute(:interesting_event, fn aggregate ->\n      %Event{data: 1}\n    end)\n    |> Multi.execute(fn aggregate, %{interesting_event: aggregate_state_after_interesting_event} ->\n      %Event{data: 2}\n    end)","ref":"Commanded.Aggregate.Multi.html#execute/3-example"},{"type":"function","title":"Commanded.Aggregate.Multi.new/1","doc":"Create a new `Commanded.Aggregate.Multi` struct.","ref":"Commanded.Aggregate.Multi.html#new/1"},{"type":"function","title":"Commanded.Aggregate.Multi.reduce/4","doc":"Reduce an enumerable by executing the function for each item.\n\nThe aggregate `apply/2` function will be called after each event returned by\nthe execute function. This allows you to calculate values from the aggregate\nstate based upon events produced by previous items in the enumerable, such as\nrunning totals.\n\nIf `step_name` is provided, the aggregate state after that step will be\nstored under that name. That can be useful in a long multi step multi\nin which one needs to know what was the agg state while procesisng\nthe multi. It's possible, then, to pattern match the step name in the\nthird parameter of the anonymous function to be executed.","ref":"Commanded.Aggregate.Multi.html#reduce/4"},{"type":"function","title":"Examples - Commanded.Aggregate.Multi.reduce/4","doc":"alias Commanded.Aggregate.Multi\n\n    aggregate\n    |> Multi.new()\n    |> Multi.reduce([1, 2, 3], fn aggregate, item ->\n      %AnEvent{item: item, total: aggregate.total + item}\n    end)","ref":"Commanded.Aggregate.Multi.html#reduce/4-examples"},{"type":"function","title":"Example with named steps - Commanded.Aggregate.Multi.reduce/4","doc":"alias Commanded.Aggregate.Multi\n\n    aggregate\n    |> Multi.new()\n    |> Multi.execute(:first, fn aggregate ->\n      %AnEvent{item: nil, total: 0}\n    end)\n    |> Multi.reduce(:second, [1, 2, 3], fn aggregate, item ->\n      %AnEvent{item: item, total: aggregate.total + item}\n    end)\n    |> Multi.reduce([4, 5, 6], fn aggregate, item, steps ->\n       %{\n         first: aggregate_state_after_first_event,\n         second: aggregate_state_after_second_event\n       } = steps\n\n        %AnEvent{item: item, total: aggregate.total + item}\n    end)","ref":"Commanded.Aggregate.Multi.html#reduce/4-example-with-named-steps"},{"type":"function","title":"Commanded.Aggregate.Multi.run/1","doc":"Run the execute functions contained within the multi, returning the updated\naggregate state, the aggregate state for each named step and all created events.","ref":"Commanded.Aggregate.Multi.html#run/1"},{"type":"type","title":"Commanded.Aggregate.Multi.t/0","doc":"","ref":"Commanded.Aggregate.Multi.html#t:t/0"},{"type":"module","title":"Commanded.Aggregates.Aggregate","doc":"Aggregate is a `GenServer` process used to provide access to an\ninstance of an event sourced aggregate.\n\nIt allows execution of commands against an aggregate instance, and handles\npersistence of created events to the configured event store. Concurrent\ncommands sent to an aggregate instance are serialized and executed in the\norder received.\n\nThe `Commanded.Commands.Router` module will locate, or start, an aggregate\ninstance when a command is dispatched. By default, an aggregate process will\nrun indefinitely once started. Its lifespan may be controlled by using the\n`Commanded.Aggregates.AggregateLifespan` behaviour.","ref":"Commanded.Aggregates.Aggregate.html"},{"type":"module","title":"Snapshotting - Commanded.Aggregates.Aggregate","doc":"You can configure state snapshots for an aggregate in config. By default\nsnapshots are *not* taken for an aggregate. The following options are\navailable to enable snapshots:\n\n  - `snapshot_every` - snapshot aggregate state every so many events. Use\n    `nil` to disable snapshotting, or exclude the configuration entirely.\n\n  - `snapshot_version` - a non-negative integer indicating the version of\n    the aggregate state snapshot. Incrementing this version forces any\n    earlier recorded snapshots to be ignored when rebuilding aggregate\n    state.","ref":"Commanded.Aggregates.Aggregate.html#module-snapshotting"},{"type":"module","title":"Example - Commanded.Aggregates.Aggregate","doc":"In `config/config.exs` enable snapshots for `MyApp.ExampleAggregate` after\nevery ten events:\n\n    config :my_app, MyApp.Application,\n      snapshotting: %{\n        MyApp.ExampleAggregate => [\n          snapshot_every: 10,\n          snapshot_version: 1\n        ]\n      }","ref":"Commanded.Aggregates.Aggregate.html#module-example"},{"type":"module","title":"Telemetry - Commanded.Aggregates.Aggregate","doc":"* `[:commanded, :aggregate, :execute, :start]`\n  * Description: Emitted when an aggregate starts executing a command\n  * Measurements: `%{system_time: integer()}`\n  * Metadata: `%{application: Commanded.Application.t(),\n  aggregate_uuid: String.t(),\n  aggregate_state: struct(),\n  aggregate_version: non_neg_integer(),\n  caller: pid(),\n  execution_context: Commanded.Aggregates.ExecutionContext.t()}\n`\n\n* `[:commanded, :aggregate, :execute, :stop]`\n  * Description: Emitted when an aggregate stops executing a command\n  * Measurements: `%{duration: non_neg_integer()}`\n  * Metadata: `%{application: Commanded.Application.t(),\n  aggregate_uuid: String.t(),\n  aggregate_state: struct(),\n  aggregate_version: non_neg_integer(),\n  caller: pid(),\n  execution_context: Commanded.Aggregates.ExecutionContext.t(),\n  events: [map()],\n  error: nil | any()}\n`\n\n* `[:commanded, :aggregate, :execute, :exception]`\n  * Description: Emitted when an aggregate raises an exception\n  * Measurements: `%{duration: non_neg_integer()}`\n  * Metadata: `%{application: Commanded.Application.t(),\n  aggregate_uuid: String.t(),\n  aggregate_state: struct(),\n  aggregate_version: non_neg_integer(),\n  caller: pid(),\n  execution_context: Commanded.Aggregates.ExecutionContext.t(),\n  kind: :throw | :error | :exit,\n  reason: any(),\n  stacktrace: list()}\n`","ref":"Commanded.Aggregates.Aggregate.html#module-telemetry"},{"type":"function","title":"Commanded.Aggregates.Aggregate.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Commanded.Aggregates.Aggregate.html#child_spec/1"},{"type":"function","title":"Commanded.Aggregates.Aggregate.execute/5","doc":"Execute the given command against the aggregate.\n\n  - `aggregate_module` - the aggregate's module (e.g. `BankAccount`).\n  - `aggregate_uuid` - uniquely identifies an instance of the aggregate.\n  - `context` - includes command execution arguments\n    (see `Commanded.Aggregates.ExecutionContext` for details).\n  - `timeout` - an non-negative integer which specifies how many milliseconds\n    to wait for a reply, or the atom :infinity to wait indefinitely.\n    The default value is five seconds (5,000ms).","ref":"Commanded.Aggregates.Aggregate.html#execute/5"},{"type":"function","title":"Return values - Commanded.Aggregates.Aggregate.execute/5","doc":"Returns `{:ok, aggregate_version, events}` on success, or `{:error, error}`\non failure.\n\n  - `aggregate_version` - the updated version of the aggregate after executing\n     the command.\n  - `events` - events produced by the command, can be an empty list.","ref":"Commanded.Aggregates.Aggregate.html#execute/5-return-values"},{"type":"function","title":"Commanded.Aggregates.Aggregate.start_link/2","doc":"","ref":"Commanded.Aggregates.Aggregate.html#start_link/2"},{"type":"type","title":"Commanded.Aggregates.Aggregate.state/0","doc":"","ref":"Commanded.Aggregates.Aggregate.html#t:state/0"},{"type":"type","title":"Commanded.Aggregates.Aggregate.uuid/0","doc":"","ref":"Commanded.Aggregates.Aggregate.html#t:uuid/0"},{"type":"behaviour","title":"Commanded.Aggregates.AggregateLifespan","doc":"The `Commanded.Aggregates.AggregateLifespan` behaviour is used to control the\naggregate `GenServer` process lifespan.\n\nBy default an aggregate instance process will run indefinitely once started.\nYou can change this default by implementing the\n`Commanded.Aggregates.AggregateLifespan` behaviour in a module and configuring\nit in your router.\n\nAfter a command successfully executes, and creates at least one domain event,\nthe `c:after_event/1` function is called passing the last created event.\n\nWhen a command is successfully handled but results in no domain events (by\nreturning `nil` or an empty list `[]`), the command struct is passed to the\n`c:after_command/1` function.\n\nFinally, if there is an error executing the command, the error reason is\npassed to the `c:after_error/1` function.\n\nFor all the above, the returned inactivity timeout value is used to shutdown\nthe aggregate process if no other messages are received.","ref":"Commanded.Aggregates.AggregateLifespan.html"},{"type":"behaviour","title":"Supported return values - Commanded.Aggregates.AggregateLifespan","doc":"- Non-negative integer - specify an inactivity timeout, in millisconds.\n  - `:infinity` - prevent the aggregate instance from shutting down.\n  - `:hibernate` - send the process into hibernation.\n  - `:stop` - immediately shutdown the aggregate process with a `:normal` exit\n    reason.\n  - `{:stop, reason}` - immediately shutdown the aggregate process with the\n    given reason.","ref":"Commanded.Aggregates.AggregateLifespan.html#module-supported-return-values"},{"type":"behaviour","title":"Hibernation - Commanded.Aggregates.AggregateLifespan","doc":"A hibernated process will continue its loop once a message is in its message\nqueue. Hibernating an aggregate causes garbage collection and minimises the\nmemory used by the process. Hibernating should not be used aggressively as too\nmuch time could be spent garbage collecting.","ref":"Commanded.Aggregates.AggregateLifespan.html#module-hibernation"},{"type":"behaviour","title":"Example - Commanded.Aggregates.AggregateLifespan","doc":"Define a module that implements the `Commanded.Aggregates.AggregateLifespan`\nbehaviour:\n\n    defmodule BankAccountLifespan do\n      @behaviour Commanded.Aggregates.AggregateLifespan\n\n      def after_event(%MoneyDeposited{}), do: :timer.hours(1)\n      def after_event(%BankAccountClosed{}), do: :stop\n      def after_event(_event), do: :infinity\n\n      def after_command(%CloseAccount{}), do: :stop\n      def after_command(_command), do: :infinity\n\n      def after_error(:invalid_initial_balance), do: :timer.minutes(5)\n      def after_error(_error), do: :stop\n    end\n\nThen specify the module as the `lifespan` option when registering the\napplicable commands in your router:\n\n    defmodule BankRouter do\n      use Commanded.Commands.Router\n\n      dispatch [OpenAccount, CloseAccount],\n        to: BankAccount,\n        lifespan: BankAccountLifespan,\n        identity: :account_number\n    end","ref":"Commanded.Aggregates.AggregateLifespan.html#module-example"},{"type":"callback","title":"Commanded.Aggregates.AggregateLifespan.after_command/1","doc":"Aggregate process will be stopped after specified inactivity timeout unless\n`:infinity`, `:hibernate`, or `:stop` are returned.","ref":"Commanded.Aggregates.AggregateLifespan.html#c:after_command/1"},{"type":"callback","title":"Commanded.Aggregates.AggregateLifespan.after_error/1","doc":"Aggregate process will be stopped after specified inactivity timeout unless\n`:infinity`, `:hibernate`, or `:stop` are returned.","ref":"Commanded.Aggregates.AggregateLifespan.html#c:after_error/1"},{"type":"callback","title":"Commanded.Aggregates.AggregateLifespan.after_event/1","doc":"Aggregate process will be stopped after specified inactivity timeout unless\n`:infinity`, `:hibernate`, or `:stop` are returned.","ref":"Commanded.Aggregates.AggregateLifespan.html#c:after_event/1"},{"type":"type","title":"Commanded.Aggregates.AggregateLifespan.lifespan/0","doc":"","ref":"Commanded.Aggregates.AggregateLifespan.html#t:lifespan/0"},{"type":"module","title":"Commanded.Aggregates.AggregateStateBuilder","doc":"","ref":"Commanded.Aggregates.AggregateStateBuilder.html"},{"type":"function","title":"Commanded.Aggregates.AggregateStateBuilder.populate/1","doc":"Populate the aggregate's state from a snapshot, if present, and it's events.\n\nAttempt to fetch a snapshot for the aggregate to use as its initial state.\nIf the snapshot exists, fetch any subsequent events to rebuild its state.\nOtherwise start with the aggregate struct and stream all existing events for\nthe aggregate from the event store to rebuild its state from those events.","ref":"Commanded.Aggregates.AggregateStateBuilder.html#populate/1"},{"type":"function","title":"Commanded.Aggregates.AggregateStateBuilder.rebuild_from_events/1","doc":"Load events from the event store, in batches, to rebuild the aggregate state","ref":"Commanded.Aggregates.AggregateStateBuilder.html#rebuild_from_events/1"},{"type":"module","title":"Commanded.Aggregates.DefaultLifespan","doc":"The default implementation of the `Commanded.Aggregates.AggregateLifespan`\nbehaviour.\n\nIt will ensure that an aggregate instance process runs indefinitely once\nstarted, unless an exception is encountered.","ref":"Commanded.Aggregates.DefaultLifespan.html"},{"type":"function","title":"Commanded.Aggregates.DefaultLifespan.after_command/1","doc":"Aggregate will run indefinitely once started.","ref":"Commanded.Aggregates.DefaultLifespan.html#after_command/1"},{"type":"function","title":"Commanded.Aggregates.DefaultLifespan.after_error/1","doc":"Aggregate is stopped on exception, but will run indefinitely for any non-\nexception error.","ref":"Commanded.Aggregates.DefaultLifespan.html#after_error/1"},{"type":"function","title":"Commanded.Aggregates.DefaultLifespan.after_event/1","doc":"Aggregate will run indefinitely once started.","ref":"Commanded.Aggregates.DefaultLifespan.html#after_event/1"},{"type":"module","title":"Commanded.Aggregates.ExecutionContext","doc":"Defines the arguments used to execute a command for an aggregate.\n\nThe available options are:\n\n  - `command` - the command to execute, typically a struct\n    (e.g. `%OpenBankAccount{...}`).\n\n  - `retry_attempts` - the number of retries permitted if an\n    `{:error, :wrong_expected_version}` is encountered when appending events.\n\n  - `causation_id` - the UUID assigned to the dispatched command.\n\n  - `correlation_id` - a UUID used to correlate related commands/events.\n\n  - `metadata` - a map of key/value pairs containing the metadata to be\n    associated with all events created by the command.\n\n  - `handler` - the module that handles the command. It may be either the\n    aggregate module itself or a separate command handler module.\n\n  - `function` - the name of the function, as an atom, that handles the command.\n    The default value is `:execute`, used to support command dispatch directly\n    to the aggregate module. For command handlers the `:handle` function is\n    used.\n\n  - `before_execute` - the name of the function, as an atom, that prepares the\n    command before execution, called just before `function`. The default value\n    is `nil`, disabling it. It should return `:ok` on success or `{:error, any()}`\n    to cancel the dispatch.\n\n  - `lifespan` - a module implementing the `Commanded.Aggregates.AggregateLifespan`\n    behaviour to control the aggregate instance process lifespan. The default\n    value, `Commanded.Aggregates.DefaultLifespan`, keeps the process running\n    indefinitely.","ref":"Commanded.Aggregates.ExecutionContext.html"},{"type":"function","title":"Commanded.Aggregates.ExecutionContext.format_reply/3","doc":"","ref":"Commanded.Aggregates.ExecutionContext.html#format_reply/3"},{"type":"function","title":"Commanded.Aggregates.ExecutionContext.retry/1","doc":"","ref":"Commanded.Aggregates.ExecutionContext.html#retry/1"},{"type":"module","title":"Commanded.Aggregates.Supervisor","doc":"Supervises `Commanded.Aggregates.Aggregate` instance processes.","ref":"Commanded.Aggregates.Supervisor.html"},{"type":"function","title":"Commanded.Aggregates.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Commanded.Aggregates.Supervisor.html#child_spec/1"},{"type":"function","title":"Commanded.Aggregates.Supervisor.init/1","doc":"","ref":"Commanded.Aggregates.Supervisor.html#init/1"},{"type":"function","title":"Commanded.Aggregates.Supervisor.open_aggregate/3","doc":"Open an aggregate instance process for the given aggregate module and unique\nidentity.\n\nReturns `{:ok, aggregate_uuid}` when a process is successfully started, or is\nalready running.","ref":"Commanded.Aggregates.Supervisor.html#open_aggregate/3"},{"type":"function","title":"Commanded.Aggregates.Supervisor.start_link/1","doc":"","ref":"Commanded.Aggregates.Supervisor.html#start_link/1"},{"type":"module","title":"Commanded.Commands.CompositeRouter","doc":"Composite router allows you to combine multiple router modules into a single\nrouter able to dispatch any registered command from an included child router.\n\nOne example usage is to define a router per context and then combine each\ncontext's router into a single top-level composite app router used for all\ncommand dispatching.","ref":"Commanded.Commands.CompositeRouter.html"},{"type":"module","title":"Example - Commanded.Commands.CompositeRouter","doc":"Define a composite router module which imports the commands from each included\nrouter:\n\n    defmodule Bank.AppRouter do\n      use Commanded.Commands.CompositeRouter\n\n      router(Bank.Accounts.Router)\n      router(Bank.MoneyTransfer.Router)\n    end\n\n  One or more routers or composite routers can be included in a\n  `Commanded.Application`  since it is also a composite router:\n\n    defmodule BankApp do\n      use Commanded.Application\n\n      router(Bank.AppRouter)\n    end\n\nYou can dispatch a command via the application which will then be routed to\nthe associated child router:\n\n    command = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n    :ok = BankApp.dispatch(command)\n\nOr via the composite router itself, specifying the application:\n\n    :ok = Bank.AppRouter.dispatch(command, application: BankApp)\n\nA composite router can include composite routers.","ref":"Commanded.Commands.CompositeRouter.html#module-example"},{"type":"macro","title":"Commanded.Commands.CompositeRouter.router/1","doc":"Register a `Commanded.Commands.Router` module within this composite router.\n\nWill allow the composite router to dispatch any commands registered by the\nincluded router module. Multiple routers can be registered.","ref":"Commanded.Commands.CompositeRouter.html#router/1"},{"type":"module","title":"Commanded.Commands.ExecutionResult","doc":"Contains the aggregate, events, and metadata created by a successfully\nexecuted command.\n\nThe available fields are:\n\n  - `aggregate_uuid` - identity of the aggregate instance.\n\n  - `aggregate_state` - resultant state of the aggregate after executing\n    the command.\n\n  - `aggregate_version` - resultant version of the aggregate after executing\n    the command.\n\n  - `events` - a list of the created events, it may be an empty list.\n\n  - `metadata` - an map containing the metadata associated with the command\n    dispatch.","ref":"Commanded.Commands.ExecutionResult.html"},{"type":"type","title":"Commanded.Commands.ExecutionResult.t/0","doc":"","ref":"Commanded.Commands.ExecutionResult.html#t:t/0"},{"type":"behaviour","title":"Commanded.Commands.Handler","doc":"Defines the behaviour a command handler module must implement to support command dispatch.","ref":"Commanded.Commands.Handler.html"},{"type":"behaviour","title":"Example - Commanded.Commands.Handler","doc":"An open account handler that delegates to a bank account aggregate:\n\n    defmodule OpenAccountHandler do\n      @behaviour Commanded.Commands.Handler\n\n      def handle(%BankAccount{} = aggregate, %OpenAccount{} = command) do\n        %OpenAccount{account_number: account_number, initial_balance: initial_balance} = command\n\n        BankAccount.open_account(aggregate, account_number, initial_balance)\n      end\n    end","ref":"Commanded.Commands.Handler.html#module-example"},{"type":"callback","title":"Commanded.Commands.Handler.handle/2","doc":"Apply the given command to the event sourced aggregate.\n\nYou must return a single domain event, a list containing the pending events,\nor `nil`, `[]`, or `:ok` when no events are produced.\n\nYou should return `{:error, reason}` on failure.","ref":"Commanded.Commands.Handler.html#c:handle/2"},{"type":"type","title":"Commanded.Commands.Handler.aggregate/0","doc":"","ref":"Commanded.Commands.Handler.html#t:aggregate/0"},{"type":"type","title":"Commanded.Commands.Handler.command/0","doc":"","ref":"Commanded.Commands.Handler.html#t:command/0"},{"type":"type","title":"Commanded.Commands.Handler.domain_event/0","doc":"","ref":"Commanded.Commands.Handler.html#t:domain_event/0"},{"type":"type","title":"Commanded.Commands.Handler.reason/0","doc":"","ref":"Commanded.Commands.Handler.html#t:reason/0"},{"type":"behaviour","title":"Commanded.Commands.Router","doc":"Command routing macro to allow configuration of each command to its command handler.","ref":"Commanded.Commands.Router.html"},{"type":"behaviour","title":"Example - Commanded.Commands.Router","doc":"Define a router module which uses `Commanded.Commands.Router` and configures\navailable commands to dispatch:\n\n    defmodule BankRouter do\n      use Commanded.Commands.Router\n\n      dispatch OpenAccount,\n        to: OpenAccountHandler,\n        aggregate: BankAccount,\n        identity: :account_number\n    end\n\nThe `to` option determines which module receives the command being dispatched.\nThis command handler module must implement a `handle/2` function. It receives\nthe aggregate's state and the command to execute. Usually the command handler\nmodule will forward the command to the aggregate.\n\nOnce configured, you can either dispatch a command by using the module and\nspecifying the application:\n\n    command = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n    :ok = BankRouter.dispatch(command, application: BankApp)\n\nOr, more simply, you should include the router module in your application:\n\n    defmodule BankApp do\n      use Commanded.Application, otp_app: :my_app\n\n      router MyApp.Router\n    end\n\nThen dispatch commands using the app:\n\n    command = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n    :ok = BankApp.dispatch(command)","ref":"Commanded.Commands.Router.html#module-example"},{"type":"behaviour","title":"Dispatch command directly to an aggregate - Commanded.Commands.Router","doc":"You can route a command directly to an aggregate, without requiring an\nintermediate command handler.","ref":"Commanded.Commands.Router.html#module-dispatch-command-directly-to-an-aggregate"},{"type":"behaviour","title":"Example - Commanded.Commands.Router","doc":"defmodule BankRouter do\n      use Commanded.Commands.Router\n\n      # Will route to `BankAccount.open_account/2`\n      dispatch OpenAccount, to: BankAccount, identity: :account_number\n    end\n\nBy default, you must define an `execute/2` function on the aggregate module, which will be\ncalled with the aggregate's state and the command to execute. Using this approach, you must\ncreate an `execute/2` clause that pattern-matches on each command that the aggregate should\nhandle.\n\nAlternatively, you may specify the name of a function (also receiving both the aggregate state\nand the command) on your aggregate module to which the command will be dispatched:","ref":"Commanded.Commands.Router.html#module-example"},{"type":"behaviour","title":"Example - Commanded.Commands.Router","doc":"defmodule BankRouter do\n      use Commanded.Commands.Router\n\n      # Will route to `BankAccount.open_account/2`\n      dispatch OpenAccount, to: BankAccount, function: :open_account, identity: :account_number\n    end","ref":"Commanded.Commands.Router.html#module-example"},{"type":"behaviour","title":"Define aggregate identity - Commanded.Commands.Router","doc":"You can define the identity field for an aggregate once using the `identify` macro.\nThe configured identity will be used for all commands registered to the aggregate,\nunless overridden by a command registration.","ref":"Commanded.Commands.Router.html#module-define-aggregate-identity"},{"type":"behaviour","title":"Example - Commanded.Commands.Router","doc":"defmodule BankRouter do\n      use Commanded.Commands.Router\n\n      identify BankAccount,\n        by: :account_number,\n        prefix: \"bank-account-\"\n\n      dispatch OpenAccount, to: BankAccount\n    end\n\nAn optional identity prefix can be used to distinguish between different\naggregates that  would otherwise share the same identity. As an example you\nmight have a `User` and a `UserPreferences` aggregate that you wish\nto share the same identity. In this scenario you should specify a `prefix`\nfor each aggregate (e.g. `\"user-\"` and `\"user-preference-\"`).\n\nThe prefix is used as the stream identity when appending and reading the\naggregate's events: `\"  \"`. It can be a string or\na zero arity function returning a string.","ref":"Commanded.Commands.Router.html#module-example"},{"type":"behaviour","title":"Consistency - Commanded.Commands.Router","doc":"You can choose the consistency guarantee when dispatching a command. The\navailable options are:\n\n  - `:eventual` (default) - don't block command dispatch while waiting for\n    event handlers\n\n        :ok = BankApp.dispatch(command)\n        :ok = BankApp.dispatch(command, consistency: :eventual)\n\n  - `:strong` - block command dispatch until all strongly\n    consistent event handlers and process managers have successfully processed\n    all events created by the command.\n\n    Use this when you have event handlers that update read models you need to\n    query immediately after dispatching the command.\n\n        :ok = BankApp.dispatch(command, consistency: :strong)\n\n  - Provide an explicit list of event handler and process manager modules (or\n    their configured names), containing only those handlers you'd like to wait\n    for. No other handlers will be awaited on, regardless of their own\n    configured consistency setting.\n\n    ```elixir\n    :ok = BankApp.dispatch(command, consistency: [ExampleHandler, AnotherHandler])\n    :ok = BankApp.dispatch(command, consistency: [\"ExampleHandler\", \"AnotherHandler\"])\n    ```\n\n    Note you cannot opt-in to strong consistency for a handler that has been\n    configured as eventually consistent.","ref":"Commanded.Commands.Router.html#module-consistency"},{"type":"behaviour","title":"Dispatch return - Commanded.Commands.Router","doc":"By default a successful command dispatch will return `:ok`. You can change\nthis behaviour by specifying a `returning` option.\n\nThe supported options are:\n\n  - `:aggregate_state` - to return the update aggregate state.\n\n  - `:aggregate_version` - to return only the aggregate version.\n\n  - `:events` - to return the resultant domain events. An empty list will be\n    returned if no events were produced.\n\n  - `:execution_result` - to return a `Commanded.Commands.ExecutionResult`\n    struct containing the aggregate's identity, state, version, and any events\n    produced from the command along with their associated metadata.\n\n  - `false` - don't return anything except an `:ok`.","ref":"Commanded.Commands.Router.html#module-dispatch-return"},{"type":"behaviour","title":"Aggregate state - Commanded.Commands.Router","doc":"Return the updated aggregate state as part of the dispatch result:\n\n    {:ok, %BankAccount{}} = BankApp.dispatch(command, returning: :aggregate_state)\n\nThis is useful when you want to immediately return fields from the aggregate's\nstate without requiring an read model projection and waiting for the event(s)\nto be projected. It may also be appropriate to use this feature for unit\ntests.\n\nHowever, be warned that tightly coupling an aggregate's state with read\nrequests may be harmful. It's why CQRS enforces the separation of reads from\nwrites by defining two separate and specialised models.","ref":"Commanded.Commands.Router.html#module-aggregate-state"},{"type":"behaviour","title":"Aggregate version - Commanded.Commands.Router","doc":"You can optionally choose to return the aggregate's version as part of the\ndispatch result:\n\n    {:ok, aggregate_version} = BankApp.dispatch(command, returning: :aggregate_version)\n\nThis is useful when you need to wait for an event handler, such as a read model\nprojection, to be up-to-date before querying its data.","ref":"Commanded.Commands.Router.html#module-aggregate-version"},{"type":"behaviour","title":"Execution results - Commanded.Commands.Router","doc":"You can also choose to return the execution result as part of the dispatch\nresult:\n\n    alias Commanded.Commands.ExecutionResult\n\n    {:ok, %ExecutionResult{} = result} = BankApp.dispatch(command, returning: :execution_result)\n\nOr by setting the `default_dispatch_return` in your application config file:\n\n    # config/config.exs\n    config :commanded, default_dispatch_return: :execution_result\n\nUse the execution result struct to get information from the events produced\nfrom the command.","ref":"Commanded.Commands.Router.html#module-execution-results"},{"type":"behaviour","title":"Metadata - Commanded.Commands.Router","doc":"You can associate metadata with all events created by the command.\n\nSupply a map containing key/value pairs comprising the metadata:\n\n    :ok = BankApp.dispatch(command, metadata: %{\"ip_address\" => \"127.0.0.1\"})","ref":"Commanded.Commands.Router.html#module-metadata"},{"type":"callback","title":"Commanded.Commands.Router.dispatch/1","doc":"Dispatch the given command to the registered handler.\n\nReturns `:ok` on success, or `{:error, reason}` on failure.","ref":"Commanded.Commands.Router.html#c:dispatch/1"},{"type":"callback","title":"Example - Commanded.Commands.Router.dispatch/1","doc":"command = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n    :ok = BankRouter.dispatch(command)","ref":"Commanded.Commands.Router.html#c:dispatch/1-example"},{"type":"macro","title":"Commanded.Commands.Router.dispatch/2","doc":"Configure the command, or list of commands, to be dispatched to the\ncorresponding handler and aggregate.","ref":"Commanded.Commands.Router.html#dispatch/2"},{"type":"macro","title":"Example - Commanded.Commands.Router.dispatch/2","doc":"defmodule BankRouter do\n      use Commanded.Commands.Router\n\n      dispatch [OpenAccount, DepositMoney], to: BankAccount, identity: :account_number\n    end","ref":"Commanded.Commands.Router.html#dispatch/2-example"},{"type":"callback","title":"Commanded.Commands.Router.dispatch/2","doc":"Dispatch the given command to the registered handler providing a timeout.\n\n  - `command` is a command struct which must be registered with the router.\n\n  - `timeout_or_opts` is either an integer timeout, `:infinity`, or a keyword\n    list of options.\n\n    The timeout must be an integer greater than zero which specifies how many\n    milliseconds to allow the command to be handled, or the atom `:infinity`\n    to wait indefinitely. The default timeout value is five seconds.\n\n    Alternatively, an options keyword list can be provided with the following\n    options.\n\n    Options:\n\n      - `causation_id` - an optional UUID used to identify the cause of the\n        command being dispatched.\n\n      - `command_uuid` - an optional UUID used to identify the command being\n        dispatched.\n\n      - `correlation_id` - an optional UUID used to correlate related\n        commands/events together.\n\n      - `consistency` - one of `:eventual` (default) or `:strong`. By\n        setting the consistency to `:strong` a successful command dispatch\n        will block until all strongly consistent event handlers and process\n        managers have handled all events created by the command.\n\n      - `metadata` - an optional map containing key/value pairs comprising\n        the metadata to be associated with all events created by the\n        command.\n\n      - `returning` - to choose what response is returned from a successful\n          command dispatch. The default is to return an `:ok`.\n\n          The available options are:\n\n          - `:aggregate_state` - to return the update aggregate state in the\n            successful response: `{:ok, aggregate_state}`.\n\n          - `:aggregate_version` - to include the aggregate stream version\n            in the successful response: `{:ok, aggregate_version}`.\n\n          - `:events` - to return the resultant domain events. An empty list\n            will be returned if no events were produced.\n\n          - `:execution_result` - to return a `Commanded.Commands.ExecutionResult`\n            struct containing the aggregate's identity, version, and any\n            events produced from the command along with their associated\n            metadata.\n\n          - `false` - don't return anything except an `:ok`.\n\n      - `timeout` - as described above.\n\nReturns `:ok` on success unless the `:returning` option is specified where\nit returns one of `{:ok, aggregate_state}`, `{:ok, aggregate_version}`, or\n`{:ok, %Commanded.Commands.ExecutionResult{}}`.\n\nReturns `{:error, reason}` on failure.","ref":"Commanded.Commands.Router.html#c:dispatch/2"},{"type":"callback","title":"Example - Commanded.Commands.Router.dispatch/2","doc":"command = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n    :ok = BankRouter.dispatch(command, consistency: :strong, timeout: 30_000)","ref":"Commanded.Commands.Router.html#c:dispatch/2-example"},{"type":"macro","title":"Commanded.Commands.Router.identify/2","doc":"Define an aggregate's identity\n\nYou can define the identity field for an aggregate using the `identify` macro.\nThe configured identity will be used for all commands registered to the\naggregate, unless overridden by a command registration.","ref":"Commanded.Commands.Router.html#identify/2"},{"type":"macro","title":"Example - Commanded.Commands.Router.identify/2","doc":"defmodule BankRouter do\n      use Commanded.Commands.Router\n\n      identify BankAccount,\n        by: :account_number,\n        prefix: \"bank-account-\"\n    end","ref":"Commanded.Commands.Router.html#identify/2-example"},{"type":"macro","title":"Commanded.Commands.Router.middleware/1","doc":"Include the given middleware module to be called before and after\nsuccess or failure of each command dispatch\n\nThe middleware module must implement the `Commanded.Middleware` behaviour.\n\nMiddleware modules are executed in the order they are defined.","ref":"Commanded.Commands.Router.html#middleware/1"},{"type":"macro","title":"Example - Commanded.Commands.Router.middleware/1","doc":"defmodule BankRouter do\n      use Commanded.Commands.Router\n\n      middleware CommandLogger\n      middleware MyCommandValidator\n      middleware AuthorizeCommand\n\n      dispatch [OpenAccount, DepositMoney], to: BankAccount, identity: :account_number\n    end","ref":"Commanded.Commands.Router.html#middleware/1-example"},{"type":"type","title":"Commanded.Commands.Router.dispatch_resp/0","doc":"","ref":"Commanded.Commands.Router.html#t:dispatch_resp/0"},{"type":"module","title":"Commanded.Event.FailureContext","doc":"Data related to an event handling failure.\n\nThe available fields are:\n\n  - `:application` - the associated `Commanded.Application`.\n\n  - `:handler_name` - the name of the event handler.\n\n  - `:handler_state` - optional event handler state.\n\n  - `:context` - a map that is passed between each failure. Use it to store\n    any transient state between failures. As an example it could be used to\n    count error failures and stop or skip the problematic event after too\n    many.\n\n  - `:metadata` - the metadata associated with the failed event.\n\n  - `:stacktrace` - the stacktrace if the error was an unhandled exception.","ref":"Commanded.Event.FailureContext.html"},{"type":"type","title":"Commanded.Event.FailureContext.t/0","doc":"","ref":"Commanded.Event.FailureContext.html#t:t/0"},{"type":"behaviour","title":"Commanded.Event.Handler","doc":"Defines the behaviour an event handler must implement and\nprovides a convenience macro that implements the behaviour, allowing you to\nhandle only the events you are interested in processing.\n\nYou should start your event handlers using a [Supervisor](supervision.html) to\nensure they are restarted on error.","ref":"Commanded.Event.Handler.html"},{"type":"behaviour","title":"Example - Commanded.Event.Handler","doc":"defmodule ExampleHandler do\n      use Commanded.Event.Handler,\n        application: ExampleApp,\n        name: \"ExampleHandler\"\n\n      def handle(%AnEvent{..}, _metadata) do\n        # ... process the event\n        :ok\n      end\n    end\n\nStart your event handler process (or use a [Supervisor](supervision.html)):\n\n    {:ok, _handler} = ExampleHandler.start_link()","ref":"Commanded.Event.Handler.html#module-example"},{"type":"behaviour","title":"Event handler name - Commanded.Event.Handler","doc":"The name you specify is used when subscribing to the event store. You must use\na unique name for each event handler and process manager you start. Also, you\n*should not* change the name once the handler has been deployed. A new\nsubscription will be created if you change the name and the event handler will\nreceive already handled events.\n\nYou can use the module name of your event handler using the `__MODULE__`\nspecial form:\n\n    defmodule ExampleHandler do\n      use Commanded.Event.Handler,\n        application: ExampleApp,\n        name: __MODULE__\n    end","ref":"Commanded.Event.Handler.html#module-event-handler-name"},{"type":"behaviour","title":"Subscription options - Commanded.Event.Handler","doc":"You can choose to start the event handler's event store subscription from\n`:origin`, `:current` position, or an exact event number using the\n`start_from` option. The default is to use the origin so your handler will\nreceive *all* events.\n\nUse the `:current` position when you don't want newly created event handlers\nto go through all previous events. An example would be adding an event handler\nto send transactional emails to an already deployed system containing many\nhistorical events.\n\nThe `start_from` option *only applies* when the subscription is initially\ncreated, the first time the handler starts. Whenever the handler restarts the\nsubscription will resume from the next event after the last successfully\nprocessed event. Restarting an event handler does not restart its\nsubscription.","ref":"Commanded.Event.Handler.html#module-subscription-options"},{"type":"behaviour","title":"Example - Commanded.Event.Handler","doc":"Set the `start_from` option (`:origin`, `:current`, or an explicit event\nnumber) when using `Commanded.Event.Handler`:\n\n    defmodule ExampleHandler do\n      use Commanded.Event.Handler,\n        application: ExampleApp,\n        name: \"ExampleHandler\",\n        start_from: :origin\n    end\n\nYou can optionally override `:start_from` by passing it as option when\nstarting your handler:\n\n    {:ok, _handler} = ExampleHandler.start_link(start_from: :current)","ref":"Commanded.Event.Handler.html#module-example"},{"type":"behaviour","title":"Subscribing to an individual stream - Commanded.Event.Handler","doc":"By default event handlers will subscribe to all events appended to any stream.\nProvide a `subscribe_to` option to subscribe to a single stream.\n\n    defmodule ExampleHandler do\n      use Commanded.Event.Handler,\n        application: ExampleApp,\n        name: __MODULE__,\n        subscribe_to: \"stream1234\"\n    end\n\nThis will ensure the handler only receives events appended to that stream.","ref":"Commanded.Event.Handler.html#module-subscribing-to-an-individual-stream"},{"type":"behaviour","title":"Runtime event handler configuration - Commanded.Event.Handler","doc":"Runtime options can be provided to the event handler's `start_link/1` function\nor its child spec. The `c:init/1` callback function can also be used to define\nruntime configuration.","ref":"Commanded.Event.Handler.html#module-runtime-event-handler-configuration"},{"type":"behaviour","title":"Example - Commanded.Event.Handler","doc":"Provide runtime configuration to `start_link/1`:\n\n    {:ok, _pid} =\n      ExampleHandler.start_link(\n        application: ExampleApp,\n        name: \"ExampleHandler\"\n      )\n\nOr when supervised:\n\n    Supervisor.start_link([\n      {ExampleHandler, application: ExampleApp, name: \"ExampleHandler\"}\n    ], strategy: :one_for_one)","ref":"Commanded.Event.Handler.html#module-example"},{"type":"behaviour","title":"Event handler state - Commanded.Event.Handler","doc":"An event handler can define and update state which is held in the `GenServer`\nprocess memory. It is passed to the `handle/2` function as part of the\nmetadata using the `:state` key. The state is transient and will be lost\nwhenever the process restarts.\n\nInitial state can be set in the `init/1` callback function by adding a\n`:state` key to the config. It can also be provided when starting the handler\nprocess:\n\n    ExampleHandler.start_link(state: initial_state)\n\nOr when supervised:\n\n    Supervisor.start_link([\n      {ExampleHandler, state: initial_state}\n    ], strategy: :one_for_one)\n\nState can be updated by returning `{:ok, new_state}` from any `handle/2`\nfunction. Returning an `:ok` reply will keep the state unchanged.\n\nHandler state is also included in the `Commanded.Event.FailureContext` struct\npassed to the `error/3` callback function.","ref":"Commanded.Event.Handler.html#module-event-handler-state"},{"type":"behaviour","title":"Example - Commanded.Event.Handler","doc":"defmodule StatefulEventHandler do\n      use Commanded.Event.Handler,\n        application: ExampleApp,\n        name: __MODULE__\n\n      def init(config) do\n        config = Keyword.put_new(config, :state, %{})\n\n        {:ok, config}\n      end\n\n      def handle(event, metadata) do\n        %{state: state} = metadata\n\n        new_state = mutate_state(state)\n\n        {:ok, new_state}\n      end\n    end","ref":"Commanded.Event.Handler.html#module-example"},{"type":"behaviour","title":"Concurrency - Commanded.Event.Handler","doc":"An event handler may be configured to start multiple processes to handle the\nevents concurrently. By default one process will be started, processing events\none at a time in order. The `:concurrency` option determines how many event\nhandler processes are started. It must be a positive integer.\n\nNote with concurrent processing events will likely by processed out of order.\nIf you need to enforce an order, such as per stream or by using a field from\nan event, you can define a `c:partition_by/2` callback function in the event\nhandler module. The function will receive each event and its metadata and must\nreturn a consistent term indicating the event's partition. Events which return\nthe same term are guaranteed to be processed in order by the same event\nhandler instance. While events with different partitions may be processed\nconcurrently by another instance. An attempt will be made to distribute\nevents as evenly as possible to all running event handler instances.\n\nOnly `:eventual` consistency is supported when multiple handler processes are\nconfigured with a `:concurrency` of greater than one.","ref":"Commanded.Event.Handler.html#module-concurrency"},{"type":"behaviour","title":"Example - Commanded.Event.Handler","doc":"defmodule ConcurrentProcssingEventHandler do\n      alias Commanded.EventStore.RecordedEvent\n\n      use Commanded.Event.Handler,\n        application: ExampleApp,\n        name: __MODULE__,\n        concurrency: 10\n\n      def init(config) do\n        # Fetch the index of this event handler instance (0..9 in this example)\n        index = Keyword.fetch!(config, :index)\n\n        {:ok, config}\n      end\n\n      def handle(event, metadata) do\n        :ok\n      end\n\n      # Partition events by their stream\n      def partition_by(event, metadata) do\n        %{stream_id: stream_id} = metadata\n\n        stream_id\n      end\n    end","ref":"Commanded.Event.Handler.html#module-example"},{"type":"behaviour","title":"Consistency - Commanded.Event.Handler","doc":"For each event handler you can define its consistency, as one of either\n`:strong` or `:eventual`.\n\nThis setting is used when dispatching commands and specifying the\n`consistency` option.\n\nWhen you dispatch a command using `:strong` consistency, after successful\ncommand dispatch the process will block until all event handlers configured to\nuse `:strong` consistency have processed the domain events created by the\ncommand. This is useful when you have a read model updated by an event handler\nthat you wish to query for data affected by the command dispatch. With\n`:strong` consistency you are guaranteed that the read model will be\nup-to-date after the command has successfully dispatched. It can be safely\nqueried for data updated by any of the events created by the command.\n\nThe default setting is `:eventual` consistency. Command dispatch will return\nimmediately upon confirmation of event persistence, not waiting for any event\nhandlers.\n\nNote strong consistency does not imply a transaction covers the command\ndispatch and event handling. It only guarantees that the event handler will\nhave processed all events produced by the command: if event handling fails\nthe events will have still been persisted.","ref":"Commanded.Event.Handler.html#module-consistency"},{"type":"behaviour","title":"Example - Commanded.Event.Handler","doc":"Define an event handler with `:strong` consistency:\n\n    defmodule ExampleHandler do\n      use Commanded.Event.Handler,\n        application: ExampleApp,\n        name: \"ExampleHandler\",\n        consistency: :strong\n    end","ref":"Commanded.Event.Handler.html#module-example"},{"type":"behaviour","title":"Dynamic application - Commanded.Event.Handler","doc":"An event handler's application can be provided as an option to `start_link/1`.\nThis can be used to start the same handler multiple times, each using a\nseparate Commanded application and event store.","ref":"Commanded.Event.Handler.html#module-dynamic-application"},{"type":"behaviour","title":"Example - Commanded.Event.Handler","doc":"Start an event handler process for each tenant in a multi-tenanted app,\nguaranteeing that the data and processing remains isolated between tenants.\n\n    for tenant <- [:tenant1, :tenant2, :tenant3] do\n      {:ok, _app} = MyApp.Application.start_link(name: tenant)\n      {:ok, _handler} = ExampleHandler.start_link(application: tenant)\n    end\n\nTypically you would start the event handlers using a supervisor:\n\n    children =\n      for tenant <- [:tenant1, :tenant2, :tenant3] do\n        {ExampleHandler, application: tenant}\n      end\n\n    Supervisor.start_link(children, strategy: :one_for_one)\n\nThe above example requires three named Commanded applications to have already\nbeen started.","ref":"Commanded.Event.Handler.html#module-example"},{"type":"behaviour","title":"Telemetry - Commanded.Event.Handler","doc":"* `[:commanded, :event, :handle, :start]`\n  * Description: Emitted when an event handler starts handling an event\n  * Measurements: `%{system_time: integer()}`\n  * Metadata: `%{application: Commanded.Application.t(),\n  context: map(),\n  handler_name: String.t(),\n  handler_module: atom(),\n  handler_state: map(),\n  recorded_event: RecordedEvent.t()}\n`\n\n* `[:commanded, :event, :handle, :stop]`\n  * Description: Emitted when an event handler stops handling an event\n  * Measurements: `%{duration: non_neg_integer()}`\n  * Metadata: `%{:application => Commanded.Application.t(),\n  :context => map(),\n  :handler_name => String.t(),\n  :handler_module => atom(),\n  :handler_state => map(),\n  :recorded_event => RecordedEvent.t(),\n  optional(:error) => any()}\n`\n\n* `[:commanded, :event, :handle, :exception]`\n  * Description: Emitted when an event handler raises an exception\n  * Measurements: `%{duration: non_neg_integer()}`\n  * Metadata: `%{application: Commanded.Application.t(),\n  context: map(),\n  handler_name: String.t(),\n  handler_module: atom(),\n  handler_state: map(),\n  recorded_event: RecordedEvent.t(),\n  kind: :throw | :error | :exit,\n  reason: any(),\n  stacktrace: list()}\n`","ref":"Commanded.Event.Handler.html#module-telemetry"},{"type":"callback","title":"Commanded.Event.Handler.after_start/1","doc":"Optional initialisation callback function called when the handler starts.\n\nCan be used to start any related processes when the event handler is started.\n\nThis callback function must return `:ok`, `{:ok, state}` to return new state,\nor `{:stop, reason}` to stop the handler process. Any other return value\nwill terminate the event handler with an error.","ref":"Commanded.Event.Handler.html#c:after_start/1"},{"type":"callback","title":"Example - Commanded.Event.Handler.after_start/1","doc":"defmodule ExampleHandler do\n      use Commanded.Event.Handler,\n        application: ExampleApp,\n        name: \"ExampleHandler\"\n\n      # Optional initialisation\n      def after_start(handler_state) do\n        new_handler_state = Map.put(handler_state, :foo, \"bar\")\n        {:ok, new_handler_state}\n      end\n\n      def handle(%AnEvent{..}, _metadata) do\n        # Process the event ...\n        :ok\n      end\n    end","ref":"Commanded.Event.Handler.html#c:after_start/1-example"},{"type":"callback","title":"Commanded.Event.Handler.before_reset/0","doc":"Called before an event handler gets reset","ref":"Commanded.Event.Handler.html#c:before_reset/0"},{"type":"function","title":"Commanded.Event.Handler.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Commanded.Event.Handler.html#child_spec/1"},{"type":"callback","title":"Commanded.Event.Handler.error/3","doc":"Called when an event `handle/2` callback returns an error.\n\nThe `c:error/3` function allows you to control how event handling failures\nare handled. The function is passed the error returned by the event handler\n(e.g. `{:error, :failure}`), the event causing the error, and a context map\ncontaining state passed between retries.\n\nUse pattern matching on the error and/or failed event to explicitly handle\ncertain errors or events. Use the context map to track any transient state you\nneed to access between retried failures.\n\nYou can return one of the following responses depending upon the\nerror severity:\n\n- `{:retry, context}` - retry the failed event, provide a context\n  map, or updated `Commanded.Event.FailureContext` struct, containing any\n  state to be passed to subsequent failures. This could be used to count the\n  number of failures, stopping after too many.\n\n- `{:retry, delay, context}` - retry the failed event, after sleeping for\n  the requested delay (in milliseconds). Context is a map or\n  `Commanded.Event.FailureContext` struct as described in `{:retry, context}`\n  above.\n\n- `:skip` - skip the failed event by acknowledging receipt.\n\n- `{:stop, reason}` - stop the event handler with the given reason.\n\nThe default behaviour if you don't provide an `c:error/3` callback is to stop\nthe event handler using the exact error reason returned from the `handle/2`\nfunction. If the event handler is supervised using restart `permanent` or\n`transient` stopping on error will cause the handler to be restarted. It will\nlikely crash again as it will reprocesse the problematic event. This can lead\nto cascading failures going up the supervision tree.","ref":"Commanded.Event.Handler.html#c:error/3"},{"type":"callback","title":"Example error handling - Commanded.Event.Handler.error/3","doc":"defmodule ExampleHandler do\n      use Commanded.Event.Handler,\n        application: ExampleApp,\n        name: __MODULE__\n\n      require Logger\n\n      alias Commanded.Event.FailureContext\n\n      def handle(%AnEvent{}, _metadata) do\n        # simulate event handling failure\n        {:error, :failed}\n      end\n\n      def error({:error, :failed}, %AnEvent{} = event, %FailureContext{context: context}) do\n        context = record_failure(context)\n\n        case Map.get(context, :failures) do\n          too_many when too_many >= 3 ->\n            # skip bad event after third failure\n            Logger.warning(\"Skipping bad event, too many failures: \" <> inspect(event))\n\n            :skip\n\n          _ ->\n            # retry event, failure count is included in context map\n            {:retry, context}\n        end\n      end\n\n      defp record_failure(context) do\n        Map.update(context, :failures, 1, fn failures -> failures + 1 end)\n      end\n    end","ref":"Commanded.Event.Handler.html#c:error/3-example-error-handling"},{"type":"callback","title":"Commanded.Event.Handler.handle/2","doc":"Handle a domain event and its metadata.\n\nReturn `:ok` on success, `{:error, :already_seen_event}` to ack and skip the\nevent, or `{:error, reason}` on failure.","ref":"Commanded.Event.Handler.html#c:handle/2"},{"type":"callback","title":"Commanded.Event.Handler.init/0","doc":"","ref":"Commanded.Event.Handler.html#c:init/0"},{"type":"callback","title":"Commanded.Event.Handler.init/1","doc":"Optional callback function called to configure the handler before it starts.\n\nIt is passed the merged compile-time and runtime config, and must return the\nupdated config as `{:ok, config}`.\n\nNote this function is called before the event handler process is started and\n*is not* run from the handler's process. You cannot use `self()` to access the\nhandler's PID.","ref":"Commanded.Event.Handler.html#c:init/1"},{"type":"callback","title":"Example - Commanded.Event.Handler.init/1","doc":"The `c:init/1` function is used to define the handler's application and name\nbased upon a value provided at runtime:\n\n    defmodule ExampleHandler do\n      use Commanded.Event.Handler\n\n      def init(config) do\n        {tenant, config} = Keyword.pop!(config, :tenant)\n\n        config =\n          config\n          |> Keyword.put(:application, Module.concat([ExampleApp, tenant]))\n          |> Keyword.put(:name, Module.concat([__MODULE__, tenant]))\n\n        {:ok, config}\n      end\n    end\n\nUsage:\n\n    {:ok, _pid} = ExampleHandler.start_link(tenant: :tenant1)","ref":"Commanded.Event.Handler.html#c:init/1-example"},{"type":"callback","title":"Commanded.Event.Handler.partition_by/2","doc":"Determine which partition an event belongs to.\n\nOnly applicable when an event handler has been configured with more than one\ninstance via the `:concurrency` option.","ref":"Commanded.Event.Handler.html#c:partition_by/2"},{"type":"type","title":"Commanded.Event.Handler.consistency/0","doc":"","ref":"Commanded.Event.Handler.html#t:consistency/0"},{"type":"type","title":"Commanded.Event.Handler.domain_event/0","doc":"","ref":"Commanded.Event.Handler.html#t:domain_event/0"},{"type":"type","title":"Commanded.Event.Handler.metadata/0","doc":"","ref":"Commanded.Event.Handler.html#t:metadata/0"},{"type":"type","title":"Commanded.Event.Handler.subscribe_from/0","doc":"","ref":"Commanded.Event.Handler.html#t:subscribe_from/0"},{"type":"module","title":"Commanded.Event.Mapper","doc":"Map events to/from the structs used for persistence.","ref":"Commanded.Event.Mapper.html"},{"type":"module","title":"Example - Commanded.Event.Mapper","doc":"Map domain event structs to `Commanded.EventStore.EventData` structs in\npreparation for appending to the configured event store:\n\n    events = [%ExampleEvent1{}, %ExampleEvent2{}]\n    event_data = Commanded.Event.Mapper.map_to_event_data(events)\n\n    :ok = Commanded.EventStore.append_to_stream(\"stream-1234\", :any_version, event_data)","ref":"Commanded.Event.Mapper.html#module-example"},{"type":"function","title":"Commanded.Event.Mapper.map_from_recorded_event/1","doc":"Map an `Commanded.EventStore.RecordedEvent` struct to its event data.","ref":"Commanded.Event.Mapper.html#map_from_recorded_event/1"},{"type":"function","title":"Commanded.Event.Mapper.map_from_recorded_events/1","doc":"Map a list of `Commanded.EventStore.RecordedEvent` structs to their event data.","ref":"Commanded.Event.Mapper.html#map_from_recorded_events/1"},{"type":"function","title":"Commanded.Event.Mapper.map_to_event_data/2","doc":"Map a domain event (or list of events) to an\n`Commanded.EventStore.EventData` struct (or list of structs).\n\nOptionally, include the `causation_id`, `correlation_id`, and `metadata`\nassociated with the event(s).","ref":"Commanded.Event.Mapper.html#map_to_event_data/2"},{"type":"function","title":"Examples - Commanded.Event.Mapper.map_to_event_data/2","doc":"event_data = Commanded.Event.Mapper.map_to_event_data(%ExampleEvent{})\n\n    event_data =\n      Commanded.Event.Mapper.map_to_event_data(\n        [\n          %ExampleEvent1{},\n          %ExampleEvent2{}\n        ],\n        causation_id: Commanded.UUID.uuid4(),\n        correlation_id: Commanded.UUID.uuid4(),\n        metadata: %{\"user_id\" => user_id}\n      )","ref":"Commanded.Event.Mapper.html#map_to_event_data/2-examples"},{"type":"type","title":"Commanded.Event.Mapper.event/0","doc":"","ref":"Commanded.Event.Mapper.html#t:event/0"},{"type":"protocol","title":"Commanded.Event.Upcaster","doc":"Protocol to allow an event to be transformed before being passed to a\nconsumer.\n\nYou can use an upcaster to change the shape of an event (e.g. add a new field\nwith a default, rename a field) or rename an event.\n\nUpcaster will run for new events and for historical events.\n\nBecause the upcaster changes any historical event to the latest version,\nconsumers (aggregates, event handlers, and process managers) only need\nto support the latest version.","ref":"Commanded.Event.Upcaster.html"},{"type":"protocol","title":"Example - Commanded.Event.Upcaster","doc":"defimpl Commanded.Event.Upcaster, for: AnEvent do\n      def upcast(%AnEvent{} = event, _metadata) do\n        %AnEvent{name: name} = event\n\n        %AnEvent{event | first_name: name}\n      end\n    end","ref":"Commanded.Event.Upcaster.html#module-example"},{"type":"protocol","title":"Metadata - Commanded.Event.Upcaster","doc":"The `upcast/2` function receives the domain event and a map of metadata\nassociated with that event. The metadata is provided during command dispatch.\n\nIn addition to the metadata key/values you provide, the following system\nvalues will be included in the metadata:\n\n  - `application` - the `Commanded.Application` used to read the event.\n  - `event_id` - a globally unique UUID to identify the event.\n  - `event_number` - a globally unique, monotonically incrementing integer\n    used to order the event amongst all events.\n  - `stream_id` - the stream identity for the event.\n  - `stream_version` - the version of the stream for the event.\n  - `causation_id` - an optional UUID identifier used to identify which\n    command caused the event.\n  - `correlation_id` - an optional UUID identifier used to correlate related\n    commands/events.\n  - `created_at` - the datetime, in UTC, indicating when the event was\n    created.\n\nThese key/value metadata pairs will use atom keys to differentiate them from\nthe user provided metadata which uses string keys.","ref":"Commanded.Event.Upcaster.html#module-metadata"},{"type":"function","title":"Commanded.Event.Upcaster.upcast/2","doc":"","ref":"Commanded.Event.Upcaster.html#upcast/2"},{"type":"type","title":"Commanded.Event.Upcaster.t/0","doc":"All the types that implement this protocol.","ref":"Commanded.Event.Upcaster.html#t:t/0"},{"type":"module","title":"Commanded.ProcessManagers.FailureContext","doc":"Data related to a process manager event handling or command dispatch failure.\n\nThe available fields are:\n\n  - `context` - the context map passed between each failure and may be used\n    to track state between retries, such as to count failures.\n\n  - `enriched_metadata` - the enriched metadata associated with the event.\n\n  - `last_event` - the last event the process manager received.\n\n  - `pending_commands` - the pending commands that were not executed yet.\n\n  - `process_manager_state` - the state the process manager would be in\n    if the event handling or command dispatch had not failed.\n\n  - `stacktrace` - the stacktrace if the error was an unhandled exception.","ref":"Commanded.ProcessManagers.FailureContext.html"},{"type":"type","title":"Commanded.ProcessManagers.FailureContext.t/0","doc":"","ref":"Commanded.ProcessManagers.FailureContext.html#t:t/0"},{"type":"behaviour","title":"Commanded.ProcessManagers.ProcessManager","doc":"Macro used to define a process manager.\n\nA process manager is responsible for coordinating one or more aggregates.\nIt handles events and dispatches commands in response. Process managers have\nstate that can be used to track which aggregates are being orchestrated.\n\nProcess managers can be used to implement long-running transactions by\nfollowing the saga pattern. This is a sequence of commands and their\ncompensating commands which can be used to rollback on failure.\n\nUse the `Commanded.ProcessManagers.ProcessManager` macro in your process\nmanager module and implement the callback functions defined in the behaviour:\n\n- `c:interested?/1`\n- `c:interested?/2`\n- `c:handle/2`\n- `c:handle/3`\n- `c:apply/2`\n- `c:after_command/2`\n- `c:after_command/3`\n- `c:error/3`\n\nPlease read the [Process managers](process-managers.html) guide for more\ndetail.","ref":"Commanded.ProcessManagers.ProcessManager.html"},{"type":"behaviour","title":"Example - Commanded.ProcessManagers.ProcessManager","doc":"defmodule ExampleProcessManager do\n      use Commanded.ProcessManagers.ProcessManager,\n        application: ExampleApp,\n        name: \"ExampleProcessManager\"\n\n      defstruct []\n\n      def interested?(%AnEvent{uuid: uuid}), do: {:start, uuid}\n\n      def interested?(%AnotherEvent{}, metadata),\n        do: {:continue, Map.fetch!(metadata, :correlation_id)}\n\n      def handle(%ExampleProcessManager{}, %ExampleEvent{}) do\n        [\n          %ExampleCommand{}\n        ]\n      end\n\n      def handle(%ExampleProcessManager{}, %AnotherEvent{}, _metadata) do\n        [\n          %AnotherCommand{}\n        ]\n      end\n\n      def after_command(%ExampleProcessManager{}, %ExampleCommand{}) do\n        :continue\n      end\n\n      def after_command(%ExampleProcessManager{}, %AnotherCommand{}, _metadata) do\n        :stop\n      end\n\n      def error({:error, failure}, %ExampleEvent{}, _failure_context) do\n        # Retry, skip, ignore, or stop process manager on error handling event\n        :skip\n      end\n\n      def error({:error, failure}, %ExampleCommand{}, _failure_context) do\n        # Retry, skip, ignore, or stop process manager on error dispatching command\n        :skip\n      end\n    end\n\nStart the process manager (or configure as a worker inside a [Supervisor](supervision.html))\n\n    {:ok, process_manager} = ExampleProcessManager.start_link()\n\n## `c:init/1` callback\n\nAn `c:init/1` function can be defined in your process manager which is used to\nprovide runtime configuration. This callback function must return\n`{:ok, config}` with the updated config.","ref":"Commanded.ProcessManagers.ProcessManager.html#module-example"},{"type":"behaviour","title":"Example - Commanded.ProcessManagers.ProcessManager","doc":"The `c:init/1` function is used to define the process manager's application\nand name based upon a value provided at runtime:\n\n    defmodule ExampleProcessManager do\n      use Commanded.ProcessManagers.ProcessManager\n\n      def init(config) do\n        {tenant, config} = Keyword.pop!(config, :tenant)\n\n        config =\n          config\n          |> Keyword.put(:application, Module.concat([ExampleApp, tenant]))\n          |> Keyword.put(:name, Module.concat([__MODULE__, tenant]))\n\n        {:ok, config}\n      end\n    end\n\nUsage:\n\n    {:ok, _pid} = ExampleProcessManager.start_link(tenant: :tenant1)","ref":"Commanded.ProcessManagers.ProcessManager.html#module-example"},{"type":"behaviour","title":"Error handling - Commanded.ProcessManagers.ProcessManager","doc":"You can define an `c:error/3` callback function to handle any errors or\nexceptions during event handling or returned by commands dispatched from your\nprocess manager. The function is passed the error (e.g. `{:error, :failure}`),\nthe failed event or command, and a failure context.\nSee `Commanded.ProcessManagers.FailureContext` for details.\n\nUse pattern matching on the error and/or failed event/command to explicitly\nhandle certain errors, events, or commands. You can choose to retry, skip,\nignore, or stop the process manager after a command dispatch error.\n\nThe default behaviour, if you don't provide an `c:error/3` callback, is to\nstop the process manager using the exact error reason returned from the\nevent handler function or command dispatch. You should supervise your\nprocess managers to ensure they are restarted on error.","ref":"Commanded.ProcessManagers.ProcessManager.html#module-error-handling"},{"type":"behaviour","title":"Example - Commanded.ProcessManagers.ProcessManager","doc":"defmodule ExampleProcessManager do\n      use Commanded.ProcessManagers.ProcessManager,\n        application: ExampleApp,\n        name: \"ExampleProcessManager\"\n\n      # stop process manager after three failures\n      def error({:error, _failure}, _failed_command, %{context: %{failures: failures}})\n        when failures >= 2\n      do\n        {:stop, :too_many_failures}\n      end\n\n      # retry command, record failure count in context map\n      def error({:error, _failure}, _failed_command, %{context: context}) do\n        context = Map.update(context, :failures, 1, fn failures -> failures + 1 end)\n\n        {:retry, context}\n      end\n    end","ref":"Commanded.ProcessManagers.ProcessManager.html#module-example"},{"type":"behaviour","title":"Idle process timeouts - Commanded.ProcessManagers.ProcessManager","doc":"Each instance of a process manager will run indefinitely once started. To\nreduce memory usage you can configure an idle timeout, in milliseconds,\nafter which the process will be shutdown.\n\nThe process will be restarted whenever another event is routed to it and its\nstate will be rehydrated from the instance snapshot.","ref":"Commanded.ProcessManagers.ProcessManager.html#module-idle-process-timeouts"},{"type":"behaviour","title":"Example - Commanded.ProcessManagers.ProcessManager","doc":"defmodule ExampleProcessManager do\n      use Commanded.ProcessManagers.ProcessManager,\n        application: ExampleApp,\n        name: \"ExampleProcessManager\"\n        idle_timeout: :timer.minutes(10)\n    end","ref":"Commanded.ProcessManagers.ProcessManager.html#module-example"},{"type":"behaviour","title":"Event handling timeout - Commanded.ProcessManagers.ProcessManager","doc":"You can configure a timeout for event handling to ensure that events are\nprocessed in a timely manner without getting stuck.\n\nAn `event_timeout` option, defined in milliseconds, may be provided when using\nthe `Commanded.ProcessManagers.ProcessManager` macro at compile time:\n\n    defmodule TransferMoneyProcessManager do\n      use Commanded.ProcessManagers.ProcessManager,\n        application: ExampleApp,\n        name: \"TransferMoneyProcessManager\",\n        router: BankRouter,\n        event_timeout: :timer.minutes(10)\n    end\n\nOr may be configured when starting a process manager:\n\n    {:ok, _pid} = TransferMoneyProcessManager.start_link(\n      event_timeout: :timer.hours(1)\n    )\n\nAfter the timeout has elapsed, indicating the process manager has not\nprocessed an event within the configured period, the process manager is\nstopped. The process manager will be restarted if supervised and will retry\nthe event, this should help resolve transient problems.","ref":"Commanded.ProcessManagers.ProcessManager.html#module-event-handling-timeout"},{"type":"behaviour","title":"Consistency - Commanded.ProcessManagers.ProcessManager","doc":"For each process manager you can define its consistency, as one of either\n`:strong` or `:eventual`.\n\nThis setting is used when dispatching commands and specifying the\n`consistency` option.\n\nWhen you dispatch a command using `:strong` consistency, after successful\ncommand dispatch the process will block until all process managers configured\nto use `:strong` consistency have processed the domain events created by the\ncommand.\n\nThe default setting is `:eventual` consistency. Command dispatch will return\nimmediately upon confirmation of event persistence, not waiting for any\nprocess managers.","ref":"Commanded.ProcessManagers.ProcessManager.html#module-consistency"},{"type":"behaviour","title":"Example - Commanded.ProcessManagers.ProcessManager","doc":"Define a process manager with `:strong` consistency:\n\n    defmodule ExampleProcessManager do\n      use Commanded.ProcessManagers.ProcessManager,\n        application: ExampleApp,\n        name: \"ExampleProcessManager\",\n        consistency: :strong\n    end","ref":"Commanded.ProcessManagers.ProcessManager.html#module-example"},{"type":"behaviour","title":"Dynamic application - Commanded.ProcessManagers.ProcessManager","doc":"A process manager's application can be provided as an option to `start_link/1`.\nThis can be used to start the same process manager multiple times, each using a\nseparate Commanded application and event store.","ref":"Commanded.ProcessManagers.ProcessManager.html#module-dynamic-application"},{"type":"behaviour","title":"Example - Commanded.ProcessManagers.ProcessManager","doc":"Start an process manager for each tenant in a multi-tenanted app, guaranteeing\nthat the data and processing remains isolated between tenants.\n\n    for tenant <- [:tenant1, :tenant2, :tenant3] do\n      {:ok, _app} = MyApp.Application.start_link(name: tenant)\n      {:ok, _handler} = ExampleProcessManager.start_link(application: tenant)\n    end\n\nTypically you would start the event handlers using a supervisor:\n\n    children =\n      for tenant <- [:tenant1, :tenant2, :tenant3] do\n        {ExampleProcessManager, application: tenant}\n      end\n\n    Supervisor.start_link(children, strategy: :one_for_one)\n\nThe above example requires three named Commanded applications to have already\nbeen started.","ref":"Commanded.ProcessManagers.ProcessManager.html#module-example"},{"type":"behaviour","title":"Telemetry - Commanded.ProcessManagers.ProcessManager","doc":"* `[:commanded, :process_manager, :handle, :start]`\n  * Description: Emitted when a process manager starts handling an event\n  * Measurements: `%{system_time: integer()}`\n  * Metadata: `%{application: Commanded.Application.t(),\n  process_manager_name: String.t() | Inspect.t(),\n  process_manager_module: module(),\n  process_state: term(),\n  process_uuid: String.t()}\n`\n\n* `[:commanded, :process_manager, :handle, :stop]`\n  * Description: Emitted when a process manager stops handling an event\n  * Measurements: `%{duration: non_neg_integer()}`\n  * Metadata: `%{application: Commanded.Application.t(),\n  commands: [struct()],\n  error: nil | any(),\n  process_manager_name: String.t() | Inspect.t(),\n  process_manager_module: module(),\n  process_state: term(),\n  process_uuid: String.t()}\n`\n\n* `[:commanded, :process_manager, :handle, :exception]`\n  * Description: Emitted when a process manager raises an exception\n  * Measurements: `%{duration: non_neg_integer()}`\n  * Metadata: `%{application: Commanded.Application.t(),\n  process_manager_name: String.t() | Inspect.t(),\n  process_manager_module: module(),\n  process_state: term(),\n  process_uuid: String.t(),\n  kind: :throw | :error | :exit,\n  reason: any(),\n  stacktrace: list()}\n`","ref":"Commanded.ProcessManagers.ProcessManager.html#module-telemetry"},{"type":"callback","title":"Commanded.ProcessManagers.ProcessManager.after_command/2","doc":"Stop the process manager instance after a command is successfully\ndispatched.\n\nSee `c:after_command/3` for details.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:after_command/2"},{"type":"callback","title":"Commanded.ProcessManagers.ProcessManager.after_command/3","doc":"Stop the process manager instance after a command is successfully\ndispatched.\n\nThe `c:after_command/3` function can be omitted if you do not need to stop\nafter a specific command or if you would instead use the `c:interested?/2`\nstop mechanism.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:after_command/3"},{"type":"callback","title":"Commanded.ProcessManagers.ProcessManager.apply/2","doc":"Mutate the process manager's state by applying the domain event.\n\nSee `c:apply/3` function for details.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:apply/2"},{"type":"callback","title":"Commanded.ProcessManagers.ProcessManager.apply/3","doc":"Mutate the process manager's state by applying the domain event.\n\nThe `c:apply/3` function is used to mutate the process manager's state. It\nreceives the current state, the domain event and the event metadata, and must\nreturn the modified state.\n\nThis callback function is optional, the default behaviour is to retain the\nprocess manager's current state.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:apply/3"},{"type":"callback","title":"Commanded.ProcessManagers.ProcessManager.error/3","doc":"Called when a command dispatch or event handling returns an error.\n\nThe `c:error/3` function allows you to control how event  handling and command\ndispatch and failures are handled. The function is passed the error (e.g.\n`{:error, :failure}`), the failed event (during failed event handling) or\nfailed command (during failed dispatch), and a failure context struct (see\n`Commanded.ProcessManagers.FailureContext` for details).\n\nThe failure context contains a context map you can use to pass transient state\nbetween failures. For example it can be used to count the number of failures.\n\nYou can return one of the following responses depending upon the\nerror severity:\n\n- `{:retry, context}` - retry the failed command, provide a context\n  map or `Commanded.ProcessManagers.FailureContext` struct, containing any\n  state passed to subsequent failures. This could be used to count the number\n  of retries, failing after too many attempts.\n\n- `{:retry, delay, context}` - retry the failed command, after sleeping for\n  the requested delay (in milliseconds). Context is a map or\n  `Commanded.ProcessManagers.FailureContext` as described in\n  `{:retry, context}` above.\n\n- `{:stop, reason}` - stop the process manager with the given reason.\n\nFor event handling failures, when failure source is an event, you can also\nreturn:\n\n- `:skip` - to skip the problematic event. No commands will be dispatched.\n\nFor command dispatch failures, when failure source is a command, you can also\nreturn:\n\n- `:skip` - skip the failed command and continue dispatching any pending\n  commands.\n\n- `{:skip, :continue_pending}` - skip the failed command, but continue\n  dispatching any pending commands.\n\n- `{:skip, :discard_pending}` - discard the failed command and any pending\n  commands.\n\n- `{:continue, commands, context}` - continue dispatching the given commands.\n  This allows you to retry the failed command, modify it and retry, drop it\n  or drop all pending commands by passing an empty list `[]`. Context is a map\n  as described in `{:retry, context}` above.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:error/3"},{"type":"callback","title":"Commanded.ProcessManagers.ProcessManager.handle/2","doc":"Process manager instance handles a domain event, returning any commands to\ndispatch.\n\nSee `c:handle/3` function for details.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:handle/2"},{"type":"callback","title":"Commanded.ProcessManagers.ProcessManager.handle/3","doc":"Process manager instance handles a domain event, returning any commands to\ndispatch.\n\nA `c:handle/3` function can be defined for each `:start` and `:continue`\ntagged event previously specified. It receives the process manager's state,\nevent to be handled, and the event's enriched metadata. It must return the\ncommands to be dispatched. This may be none, a single command, or many\ncommands.\n\nThe `c:handle/3` function can be omitted if you do not need to dispatch a\ncommand and are only mutating the process manager's state.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:handle/3"},{"type":"function","title":"Commanded.ProcessManagers.ProcessManager.identity/0","doc":"Get the identity of the current process instance.\n\nThis must only be called within a process manager's `handle/2`, `handle/3`, `apply/2`, or\n`apply/3` callback functions.","ref":"Commanded.ProcessManagers.ProcessManager.html#identity/0"},{"type":"function","title":"Example - Commanded.ProcessManagers.ProcessManager.identity/0","doc":"defmodule ExampleProcessManager do\n      use Commanded.ProcessManagers.ProcessManager,\n        application: MyApp.Application,\n        name: __MODULE__\n\n      def interested?(%ProcessStarted{uuids: uuids}), do: {:start, uuids}\n\n      def handle(%IdentityProcessManager{}, %ProcessStarted{} = event) do\n        # Identify which uuid is associated with the current instance from the\n        # list of uuids in the event.\n        uuid = Commanded.ProcessManagers.ProcessManager.identity()\n\n        # ...\n      end\n    end","ref":"Commanded.ProcessManagers.ProcessManager.html#identity/0-example"},{"type":"callback","title":"Commanded.ProcessManagers.ProcessManager.init/1","doc":"Optional callback function called to configure the process manager before it\nstarts.\n\nIt is passed the merged compile-time and runtime config, and must return the\nupdated config.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:init/1"},{"type":"callback","title":"Commanded.ProcessManagers.ProcessManager.interested?/1","doc":"Is the process manager interested in the given event?\n\nSee `c:interested?/2` for details.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:interested?/1"},{"type":"callback","title":"Commanded.ProcessManagers.ProcessManager.interested?/2","doc":"Is the process manager interested in the given event?\n\nThe `c:interested?/2` function is used to indicate which events\nthe process manager receives. The response is used to route the event to an existing\ninstance or start a new process instance:\n\n- `{:start, process_uuid}` - create a new instance of the process manager.\n- `{:start!, process_uuid}` - create a new instance of the process manager (strict).\n- `{:continue, process_uuid}` - continue execution of an existing process manager.\n- `{:continue!, process_uuid}` - continue execution of an existing process manager (strict).\n- `{:stop, process_uuid}` - stop an existing process manager, shutdown its\n  process, and delete its persisted state.\n- `false` - ignore the event.\n\nYou can return a list of process identifiers when a single domain event is to\nbe handled by multiple process instances.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:interested?/2"},{"type":"callback","title":"Strict process routing - Commanded.ProcessManagers.ProcessManager.interested?/2","doc":"Using strict routing, with `:start!` or `:continue`, enforces the following\nvalidation checks:\n\n- `{:start!, process_uuid}` - validate process does not already exist.\n- `{:continue!, process_uuid}` - validate process already exists.\n\nIf the check fails an error will be passed to the `error/3` callback function:\n\n- `{:error, {:start!, :process_already_started}}`\n- `{:error, {:continue!, :process_not_started}}`\n\nThe `error/3` function can choose to `:stop` the process or `:skip` the\nproblematic event.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:interested?/2-strict-process-routing"},{"type":"function","title":"Commanded.ProcessManagers.ProcessManager.parse_config!/2","doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#parse_config!/2"},{"type":"type","title":"Commanded.ProcessManagers.ProcessManager.command/0","doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#t:command/0"},{"type":"type","title":"Commanded.ProcessManagers.ProcessManager.consistency/0","doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#t:consistency/0"},{"type":"type","title":"Commanded.ProcessManagers.ProcessManager.domain_event/0","doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#t:domain_event/0"},{"type":"type","title":"Commanded.ProcessManagers.ProcessManager.enriched_metadata/0","doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#t:enriched_metadata/0"},{"type":"type","title":"Commanded.ProcessManagers.ProcessManager.process_manager/0","doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#t:process_manager/0"},{"type":"type","title":"Commanded.ProcessManagers.ProcessManager.process_uuid/0","doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#t:process_uuid/0"},{"type":"module","title":"Commanded.EventStore","doc":"Use the event store configured for a Commanded application.","ref":"Commanded.EventStore.html"},{"type":"module","title":"Telemetry Events - Commanded.EventStore","doc":"Adds telemetry events for the following functions. Events are emitted in the form\n\n`[:commanded, :event_store, event]` with their spannable postfixes (`start`, `stop`, `exception`)\n\n  * ack_event/3\n  * adapter/2\n  * append_to_stream/4\n  * delete_snapshot/2\n  * delete_subscription/3\n  * read_snapshot/2\n  * record_snapshot/2\n  * stream_forward/2\n  * stream_forward/3\n  * stream_forward/4\n  * subscribe/2\n  * subscribe_to/5\n  * subscribe_to/6\n  * unsubscribe/2","ref":"Commanded.EventStore.html#module-telemetry-events"},{"type":"function","title":"Commanded.EventStore.ack_event/3","doc":"Acknowledge receipt and successful processing of the given event received from\na subscription to an event stream.","ref":"Commanded.EventStore.html#ack_event/3"},{"type":"function","title":"Commanded.EventStore.adapter/2","doc":"Get the configured event store adapter for the given application.","ref":"Commanded.EventStore.html#adapter/2"},{"type":"function","title":"Commanded.EventStore.append_to_stream/5","doc":"Append one or more events to a stream atomically.","ref":"Commanded.EventStore.html#append_to_stream/5"},{"type":"function","title":"Commanded.EventStore.delete_snapshot/2","doc":"Delete a previously recorded snapshot for a given source","ref":"Commanded.EventStore.html#delete_snapshot/2"},{"type":"function","title":"Commanded.EventStore.delete_subscription/3","doc":"Delete an existing subscription.","ref":"Commanded.EventStore.html#delete_subscription/3"},{"type":"function","title":"Example - Commanded.EventStore.delete_subscription/3","doc":":ok = Commanded.EventStore.delete_subscription(MyApp, :all, \"Example\")","ref":"Commanded.EventStore.html#delete_subscription/3-example"},{"type":"function","title":"Commanded.EventStore.read_snapshot/2","doc":"Read a snapshot, if available, for a given source.","ref":"Commanded.EventStore.html#read_snapshot/2"},{"type":"function","title":"Commanded.EventStore.record_snapshot/2","doc":"Record a snapshot of the data and metadata for a given source","ref":"Commanded.EventStore.html#record_snapshot/2"},{"type":"function","title":"Commanded.EventStore.stream_forward/4","doc":"Streams events from the given stream, in the order in which they were originally written.","ref":"Commanded.EventStore.html#stream_forward/4"},{"type":"function","title":"Commanded.EventStore.subscribe/2","doc":"Create a transient subscription to a single event stream.\n\nThe event store will publish any events appended to the given stream to the\n`subscriber` process as an `{:events, events}` message.\n\nThe subscriber does not need to acknowledge receipt of the events.","ref":"Commanded.EventStore.html#subscribe/2"},{"type":"function","title":"Commanded.EventStore.subscribe_to/6","doc":"Create a persistent subscription to an event stream.\n\nTo subscribe to all events appended to any stream use `:all` as the stream\nwhen subscribing.\n\nThe event store will remember the subscribers last acknowledged event.\nRestarting the named subscription will resume from the next event following\nthe last seen.\n\nOnce subscribed, the subscriber process should be sent a\n`{:subscribed, subscription}` message to allow it to defer initialisation\nuntil the subscription has started.\n\nThe subscriber process will be sent all events persisted to the stream. It\nwill receive a `{:events, events}` message for each batch of events persisted\nfor a single aggregate.\n\nThe subscriber must ack each received, and successfully processed event, using\n`Commanded.EventStore.ack_event/3`.","ref":"Commanded.EventStore.html#subscribe_to/6"},{"type":"function","title":"Examples - Commanded.EventStore.subscribe_to/6","doc":"Subscribe to all streams:\n\n    {:ok, subscription} =\n      Commanded.EventStore.subscribe_to(MyApp, :all, \"Example\", self(), :current)\n\nSubscribe to a single stream:\n\n    {:ok, subscription} =\n      Commanded.EventStore.subscribe_to(MyApp, \"stream1\", \"Example\", self(), :origin)","ref":"Commanded.EventStore.html#subscribe_to/6-examples"},{"type":"function","title":"Commanded.EventStore.unsubscribe/2","doc":"Unsubscribe an existing subscriber from event notifications.\n\nThis will not delete the subscription.","ref":"Commanded.EventStore.html#unsubscribe/2"},{"type":"function","title":"Example - Commanded.EventStore.unsubscribe/2","doc":":ok = Commanded.EventStore.unsubscribe(MyApp, subscription)","ref":"Commanded.EventStore.html#unsubscribe/2-example"},{"type":"type","title":"Commanded.EventStore.application/0","doc":"","ref":"Commanded.EventStore.html#t:application/0"},{"type":"type","title":"Commanded.EventStore.config/0","doc":"","ref":"Commanded.EventStore.html#t:config/0"},{"type":"behaviour","title":"Commanded.EventStore.Adapter","doc":"Defines the behaviour to be implemented by an event store adapter to be used by Commanded.","ref":"Commanded.EventStore.Adapter.html"},{"type":"callback","title":"Commanded.EventStore.Adapter.ack_event/3","doc":"Acknowledge receipt and successful processing of the given event received from\na subscription to an event stream.","ref":"Commanded.EventStore.Adapter.html#c:ack_event/3"},{"type":"callback","title":"Commanded.EventStore.Adapter.append_to_stream/5","doc":"Append one or more events to a stream atomically.","ref":"Commanded.EventStore.Adapter.html#c:append_to_stream/5"},{"type":"callback","title":"Commanded.EventStore.Adapter.child_spec/2","doc":"Return a child spec defining all processes required by the event store.","ref":"Commanded.EventStore.Adapter.html#c:child_spec/2"},{"type":"callback","title":"Commanded.EventStore.Adapter.delete_snapshot/2","doc":"Delete a previously recorded snapshot for a given source","ref":"Commanded.EventStore.Adapter.html#c:delete_snapshot/2"},{"type":"callback","title":"Commanded.EventStore.Adapter.delete_subscription/3","doc":"Delete an existing subscription.","ref":"Commanded.EventStore.Adapter.html#c:delete_subscription/3"},{"type":"callback","title":"Commanded.EventStore.Adapter.read_snapshot/2","doc":"Read a snapshot, if available, for a given source.","ref":"Commanded.EventStore.Adapter.html#c:read_snapshot/2"},{"type":"callback","title":"Commanded.EventStore.Adapter.record_snapshot/2","doc":"Record a snapshot of the data and metadata for a given source","ref":"Commanded.EventStore.Adapter.html#c:record_snapshot/2"},{"type":"callback","title":"Commanded.EventStore.Adapter.stream_forward/4","doc":"Streams events from the given stream, in the order in which they were\noriginally written.","ref":"Commanded.EventStore.Adapter.html#c:stream_forward/4"},{"type":"callback","title":"Commanded.EventStore.Adapter.subscribe/2","doc":"Create a transient subscription to a single event stream.\n\nThe event store will publish any events appended to the given stream to the\n`subscriber` process as an `{:events, events}` message.\n\nThe subscriber does not need to acknowledge receipt of the events.","ref":"Commanded.EventStore.Adapter.html#c:subscribe/2"},{"type":"callback","title":"Commanded.EventStore.Adapter.subscribe_to/6","doc":"Create a persistent subscription to an event stream.","ref":"Commanded.EventStore.Adapter.html#c:subscribe_to/6"},{"type":"callback","title":"Commanded.EventStore.Adapter.unsubscribe/2","doc":"Unsubscribe an existing subscriber from event notifications.\n\nThis should not delete the subscription.","ref":"Commanded.EventStore.Adapter.html#c:unsubscribe/2"},{"type":"type","title":"Commanded.EventStore.Adapter.adapter_meta/0","doc":"","ref":"Commanded.EventStore.Adapter.html#t:adapter_meta/0"},{"type":"type","title":"Commanded.EventStore.Adapter.application/0","doc":"","ref":"Commanded.EventStore.Adapter.html#t:application/0"},{"type":"type","title":"Commanded.EventStore.Adapter.config/0","doc":"","ref":"Commanded.EventStore.Adapter.html#t:config/0"},{"type":"type","title":"Commanded.EventStore.Adapter.error/0","doc":"","ref":"Commanded.EventStore.Adapter.html#t:error/0"},{"type":"type","title":"Commanded.EventStore.Adapter.expected_version/0","doc":"","ref":"Commanded.EventStore.Adapter.html#t:expected_version/0"},{"type":"type","title":"Commanded.EventStore.Adapter.source_uuid/0","doc":"","ref":"Commanded.EventStore.Adapter.html#t:source_uuid/0"},{"type":"type","title":"Commanded.EventStore.Adapter.start_from/0","doc":"","ref":"Commanded.EventStore.Adapter.html#t:start_from/0"},{"type":"type","title":"Commanded.EventStore.Adapter.stream_uuid/0","doc":"","ref":"Commanded.EventStore.Adapter.html#t:stream_uuid/0"},{"type":"type","title":"Commanded.EventStore.Adapter.subscriber/0","doc":"","ref":"Commanded.EventStore.Adapter.html#t:subscriber/0"},{"type":"type","title":"Commanded.EventStore.Adapter.subscription/0","doc":"","ref":"Commanded.EventStore.Adapter.html#t:subscription/0"},{"type":"type","title":"Commanded.EventStore.Adapter.subscription_name/0","doc":"","ref":"Commanded.EventStore.Adapter.html#t:subscription_name/0"},{"type":"module","title":"Commanded.EventStore.Adapters.InMemory","doc":"An in-memory event store adapter implemented as a `GenServer` process which\nstores events in memory only.\n\nThis is only designed for testing purposes.","ref":"Commanded.EventStore.Adapters.InMemory.html"},{"type":"function","title":"Commanded.EventStore.Adapters.InMemory.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Commanded.EventStore.Adapters.InMemory.html#child_spec/1"},{"type":"function","title":"Commanded.EventStore.Adapters.InMemory.reset!/2","doc":"","ref":"Commanded.EventStore.Adapters.InMemory.html#reset!/2"},{"type":"function","title":"Commanded.EventStore.Adapters.InMemory.start_link/1","doc":"","ref":"Commanded.EventStore.Adapters.InMemory.html#start_link/1"},{"type":"module","title":"Commanded.EventStore.EventData","doc":"EventData contains the data for a single event before being persisted to\nstorage.","ref":"Commanded.EventStore.EventData.html"},{"type":"type","title":"Commanded.EventStore.EventData.t/0","doc":"","ref":"Commanded.EventStore.EventData.html#t:t/0"},{"type":"type","title":"Commanded.EventStore.EventData.uuid/0","doc":"","ref":"Commanded.EventStore.EventData.html#t:uuid/0"},{"type":"module","title":"Commanded.EventStore.RecordedEvent","doc":"Contains the persisted stream identity, type, data, and metadata for a single event.\n\nEvents are immutable once recorded.","ref":"Commanded.EventStore.RecordedEvent.html"},{"type":"module","title":"Recorded event fields - Commanded.EventStore.RecordedEvent","doc":"- `event_id` - a globally unique UUID to identify the event.\n\n  - `event_number` - a globally unique, monotonically incrementing and gapless\n    integer used to order the event amongst all events.\n\n  - `stream_id` - the stream identity for the event.\n\n  - `stream_version` - the version of the stream for the event.\n\n  - `causation_id` - an optional UUID identifier used to identify which\n    message you are responding to.\n\n  - `correlation_id` - an optional UUID identifier used to correlate related\n    messages.\n\n  - `data` - the event data deserialized into a struct.\n\n  - `metadata` - a string keyed map of metadata associated with the event.\n\n  - `created_at` - the datetime, in UTC, indicating when the event was\n    created.","ref":"Commanded.EventStore.RecordedEvent.html#module-recorded-event-fields"},{"type":"function","title":"Commanded.EventStore.RecordedEvent.enrich_metadata/2","doc":"Enrich the event's metadata with fields from the `RecordedEvent` struct and\nany additional metadata passed as an option.","ref":"Commanded.EventStore.RecordedEvent.html#enrich_metadata/2"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.causation_id/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:causation_id/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.correlation_id/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:correlation_id/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.created_at/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:created_at/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.data/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:data/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.domain_event/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:domain_event/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.enriched_metadata/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:enriched_metadata/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.event_id/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:event_id/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.event_number/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:event_number/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.event_type/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:event_type/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.metadata/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:metadata/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.stream_id/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:stream_id/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.stream_version/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:stream_version/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.t/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:t/0"},{"type":"type","title":"Commanded.EventStore.RecordedEvent.uuid/0","doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:uuid/0"},{"type":"module","title":"Commanded.EventStore.SnapshotData","doc":"Snapshot data","ref":"Commanded.EventStore.SnapshotData.html"},{"type":"type","title":"Commanded.EventStore.SnapshotData.t/0","doc":"","ref":"Commanded.EventStore.SnapshotData.html#t:t/0"},{"type":"behaviour","title":"Commanded.EventStore.TypeProvider","doc":"Specification to convert between an Elixir struct and a corresponding string type.","ref":"Commanded.EventStore.TypeProvider.html"},{"type":"callback","title":"Commanded.EventStore.TypeProvider.to_string/1","doc":"Type of the given Elixir struct as a string","ref":"Commanded.EventStore.TypeProvider.html#c:to_string/1"},{"type":"callback","title":"Commanded.EventStore.TypeProvider.to_struct/1","doc":"Convert the given type string to an Elixir struct","ref":"Commanded.EventStore.TypeProvider.html#c:to_struct/1"},{"type":"function","title":"Commanded.EventStore.TypeProvider.type_provider/0","doc":"Get the configured type provider","ref":"Commanded.EventStore.TypeProvider.html#type_provider/0"},{"type":"type","title":"Commanded.EventStore.TypeProvider.t/0","doc":"","ref":"Commanded.EventStore.TypeProvider.html#t:t/0"},{"type":"type","title":"Commanded.EventStore.TypeProvider.type/0","doc":"","ref":"Commanded.EventStore.TypeProvider.html#t:type/0"},{"type":"module","title":"Commanded.PubSub","doc":"Use the pubsub configured for a Commanded application.","ref":"Commanded.PubSub.html"},{"type":"function","title":"Commanded.PubSub.adapter/2","doc":"Get the configured pub/sub adapter.\n\nDefaults to a local pub/sub, restricted to running on a single node.","ref":"Commanded.PubSub.html#adapter/2"},{"type":"function","title":"Commanded.PubSub.broadcast/3","doc":"Broadcasts message on given topic.\n\n  * `topic` - The topic to broadcast to, ie: `\"users:123\"`\n  * `message` - The payload of the broadcast","ref":"Commanded.PubSub.html#broadcast/3"},{"type":"function","title":"Commanded.PubSub.subscribe/2","doc":"Subscribes the caller to the PubSub adapter's topic.","ref":"Commanded.PubSub.html#subscribe/2"},{"type":"type","title":"Commanded.PubSub.application/0","doc":"","ref":"Commanded.PubSub.html#t:application/0"},{"type":"type","title":"Commanded.PubSub.config/0","doc":"","ref":"Commanded.PubSub.html#t:config/0"},{"type":"behaviour","title":"Commanded.PubSub.Adapter","doc":"Pub/sub behaviour for use by Commanded to subscribe to and broadcast messages.","ref":"Commanded.PubSub.Adapter.html"},{"type":"callback","title":"Commanded.PubSub.Adapter.broadcast/3","doc":"Broadcasts message on given topic.\n\n  * `topic` - The topic to broadcast to, ie: `\"users:123\"`\n  * `message` - The payload of the broadcast","ref":"Commanded.PubSub.Adapter.html#c:broadcast/3"},{"type":"callback","title":"Commanded.PubSub.Adapter.child_spec/2","doc":"Return an optional supervisor spec for pub/sub.","ref":"Commanded.PubSub.Adapter.html#c:child_spec/2"},{"type":"callback","title":"Commanded.PubSub.Adapter.list/2","doc":"List tracked PIDs for a given topic.","ref":"Commanded.PubSub.Adapter.html#c:list/2"},{"type":"callback","title":"Commanded.PubSub.Adapter.subscribe/2","doc":"Subscribes the caller to the PubSub adapter's topic.","ref":"Commanded.PubSub.Adapter.html#c:subscribe/2"},{"type":"callback","title":"Commanded.PubSub.Adapter.track/3","doc":"Track the current process under the given `topic`, uniquely identified by\n`key`.","ref":"Commanded.PubSub.Adapter.html#c:track/3"},{"type":"type","title":"Commanded.PubSub.Adapter.adapter_meta/0","doc":"","ref":"Commanded.PubSub.Adapter.html#t:adapter_meta/0"},{"type":"type","title":"Commanded.PubSub.Adapter.application/0","doc":"","ref":"Commanded.PubSub.Adapter.html#t:application/0"},{"type":"module","title":"Commanded.PubSub.LocalPubSub","doc":"Local pub/sub adapter, restricted to a single node, using Elixir's `Registry`.\n\nYou can configure this adapter in your environment config file:\n\n    # `config/config.exs`\n    config :my_app, MyApp.Application, pubsub: :local\n\nThis adapter will be used by default when none is specified in config.","ref":"Commanded.PubSub.LocalPubSub.html"},{"type":"function","title":"Commanded.PubSub.LocalPubSub.broadcast/3","doc":"Broadcasts message on given topic.","ref":"Commanded.PubSub.LocalPubSub.html#broadcast/3"},{"type":"function","title":"Commanded.PubSub.LocalPubSub.child_spec/2","doc":"Start a `Registry` for local pub/sub.","ref":"Commanded.PubSub.LocalPubSub.html#child_spec/2"},{"type":"function","title":"Commanded.PubSub.LocalPubSub.list/2","doc":"List tracked terms and associated PIDs for a given topic.","ref":"Commanded.PubSub.LocalPubSub.html#list/2"},{"type":"function","title":"Commanded.PubSub.LocalPubSub.subscribe/2","doc":"Subscribes the caller to the topic.","ref":"Commanded.PubSub.LocalPubSub.html#subscribe/2"},{"type":"function","title":"Commanded.PubSub.LocalPubSub.track/3","doc":"Track the current process under the given `topic`, uniquely identified by\n`key`.","ref":"Commanded.PubSub.LocalPubSub.html#track/3"},{"type":"module","title":"Commanded.PubSub.PhoenixPubSub","doc":"Pub/sub adapter using Phoenix's distributed pub/sub and presence platform.\n\nTo use Phoenix pub/sub you must add it as a dependency in your project's\n`mix.exs` file:\n\n    defp deps do\n      [\n        {:phoenix_pubsub, \"~> 1.0\"}\n      ]\n    end\n\nFetch mix deps and configure the pubsub settings in your environment config\nfile:\n\n    # `config/config.exs`\n    config :my_app, MyApp.Application,\n      pubsub: [\n        phoenix_pubsub: [\n          adapter: Phoenix.PubSub.PG2,\n          pool_size: 1\n        ]\n      ]\n\nSpecify the Phoenix pub/sub adapter you wish to use from:\n\n  * `Phoenix.PubSub.PG2` - uses Distributed Elixir, directly exchanging\n    notifications between servers\n\n  * `Phoenix.PubSub.Redis` - uses Redis to exchange data between servers","ref":"Commanded.PubSub.PhoenixPubSub.html"},{"type":"function","title":"Commanded.PubSub.PhoenixPubSub.broadcast/3","doc":"Broadcasts message on given topic.","ref":"Commanded.PubSub.PhoenixPubSub.html#broadcast/3"},{"type":"function","title":"Commanded.PubSub.PhoenixPubSub.child_spec/2","doc":"Start the configured Phoenix pub/sub adapter and a presence tracker.","ref":"Commanded.PubSub.PhoenixPubSub.html#child_spec/2"},{"type":"function","title":"Commanded.PubSub.PhoenixPubSub.list/2","doc":"List tracked terms and associated PIDs for a given topic.","ref":"Commanded.PubSub.PhoenixPubSub.html#list/2"},{"type":"function","title":"Commanded.PubSub.PhoenixPubSub.subscribe/2","doc":"Subscribes the caller to the topic.","ref":"Commanded.PubSub.PhoenixPubSub.html#subscribe/2"},{"type":"function","title":"Commanded.PubSub.PhoenixPubSub.track/3","doc":"Track the current process under the given `topic`, uniquely identified by\n`key`.","ref":"Commanded.PubSub.PhoenixPubSub.html#track/3"},{"type":"module","title":"Commanded.Registration","doc":"Use the process registry configured for a Commanded application.","ref":"Commanded.Registration.html"},{"type":"macro","title":"Commanded.Registration.__before_compile__/1","doc":"Allow a registry adapter to handle the standard `GenServer` callback\nfunctions.","ref":"Commanded.Registration.html#__before_compile__/1"},{"type":"macro","title":"Commanded.Registration.__using__/1","doc":"Use the `Commanded.Registration` module to import the registry adapter and\nvia tuple functions.","ref":"Commanded.Registration.html#__using__/1"},{"type":"function","title":"Commanded.Registration.adapter/2","doc":"Get the configured process registry.\n\nDefaults to a local registry, restricted to running on a single node.","ref":"Commanded.Registration.html#adapter/2"},{"type":"type","title":"Commanded.Registration.application/0","doc":"","ref":"Commanded.Registration.html#t:application/0"},{"type":"type","title":"Commanded.Registration.config/0","doc":"","ref":"Commanded.Registration.html#t:config/0"},{"type":"behaviour","title":"Commanded.Registration.Adapter","doc":"Defines a behaviour for a process registry to be used by Commanded.\n\nBy default, Commanded will use a local process registry, defined in\n`Commanded.Registration.LocalRegistry`, that uses Elixir's `Registry` module\nfor local process registration. This limits Commanded to only run on a single\nnode. However the `Commanded.Registration` behaviour can be implemented by a\nlibrary to provide distributed process registration to support running on a\ncluster of nodes.","ref":"Commanded.Registration.Adapter.html"},{"type":"callback","title":"Commanded.Registration.Adapter.child_spec/2","doc":"Return an optional supervisor spec for the registry","ref":"Commanded.Registration.Adapter.html#c:child_spec/2"},{"type":"callback","title":"Commanded.Registration.Adapter.start_child/4","doc":"Starts a uniquely named child process of a supervisor using the given module\nand args.\n\nRegisters the pid with the given name.","ref":"Commanded.Registration.Adapter.html#c:start_child/4"},{"type":"callback","title":"Commanded.Registration.Adapter.start_link/5","doc":"Starts a uniquely named `GenServer` process for the given module and args.\n\nRegisters the pid with the given name.","ref":"Commanded.Registration.Adapter.html#c:start_link/5"},{"type":"callback","title":"Commanded.Registration.Adapter.supervisor_child_spec/3","doc":"Use to start a supervisor.","ref":"Commanded.Registration.Adapter.html#c:supervisor_child_spec/3"},{"type":"callback","title":"Commanded.Registration.Adapter.via_tuple/2","doc":"Return a `:via` tuple to route a message to a process by its registered name","ref":"Commanded.Registration.Adapter.html#c:via_tuple/2"},{"type":"callback","title":"Commanded.Registration.Adapter.whereis_name/2","doc":"Get the pid of a registered name.\n\nReturns `:undefined` if the name is unregistered.","ref":"Commanded.Registration.Adapter.html#c:whereis_name/2"},{"type":"type","title":"Commanded.Registration.Adapter.adapter_meta/0","doc":"","ref":"Commanded.Registration.Adapter.html#t:adapter_meta/0"},{"type":"type","title":"Commanded.Registration.Adapter.application/0","doc":"","ref":"Commanded.Registration.Adapter.html#t:application/0"},{"type":"type","title":"Commanded.Registration.Adapter.config/0","doc":"","ref":"Commanded.Registration.Adapter.html#t:config/0"},{"type":"type","title":"Commanded.Registration.Adapter.start_child_arg/0","doc":"","ref":"Commanded.Registration.Adapter.html#t:start_child_arg/0"},{"type":"module","title":"Commanded.Registration.GlobalRegistry","doc":"Distributed process registration using Erlangs `:global` registry[1].\n\n[1] http://erlang.org/doc/man/global.html","ref":"Commanded.Registration.GlobalRegistry.html"},{"type":"function","title":"Commanded.Registration.GlobalRegistry.child_spec/2","doc":"Return an optional supervisor spec for the registry","ref":"Commanded.Registration.GlobalRegistry.html#child_spec/2"},{"type":"function","title":"Commanded.Registration.GlobalRegistry.start_child/4","doc":"Starts a uniquely named child process of a supervisor using the given module\nand args.\n\nRegisters the pid with the given name.","ref":"Commanded.Registration.GlobalRegistry.html#start_child/4"},{"type":"function","title":"Commanded.Registration.GlobalRegistry.start_link/5","doc":"Starts a uniquely named `GenServer` process for the given module and args.\n\nRegisters the pid with the given name.","ref":"Commanded.Registration.GlobalRegistry.html#start_link/5"},{"type":"function","title":"Commanded.Registration.GlobalRegistry.supervisor_child_spec/3","doc":"Starts a supervisor.","ref":"Commanded.Registration.GlobalRegistry.html#supervisor_child_spec/3"},{"type":"function","title":"Commanded.Registration.GlobalRegistry.via_tuple/2","doc":"Return a `:via` tuple to route a message to a process by its registered name","ref":"Commanded.Registration.GlobalRegistry.html#via_tuple/2"},{"type":"function","title":"Commanded.Registration.GlobalRegistry.whereis_name/2","doc":"Get the pid of a registered name.\n\nReturns `:undefined` if the name is unregistered.","ref":"Commanded.Registration.GlobalRegistry.html#whereis_name/2"},{"type":"module","title":"Commanded.Registration.LocalRegistry","doc":"Local process registration, restricted to a single node, using Elixir's\n`Registry` module.","ref":"Commanded.Registration.LocalRegistry.html"},{"type":"function","title":"Commanded.Registration.LocalRegistry.child_spec/2","doc":"Return a supervisor spec for the registry.","ref":"Commanded.Registration.LocalRegistry.html#child_spec/2"},{"type":"function","title":"Commanded.Registration.LocalRegistry.start_child/4","doc":"Starts a uniquely named child process of a supervisor using the given module\nand args.\n\nRegisters the pid with the given name.","ref":"Commanded.Registration.LocalRegistry.html#start_child/4"},{"type":"function","title":"Commanded.Registration.LocalRegistry.start_link/5","doc":"Starts a uniquely named `GenServer` process for the given module and args.\n\nRegisters the pid with the given name.","ref":"Commanded.Registration.LocalRegistry.html#start_link/5"},{"type":"function","title":"Commanded.Registration.LocalRegistry.supervisor_child_spec/3","doc":"Starts a supervisor.","ref":"Commanded.Registration.LocalRegistry.html#supervisor_child_spec/3"},{"type":"function","title":"Commanded.Registration.LocalRegistry.via_tuple/2","doc":"Return a `:via` tuple to route a message to a process by its registered name.","ref":"Commanded.Registration.LocalRegistry.html#via_tuple/2"},{"type":"function","title":"Commanded.Registration.LocalRegistry.whereis_name/2","doc":"Get the pid of a registered name.\n\nReturns `:undefined` if the name is unregistered.","ref":"Commanded.Registration.LocalRegistry.html#whereis_name/2"},{"type":"protocol","title":"Commanded.Serialization.JsonDecoder","doc":"","ref":"Commanded.Serialization.JsonDecoder.html"},{"type":"function","title":"Commanded.Serialization.JsonDecoder.decode/1","doc":"Protocol to allow additional decoding of a value that has been deserialized\nusing the `Commanded.Serialization.JsonSerializer`.\n\nThe protocol is optional. The default behaviour is to to return the value if\nan explicit protocol is not defined.","ref":"Commanded.Serialization.JsonDecoder.html#decode/1"},{"type":"type","title":"Commanded.Serialization.JsonDecoder.t/0","doc":"All the types that implement this protocol.","ref":"Commanded.Serialization.JsonDecoder.html#t:t/0"},{"type":"module","title":"Commanded.Serialization.JsonSerializer","doc":"A serializer that uses the JSON format and Jason library.","ref":"Commanded.Serialization.JsonSerializer.html"},{"type":"function","title":"Commanded.Serialization.JsonSerializer.deserialize/2","doc":"Deserialize given JSON binary data to the expected type.","ref":"Commanded.Serialization.JsonSerializer.html#deserialize/2"},{"type":"function","title":"Commanded.Serialization.JsonSerializer.serialize/1","doc":"Serialize given term to JSON binary data.","ref":"Commanded.Serialization.JsonSerializer.html#serialize/1"},{"type":"module","title":"Commanded.Serialization.ModuleNameTypeProvider","doc":"A type provider that uses the Elixir module name\n\nExample:\n\n  - %An.Event{} module mapped to \"Elixir.An.Event\".","ref":"Commanded.Serialization.ModuleNameTypeProvider.html"},{"type":"function","title":"Commanded.Serialization.ModuleNameTypeProvider.to_string/1","doc":"","ref":"Commanded.Serialization.ModuleNameTypeProvider.html#to_string/1"},{"type":"function","title":"Commanded.Serialization.ModuleNameTypeProvider.to_struct/1","doc":"","ref":"Commanded.Serialization.ModuleNameTypeProvider.html#to_struct/1"},{"type":"task","title":"mix commanded.reset","doc":"Reset an event handler.","ref":"Mix.Tasks.Commanded.Reset.html"},{"type":"task","title":"Usage - mix commanded.reset","doc":"mix commanded.reset --app   --handler","ref":"Mix.Tasks.Commanded.Reset.html#module-usage"},{"type":"task","title":"Examples - mix commanded.reset","doc":"mix commanded.reset --app MyApp --handler MyHandler\n    mix commanded.reset -a MyApp -h MyHandler","ref":"Mix.Tasks.Commanded.Reset.html#module-examples"},{"type":"task","title":"Command line options - mix commanded.reset","doc":"* `-a`, `--app` - the Commanded application\n  * `-h`, `--handler` - the name of the event handler to reset\n  * `-q`, `--quiet` - do not log output","ref":"Mix.Tasks.Commanded.Reset.html#module-command-line-options"},{"type":"behaviour","title":"Commanded.Middleware","doc":"Middleware provides an extension point to add functions that you want to be\ncalled for every command the router dispatches.\n\nExamples include command validation, authorization, and logging.\n\nImplement the `Commanded.Middleware` behaviour in your module and define the\n`c:before_dispatch/1`, `c:after_dispatch/1`, and `c:after_failure/1` callback\nfunctions.","ref":"Commanded.Middleware.html"},{"type":"behaviour","title":"Example middleware - Commanded.Middleware","doc":"defmodule NoOpMiddleware do\n      @behaviour Commanded.Middleware\n\n      alias Commanded.Middleware.Pipeline\n      import Pipeline\n\n      def before_dispatch(%Pipeline{command: command} = pipeline) do\n        pipeline\n      end\n\n      def after_dispatch(%Pipeline{command: command} = pipeline) do\n        pipeline\n      end\n\n      def after_failure(%Pipeline{command: command} = pipeline) do\n        pipeline\n      end\n    end\n\nImport the `Commanded.Middleware.Pipeline` module to access convenience\nfunctions.\n\n  * `assign/3` - puts a key and value into the `assigns` map\n  * `halt/1` - stops execution of further middleware downstream and prevents\n    dispatch of the command when used in a `before_dispatch` callback","ref":"Commanded.Middleware.html#module-example-middleware"},{"type":"callback","title":"Commanded.Middleware.after_dispatch/1","doc":"","ref":"Commanded.Middleware.html#c:after_dispatch/1"},{"type":"callback","title":"Commanded.Middleware.after_failure/1","doc":"","ref":"Commanded.Middleware.html#c:after_failure/1"},{"type":"callback","title":"Commanded.Middleware.before_dispatch/1","doc":"","ref":"Commanded.Middleware.html#c:before_dispatch/1"},{"type":"type","title":"Commanded.Middleware.pipeline/0","doc":"","ref":"Commanded.Middleware.html#t:pipeline/0"},{"type":"module","title":"Commanded.Middleware.ConsistencyGuarantee","doc":"An internal `Commanded.Middleware` that blocks after successful command\ndispatch until the requested dispatch consistency has been met.\n\nOnly applies when the requested consistency is `:strong`. Has no effect for\n`:eventual` consistency.","ref":"Commanded.Middleware.ConsistencyGuarantee.html"},{"type":"function","title":"Commanded.Middleware.ConsistencyGuarantee.after_dispatch/1","doc":"","ref":"Commanded.Middleware.ConsistencyGuarantee.html#after_dispatch/1"},{"type":"function","title":"Commanded.Middleware.ConsistencyGuarantee.after_failure/1","doc":"","ref":"Commanded.Middleware.ConsistencyGuarantee.html#after_failure/1"},{"type":"function","title":"Commanded.Middleware.ConsistencyGuarantee.before_dispatch/1","doc":"","ref":"Commanded.Middleware.ConsistencyGuarantee.html#before_dispatch/1"},{"type":"module","title":"Commanded.Middleware.ExtractAggregateIdentity","doc":"An internal `Commanded.Middleware` that extracts the target aggregate's\nidentity from the command.","ref":"Commanded.Middleware.ExtractAggregateIdentity.html"},{"type":"function","title":"Commanded.Middleware.ExtractAggregateIdentity.after_dispatch/1","doc":"","ref":"Commanded.Middleware.ExtractAggregateIdentity.html#after_dispatch/1"},{"type":"function","title":"Commanded.Middleware.ExtractAggregateIdentity.after_failure/1","doc":"","ref":"Commanded.Middleware.ExtractAggregateIdentity.html#after_failure/1"},{"type":"function","title":"Commanded.Middleware.ExtractAggregateIdentity.before_dispatch/1","doc":"","ref":"Commanded.Middleware.ExtractAggregateIdentity.html#before_dispatch/1"},{"type":"module","title":"Commanded.Middleware.Logger","doc":"A `Commanded.Middleware` that logs each stage of the command dispatch using\nthe Elixir `Logger`:\n\n- Before dispatch.\n- After successful dispatch.\n- After failed dispatch.","ref":"Commanded.Middleware.Logger.html"},{"type":"function","title":"Commanded.Middleware.Logger.after_dispatch/1","doc":"","ref":"Commanded.Middleware.Logger.html#after_dispatch/1"},{"type":"function","title":"Commanded.Middleware.Logger.after_failure/1","doc":"","ref":"Commanded.Middleware.Logger.html#after_failure/1"},{"type":"function","title":"Commanded.Middleware.Logger.before_dispatch/1","doc":"","ref":"Commanded.Middleware.Logger.html#before_dispatch/1"},{"type":"module","title":"Commanded.Middleware.Pipeline","doc":"Pipeline is a struct used as an argument in the callback functions of modules\nimplementing the `Commanded.Middleware` behaviour.\n\nThis struct must be returned by each function to be used in the next\nmiddleware based on the configured middleware chain.","ref":"Commanded.Middleware.Pipeline.html"},{"type":"module","title":"Pipeline fields - Commanded.Middleware.Pipeline","doc":"- `application` - the Commanded application.\n  \n  - `assigns` - shared user data as a map.\n\n  - `causation_id` - an optional UUID used to identify the cause of the\n     command being dispatched.\n\n  - `correlation_id` - an optional UUID used to correlate related\n     commands/events together.\n\n  - `command` - command struct being dispatched.\n\n  - `command_uuid` - UUID assigned to the command being dispatched.\n\n  - `consistency` - requested dispatch consistency, either: `:eventual`\n     (default) or `:strong`.\n\n  - `halted` - flag indicating whether the pipeline was halted.\n\n  - `identity` - an atom specifying a field in the command containing the\n     aggregate's identity or a one-arity function that returns an identity\n     from the command being dispatched.\n\n  - `identity_prefix` - an optional prefix to the aggregate's identity. It may\n     be a string (e.g. \"prefix-\") or a zero arity function\n     (e.g. `&MyRouter.identity_prefix/0`).\n\n  - `metadata` - the metadata map to be persisted along with the events.\n\n  - `response` - sets the response to send back to the caller.","ref":"Commanded.Middleware.Pipeline.html#module-pipeline-fields"},{"type":"function","title":"Commanded.Middleware.Pipeline.assign/3","doc":"Puts the `key` with value equal to `value` into `assigns` map.","ref":"Commanded.Middleware.Pipeline.html#assign/3"},{"type":"function","title":"Commanded.Middleware.Pipeline.assign_metadata/3","doc":"Puts the `key` with value equal to `value` into `metadata` map.\n\nNote: Use of atom keys in metadata is deprecated in favour of binary strings.","ref":"Commanded.Middleware.Pipeline.html#assign_metadata/3"},{"type":"function","title":"Commanded.Middleware.Pipeline.chain/3","doc":"Executes the middleware chain.","ref":"Commanded.Middleware.Pipeline.html#chain/3"},{"type":"function","title":"Commanded.Middleware.Pipeline.halt/1","doc":"Halts the pipeline by preventing further middleware downstream from being invoked.\n\nPrevents dispatch of the command if `halt` occurs in a `before_dispatch` callback.","ref":"Commanded.Middleware.Pipeline.html#halt/1"},{"type":"function","title":"Commanded.Middleware.Pipeline.halted?/1","doc":"Has the pipeline been halted?","ref":"Commanded.Middleware.Pipeline.html#halted?/1"},{"type":"function","title":"Commanded.Middleware.Pipeline.respond/2","doc":"Sets the response to be returned to the dispatch caller, unless already set.","ref":"Commanded.Middleware.Pipeline.html#respond/2"},{"type":"function","title":"Commanded.Middleware.Pipeline.response/1","doc":"Extract the response from the pipeline","ref":"Commanded.Middleware.Pipeline.html#response/1"},{"type":"module","title":"Commanded.Assertions.EventAssertions","doc":"Provides test assertion and wait for event functions to help test applications\nbuilt using Commanded.\n\nThe default assert and refute receive timeouts are one second.\n\nYou can override the default timeout in config (e.g. `config/test.exs`):\n\n    config :commanded,\n      assert_receive_event_timeout: 1_000,\n      refute_receive_event_timeout: 1_000","ref":"Commanded.Assertions.EventAssertions.html"},{"type":"function","title":"Commanded.Assertions.EventAssertions.assert_correlated/5","doc":"Assert that events matching their respective predicates have a matching\ncorrelation id.\n\nUseful when there is a chain of events that is connected through event handlers.","ref":"Commanded.Assertions.EventAssertions.html#assert_correlated/5"},{"type":"function","title":"Example - Commanded.Assertions.EventAssertions.assert_correlated/5","doc":"assert_correlated(\n      BankApp,\n      BankAccountOpened, fn opened -> opened.id == 1 end,\n      InitialAmountDeposited, fn deposited -> deposited.id == 2 end\n    )","ref":"Commanded.Assertions.EventAssertions.html#assert_correlated/5-example"},{"type":"function","title":"Commanded.Assertions.EventAssertions.assert_receive_event/3","doc":"Assert that an event of the given event type is published.\nVerify that event using the assertion function.","ref":"Commanded.Assertions.EventAssertions.html#assert_receive_event/3"},{"type":"function","title":"Example - Commanded.Assertions.EventAssertions.assert_receive_event/3","doc":"assert_receive_event(BankApp, BankAccountOpened, fn opened ->\n      assert opened.account_number == \"ACC123\"\n    end)","ref":"Commanded.Assertions.EventAssertions.html#assert_receive_event/3-example"},{"type":"function","title":"Commanded.Assertions.EventAssertions.assert_receive_event/4","doc":"Assert that an event of the given event type, matching the predicate, is\npublished. Verify that event using the assertion function.","ref":"Commanded.Assertions.EventAssertions.html#assert_receive_event/4"},{"type":"function","title":"Example - Commanded.Assertions.EventAssertions.assert_receive_event/4","doc":"assert_receive_event(BankApp, BankAccountOpened,\n      fn opened -> opened.account_number == \"ACC123\" end,\n      fn opened ->\n        assert opened.balance == 1_000\n      end)","ref":"Commanded.Assertions.EventAssertions.html#assert_receive_event/4-example"},{"type":"function","title":"Commanded.Assertions.EventAssertions.refute_receive_event/4","doc":"Refute that an event of the given type has been received.\n\nAn optional predicate may be provided to filter events matching the refuted\ntype.","ref":"Commanded.Assertions.EventAssertions.html#refute_receive_event/4"},{"type":"function","title":"Examples - Commanded.Assertions.EventAssertions.refute_receive_event/4","doc":"Refute that `ExampleEvent` is produced by given anonymous function:\n\n    refute_receive_event(ExampleApp, ExampleEvent, fn ->\n      :ok = MyApp.dispatch(command)\n    end)\n\nRefute that `ExampleEvent` is produced by `some_func/0` function:\n\n    refute_receive_event(ExampleApp, ExampleEvent, &some_func/0)\n\nRefute that `ExampleEvent` matching given `event_matches?/1` predicate function\nis produced by `some_func/0` function:\n\n    refute_receive_event(ExampleApp, ExampleEvent, &some_func/0,\n      predicate: &event_matches?/1\n    )\n\nRefute that `ExampleEvent` matching given anonymous predicate function\nis produced by `some_func/0` function:\n\n    refute_receive_event(ExampleApp, ExampleEvent, &some_func/0,\n      predicate: fn event -> event.value == 1 end\n    )\n\nRefute that `ExampleEvent` produced by `some_func/0` function is published to\na given stream:\n\n    refute_receive_event(ExampleApp, ExampleEvent, &some_func/0,\n      predicate: fn event -> event.value == 1 end,\n      stream: \"foo-1234\"\n    )","ref":"Commanded.Assertions.EventAssertions.html#refute_receive_event/4-examples"},{"type":"function","title":"Commanded.Assertions.EventAssertions.wait_for_event/2","doc":"Wait for an event of the given event type to be published.","ref":"Commanded.Assertions.EventAssertions.html#wait_for_event/2"},{"type":"function","title":"Examples - Commanded.Assertions.EventAssertions.wait_for_event/2","doc":"wait_for_event(BankApp, BankAccountOpened)","ref":"Commanded.Assertions.EventAssertions.html#wait_for_event/2-examples"},{"type":"function","title":"Commanded.Assertions.EventAssertions.wait_for_event/3","doc":"Wait for an event of the given event type, matching the predicate, to be\npublished.","ref":"Commanded.Assertions.EventAssertions.html#wait_for_event/3"},{"type":"function","title":"Examples - Commanded.Assertions.EventAssertions.wait_for_event/3","doc":"wait_for_event(BankApp, BankAccountOpened, fn opened ->\n      opened.account_number == \"ACC123\"\n    end)","ref":"Commanded.Assertions.EventAssertions.html#wait_for_event/3-examples"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v1.4.8 - Changelog","doc":"","ref":"changelog.html#v1-4-8"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* `Commanded.Event.ErrorHandler` now keeps surounding failure context by @drteeth in https://github.com/commanded/commanded/issues/617","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.4.7 - Changelog","doc":"","ref":"changelog.html#v1-4-7"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Application-wide event handler error handling by @drteeth in https://github.com/commanded/commanded/pull/605\n* chore: remove asdf file by @yordis in https://github.com/commanded/commanded/pull/570\n* chore: improve docs about aggregate version by @yordis in https://github.com/commanded/commanded/pull/608\n* Update include_aggregate_version documentation by @TylerPachal in https://github.com/commanded/commanded/pull/609","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fix flakey test by @drteeth in https://github.com/commanded/commanded/pull/599\n* feat: default aggregate lifespan configuration by @yordis in https://github.com/commanded/commanded/pull/548\n* Aggregate.handle_* now properly handles lifespans by @drteeth in https://github.com/commanded/commanded/pull/606\n* Allow registration handle_call/cast callbacks to be called by @drteeth in https://github.com/commanded/commanded/pull/607\n* Update local_cluster by @drteeth in https://github.com/commanded/commanded/pull/610","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.4.6 - Changelog","doc":"","ref":"changelog.html#v1-4-6"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Includes changelog updates\n- Version bump","ref":"changelog.html#enhancements"},{"type":"extras","title":"v1.4.5 - Changelog","doc":"","ref":"changelog.html#v1-4-5"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support OTP 26 and Elixir 1.17 ([#595](https://github.com/commanded/commanded/pull/595)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"v1.4.4 - Changelog","doc":"","ref":"changelog.html#v1-4-4"},{"type":"extras","title":"Enhancements - Changelog","doc":"- feat: put aggregate_state into assigns of the pipeline ([#502](https://github.com/commanded/commanded/pull/502)).\n- Add tag to partition test case ([#525](https://github.com/commanded/commanded/pull/525)).\n- Make before_reset/0 an explicit callback function ([#550](https://github.com/commanded/commanded/pull/550)).\n- New `Event.Handler.after_start/1` callback allows configuration in the handler's process ([#568](https://github.com/commanded/commanded/pull/568)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix EventData typespec ([#495](https://github.com/commanded/commanded/pull/495)).\n- Fix refute_receive_event examples ([#557](https://github.com/commanded/commanded/pull/557)).\n- Fix interested? function doc ([#562](https://github.com/commanded/commanded/pull/562)).\n- Use TypeProvider for process managers snapshot serialization ([#558](https://github.com/commanded/commanded/pull/558)).\n- fix(router.ex): Telemetry is not emitted if dispatch fails for {:error, :unregistered_command} ([#563](in https://github.com/commanded/commanded/pull/563)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.4.3 - Changelog","doc":"","ref":"changelog.html#v1-4-3"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Use `Logger.warning` to fix deprecation warnings ([#542](https://github.com/commanded/commanded/pull/542)).\n- Add typespec to `CompositeRouter.dispatch/2` function ([#536](https://github.com/commanded/commanded/pull/536)).\n- Support `opts` in `Commanded.EventStore.append_to_stream` function ([#528](https://github.com/commanded/commanded/pull/528)).\n- Process manager metadata access ([#514](https://github.com/commanded/commanded/pull/514)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"-  Correct parameter type in `ProcessManager.after_command/1` callback function ([#533](https://github.com/commanded/commanded/pull/533)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.4.2 - Changelog","doc":"- Record aggregate state while processing `Commanded.Aggregate.Multi` ([#507](https://github.com/commanded/commanded/pull/507)).\n- Properly handle EXIT signal in event handler ([#512](https://github.com/commanded/commanded/pull/512)).\n- Separate logging a process managers error ([#513](https://github.com/commanded/commanded/pull/513)).","ref":"changelog.html#v1-4-2"},{"type":"extras","title":"v1.4.1 - Changelog","doc":"","ref":"changelog.html#v1-4-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Retry command execution when the aggregate process is down ([#494](https://github.com/commanded/commanded/pull/494)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Remove duplicate apply function call when receiving missed events published to an aggregate's event stream ([364c877](https://github.com/commanded/commanded/commit/364c877e8f30a18d90544676fb58b94132d50720)).\n- Fix typespec typo in Commanded.Application ([#503](https://github.com/commanded/commanded/pull/503)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.4.0 - Changelog","doc":"","ref":"changelog.html#v1-4-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Allow a process manager to stop after dispatching a command ([#460](https://github.com/commanded/commanded/pull/460)).\n- Replace `use Mix.Config` with `import Config` in config files ([#467](https://github.com/commanded/commanded/pull/467)).\n- Event handler concurrency ([#486](https://github.com/commanded/commanded/pull/486)).\n- Remove `elixir_uuid` dependency ([#493](https://github.com/commanded/commanded/pull/493)).\n- Support and test for OTP 25 ([#489](https://github.com/commanded/commanded/pull/489)).\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v1.3.1 - Changelog","doc":"","ref":"changelog.html#v1-3-1"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Event Handler not calling `init/1` callback function on restart ([#463](https://github.com/commanded/commanded/pull/463)).\n- Call process manager `init/1` function on process restart ([#464](https://github.com/commanded/commanded/pull/464)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.3.0 - Changelog","doc":"","ref":"changelog.html#v1-3-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Allow command identity to be provided during dispatch ([#406](https://github.com/commanded/commanded/pull/406)).\n- Define `Commanded.Telemetry` module to emit consistent telemetry events ([#414](https://github.com/commanded/commanded/pull/414)).\n- Telemetry `[:commanded, :aggregate, :execute]` events ([#407](https://github.com/commanded/commanded/pull/407)).\n- Telemetry `[:commanded, :event, :handle]` events ([#408](https://github.com/commanded/commanded/pull/408)).\n- Telemetry `[:commanded, :process_manager, :handle]` events ([#418](https://github.com/commanded/commanded/pull/418)).\n- Telemetry `[:commanded, :application, :dispatch]` ([#423](https://github.com/commanded/commanded/pull/423)).\n- Graceful shutdown of event handlers ([#431](https://github.com/commanded/commanded/pull/431)).\n- Ensure command dispatch metadata is a map ([#432](https://github.com/commanded/commanded/pull/432)).\n- Retry command execution on node down ([#429](https://github.com/commanded/commanded/pull/429)).\n- Dispatch returning resultant events ([#444](https://github.com/commanded/commanded/pull/444)).\n- Get aggregate state ([#448](https://github.com/commanded/commanded/pull/448)).\n- Support telemetry v1.0 ([#456](https://github.com/commanded/commanded/pull/456)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"v1.2.0 - Changelog","doc":"","ref":"changelog.html#v1-2-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `init/1` callback function to event handlers and process managers ([#393](https://github.com/commanded/commanded/pull/393)).\n- Include `application` and `handler_name` as additional event handler metadata ([#396](https://github.com/commanded/commanded/pull/396)).\n- Allow `GenServer` start options to be provided when starting event handlers and process managers ([#398](https://github.com/commanded/commanded/pull/398)).\n- Add `hibernate_after` option to application config ([#399](https://github.com/commanded/commanded/pull/399)).\n- Add support for providing adapter-specific event store subscription options ([#391](https://github.com/commanded/commanded/pull/391)).\n- Support custom state for event handlers ([#400](https://github.com/commanded/commanded/pull/400)).\n- Allow event handlers and process manager `error` callback to return failure context struct ([#397](https://github.com/commanded/commanded/issues/397)).\n- Allow a before execute function to be defined which is called with the command dispatch execution context and aggregate state before ([#402](https://github.com/commanded/commanded/pull/402)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Allow process manager `error/3` callback to return `:skip` for failed commands, not just failed events ([#362](https://github.com/commanded/commanded/issues/362)).\n\n---","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.1.1 - Changelog","doc":"","ref":"changelog.html#v1-1-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Capture exception on Process Manager `apply/2` and call `error/3` callback functions ([#380](https://github.com/commanded/commanded/pull/380)).\n- Include metadata in upcaster protocol ([#389](https://github.com/commanded/commanded/pull/389)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix `Commanded.Aggregate.Multi.execute/2` calls which return ` Multi` struct ([#385](https://github.com/commanded/commanded/pull/385])).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.1.0 - Changelog","doc":"","ref":"changelog.html#v1-1-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Dynamic Commanded applications ([#324](https://github.com/commanded/commanded/pull/324)).\n- Log and ignore unexpected messages received by event handlers and process manager instances ([#333](https://github.com/commanded/commanded/pull/333))\n- Process manager `identity/0` function ([#334](https://github.com/commanded/commanded/pull/334)).\n- Extend `Commanded.AggregateCase` ExUnit case template to support `Commanded.Aggregate.Multi`.\n- Allow `Commanded.Aggregate.Multi` to return events as `:ok` tagged tuples.\n- Run the formatter in CI ([#341](https://github.com/commanded/commanded/pull/341)).\n- Add stacktraces to EventHandler error logging ([#340](https://github.com/commanded/commanded/pull/340))\n- `refute_receive_event/4` only tests newly created events ([#347](https://github.com/commanded/commanded/pull/347)).\n- Allow Commanded Application name to be set dynamically in middleware ([#352](https://github.com/commanded/commanded/pull/352)).\n- Remove router module compile-time checking ([#363](https://github.com/commanded/commanded/pull/363)).\n- Reduce memory consumption during aggregate state rebuild ([#368](https://github.com/commanded/commanded/pull/368)).\n- Upgrade to `phoenix_pubsub` to 2.0 ([#365](https://github.com/commanded/commanded/pull/365)).\n- Ignore `:not_found` error when resetting InMemory event store ([#354](https://github.com/commanded/commanded/pull/354)).\n- Add `router/1` to `locals_without_parens` in Mix format config ([#351](https://github.com/commanded/commanded/pull/351)).\n- Include stacktrace in event handler and process manager `error` callback functions ([#342](https://github.com/commanded/commanded/pull/342)).\n- Call event handler's `error/3` callback function when `handle/2` function returns an invalid value ([#372](https://github.com/commanded/commanded/pull/372)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fixes the typespec for command dispatch ([#325](https://github.com/commanded/commanded/pull/325)).\n- Process manager stops if `interested?/1` returns an empty list ([#335](https://github.com/commanded/commanded/pull/335)).\n\n---","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.0.1 - Changelog","doc":"","ref":"changelog.html#v1-0-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Global registry using Erlang's `:global` module ([#344](https://github.com/commanded/commanded/pull/344)).\n- Command dispatch return ([#331](https://github.com/commanded/commanded/pull/331)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix distributed subscription registration bug ([#345](https://github.com/commanded/commanded/pull/345)).\n- Retry event handler and process manager subscriptions on error ([#348](https://github.com/commanded/commanded/pull/348)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.0.0 - Changelog","doc":"","ref":"changelog.html#v1-0-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Support multiple Commanded apps ([#298](https://github.com/commanded/commanded/pull/298)).","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Define adapter behaviour modules for event store, pubsub, and registry ([#311](https://github.com/commanded/commanded/pull/311)).\n- Add `AggregateCase` ExUnit case template to support aggregate unit testing ([#315](https://github.com/commanded/commanded/pull/315)).\n- Application config lookup ([#318](https://github.com/commanded/commanded/pull/318)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix process manager exception on start ([#307](https://github.com/commanded/commanded/pull/307)).\n- Fix commanded aggregate race ([#308](https://github.com/commanded/commanded/pull/308)).\n- Fix Dialyzer warnings and include in Travis CI ([#317](https://github.com/commanded/commanded/pull/317)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Upgrading - Changelog","doc":"[Follow the upgrade guide](https://hexdocs.pm/commanded/1.0.0/0.19-1.0.html) to define and use your own Commanded application.\n\n---","ref":"changelog.html#upgrading"},{"type":"extras","title":"v0.19.1 - Changelog","doc":"","ref":"changelog.html#v0-19-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Reset event handler mix task `mix commanded.reset MyApp.Handler` ([#293](https://github.com/commanded/commanded/pull/293)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix regression in `Commanded.Middleware.Logger.delta` ([#295](https://github.com/commanded/commanded/pull/295)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.19.0 - Changelog","doc":"","ref":"changelog.html#v0-19-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Update typespec for `data` and `metadata` fields in `Commanded.EventStore.EventData` struct ([#246](https://github.com/commanded/commanded/pull/246)).\n- Add `include_execution_result` and `aggregate_version` to typespec for router command dispatch ([#262](https://github.com/commanded/commanded/pull/262)).\n- Add `.formatter.exs` to Hex package ([#247](https://github.com/commanded/commanded/pull/247)).\n- Event upcasting ([#263](https://github.com/commanded/commanded/pull/263)).\n- Support `:ok` tagged tuple events from aggregate ([#268](https://github.com/commanded/commanded/pull/268)).\n- Modify `Commanded.Registration.start_child` to pass a child_spec ([#273](https://github.com/commanded/commanded/pull/273)).\n- Add `supervisor_child_spec/2` to `Commanded.Registration` behaviour ([#277](https://github.com/commanded/commanded/pull/277)) used by [Commanded Horde Registry](https://github.com/uberbrodt/commanded_horde_registry).\n- Ensure Commanded can be compiled when optional Jason dependency is not present ([#286](https://github.com/commanded/commanded/pull/286)).\n- Fix Aggregate initialization races ([#287](https://github.com/commanded/commanded/pull/287)).\n- Support `{:system, varname}` format in Phoenix PubSub config ([#291](https://github.com/commanded/commanded/pull/291)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Use `DateTime` instead of `NaiveDateTime` for all datetimes ([#254](https://github.com/commanded/commanded/pull/254)).\n\n    This affects the `created_at` field defined in the `Commanded.EventStore.RecordedEvent`. You will need to migrate from `NaiveDateTime` to `DateTime` if you use this field in your code (such as in an event handler's metadata).\n\n---","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.18.1 - Changelog","doc":"","ref":"changelog.html#v0-18-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Process manager idle process timeout ([#290](https://github.com/commanded/commanded/pull/290)).\n- Register event handler and process manager subscriptions on process start ([#272](https://github.com/commanded/commanded/pull/272)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.18.0 - Changelog","doc":"","ref":"changelog.html#v0-18-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Rename `uuid` dependency to `elixir_uuid` ([#178](https://github.com/commanded/commanded/pull/178)).\n- Allow aggregate identity to be of any type that implements the `String.Chars` protocol ([#166](https://github.com/commanded/commanded/pull/166)).\n- Process manager and event handler error & exception handling ([#192](https://github.com/commanded/commanded/pull/192)).\n- Process manager event handling timeout ([#193](https://github.com/commanded/commanded/pull/193)).\n- Allow event handlers to subscribe to individual streams ([#203](https://github.com/commanded/commanded/pull/203)).\n- Add new values for `expected_version` for event store append events behaviour ([#127](https://github.com/commanded/commanded/pull/127)).\n- Export `Commanded.Commands.Router` macros in `.formatter.exs` file ([#204](https://github.com/commanded/commanded/pull/204)).\n- Generate specs and docs for Router dispatch functions only once ([#206](https://github.com/commanded/commanded/pull/206)).\n- Allow two-arity predicate function in `wait_for_event` receiving both event data and recorded event struct ([#213](https://github.com/commanded/commanded/pull/213)).\n- Allow `:infinity` timeout on command dispatch ([#227](https://github.com/commanded/commanded/pull/227))\n- Strict process manager routing ([#243](https://github.com/commanded/commanded/pull/243)).\n- Allow `Commanded.Aggregate.Multi` to be nested ([#244](https://github.com/commanded/commanded/pull/244)).\n- Add `child_spec/0` function to `Commanded.EventStore` behaviour.\n- Add `delete_subscription/2` to `Commanded.EventStore` behaviour ([#245](https://github.com/commanded/commanded/pull/245)).\n- Add `refute_receive_event/2` to `Commanded.Assertions.EventAssertions` test helpers.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix typo in `include_execution_result` global router option ([#216](https://github.com/commanded/commanded/pull/216)).\n- Handle the `{:ok, _}` tuple dispatch result in process manager command dispatch ([#236](https://github.com/commanded/commanded/pull/236)).\n- Allow string keys for `Commanded.Middleware.Pipeline.assign_metadata/3`, atoms are being deprecated ([#228](https://github.com/commanded/commanded/pull/228))\n- Fix `Commanded.PubSub.subscribe` typespec ([#222](https://github.com/commanded/commanded/pull/222)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Migrate to [Jason](https://hex.pm/packages/jason) for JSON serialization ([#234](https://github.com/commanded/commanded/pull/234)).\n\n  You will need to add Jason as a dependency in `mix.exs`:\n\n  ```elixir\n  defp deps do\n    [{:jason, \"~> 1.2\"}]\n  end\n  ```\n\n  Jason has _no support_ for encoding arbitrary structs - explicit implementation of the `Jason.Encoder` protocol is always required. You *must* update all your domain event modules, aggregate state (when using state snapshotting), and process manager state to include `@derive Jason.Encoder` as shown below:\n\n  ```elixir\n  defmodule AnEvent do\n    @derive Jason.Encoder\n    defstruct [:field]\n  end\n  ```\n\n- Extend aggregate lifespan behaviour to include `after_error/1` and `after_command/1` callbacks ([#210](https://github.com/commanded/commanded/pull/210)).\n\n  Previously you only had to define an `after_event/1` callback function to implement the `Commanded.Aggregates.AggregateLifespan` behaviour:\n\n  ```elixir\n  defmodule BankAccountLifespan do\n    @behaviour Commanded.Aggregates.AggregateLifespan\n\n    def after_event(%BankAccountClosed{}), do: :stop\n    def after_event(_event), do: :infinity\n  end\n  ```\n\n  Now you must also define `after_command/1` and `after_error/1` callback functions:\n\n  ```elixir\n  defmodule BankAccountLifespan do\n    @behaviour Commanded.Aggregates.AggregateLifespan\n\n    def after_event(%BankAccountClosed{}), do: :stop\n    def after_event(_event), do: :infinity\n\n    def after_command(%CloseAccount{}), do: :stop\n    def after_command(_command), do: :infinity\n\n    def after_error(:invalid_initial_balance), do: :stop\n    def after_error(_error), do: :stop\n  end\n  ```","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Upgrading - Changelog","doc":"Please ensure you upgrade the following event store dependencies.\n\nUsing the Elixir [EventStore](https://github.com/commanded/eventstore):\n\n- `eventstore` to [v0.16.0](https://hex.pm/packages/eventstore)\n- `commanded_eventstore_adapter` to [v0.5.0](https://hex.pm/packages/commanded_eventstore_adapter)\n\nUsing [EventStoreDB](https://www.eventstore.com/):\n\n- `commanded_extreme_adapter` to [v0.6.0](https://hex.pm/packages/commanded_extreme_adapter)\n\nCommanded Ecto projections:\n\n- `commanded_ecto_projections` to [v0.8.0](https://hex.pm/packages/commanded_ecto_projections)\n\nCommanded scheduler:\n\n- `commanded_scheduler` to [v0.2.0](https://hex.pm/packages/commanded_scheduler)\n\n---","ref":"changelog.html#upgrading"},{"type":"extras","title":"v0.17.5 - Changelog","doc":"","ref":"changelog.html#v0-17-5"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Process manager idle process timeout ([#290](https://github.com/commanded/commanded/pull/290)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.17.4 - Changelog","doc":"","ref":"changelog.html#v0-17-4"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Register event handler and process manager subscriptions on process start ([#272](https://github.com/commanded/commanded/pull/272)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.17.3 - Changelog","doc":"","ref":"changelog.html#v0-17-3"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix snapshot recording ([#196](https://github.com/commanded/commanded/pull/196)).\n- Fixed warning about deprecated time unit in elixir 1.8 ([#239](https://github.com/commanded/commanded/pull/239)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.17.2 - Changelog","doc":"","ref":"changelog.html#v0-17-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Remove default `error/4` callback function from process manager to silence deprecation warning.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.17.1 - Changelog","doc":"","ref":"changelog.html#v0-17-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support `Phoenix.PubSub` v1.1.0.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Set default aggregate lifespan timeout to `:infinity` ([#200](https://github.com/commanded/commanded/pull/200)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.17.0 - Changelog","doc":"","ref":"changelog.html#v0-17-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Ability to globally override `include_execution_result` and `include_aggregate_version` in environment config ([#168](https://github.com/commanded/commanded/pull/168)).\n- Handle custom type serialization in snapshot source type ([#165](https://github.com/commanded/commanded/pull/165)).\n- Fix compiler warnings in generated code (routers, event handlers, and process managers).\n- Add `InMemory.reset!/0` for testing purposes ([#175](https://github.com/commanded/commanded/pull/175)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Ensure process managers can be configured with `:strong` consistency.\n- Fix error when subscription process already tracked ([#180](https://github.com/commanded/commanded/pull/180)).\n\n---","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.16.0 - Changelog","doc":"- Support composite command routers ([#111](https://github.com/commanded/commanded/pull/111)).\n- Aggregate state snapshots ([#121](https://github.com/commanded/commanded/pull/121)).\n- New `error/3` callback for process manager and deprecated `error/4` ([#124](https://github.com/commanded/commanded/pull/124))\n- Router support for identity prefix function.\n- Retry command execution on concurrency error ([#132](https://github.com/commanded/commanded/pull/132)).\n- Event handler `error/3` callback ([#133](https://github.com/commanded/commanded/pull/133)).\n- Support distributed dispatch consistency ([#135](https://github.com/commanded/commanded/pull/135)).\n- Defer event handler and process router init until after subscribed ([#138](https://github.com/commanded/commanded/pull/138)).\n- Replace aggregate lifespan `after_command/1` callback with `after_event/1` ([#139](https://github.com/commanded/commanded/issues/139)).\n- Support process manager routing to multiple instances ([#141](https://github.com/commanded/commanded/pull/141)).\n- Allow a default consistency to be set via the application env ([#150](https://github.com/commanded/commanded/pull/150)).\n- Command dispatch consistency using explicit handler names ([#161](https://github.com/commanded/commanded/pull/161)).","ref":"changelog.html#v0-16-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- The `Commanded.Aggregates.AggregateLifespan` behaviour has been changed from `after_command/1` to `after_event/1`. You will need to update your own lifespan modules to use events instead of commands to shutdown an aggregate process after an inactivity timeout.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Upgrading - Changelog","doc":"Please ensure you upgrade the following event store dependencies.\n\nUsing the Elixir [EventStore](https://github.com/commanded/eventstore):\n\n- `eventstore` to [v0.14.0](https://hex.pm/packages/eventstore)\n- `commanded_eventstore_adapter` to [v0.4.0](https://hex.pm/packages/commanded_eventstore_adapter)\n\nUsing [EventStoreDB](https://www.eventstore.com/):\n\n- `commanded_extreme_adapter` to [v0.5.0](https://hex.pm/packages/commanded_extreme_adapter)\n\n---","ref":"changelog.html#upgrading"},{"type":"extras","title":"v0.15.1 - Changelog","doc":"","ref":"changelog.html#v0-15-1"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Event handler `child_spec/1` must include config options defined by use macro.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.15.0 - Changelog","doc":"","ref":"changelog.html#v0-15-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Process manager command dispatch error handling ([#93](https://github.com/commanded/commanded/issues/93)).\n- Event handlers may define an `init/0` callback function to start any related processes. It must return `:ok`, otherwise the handler process will be stopped.\n- Add `include_execution_result` option to command dispatch ([#96](https://github.com/commanded/commanded/pull/96)).\n- Add `Commanded.Aggregate.Multi` ([#98](https://github.com/commanded/commanded/pull/98)) as a way to return multiple events from a command dispatch that require aggregate state to be updated after each event.\n- Correlation and causation ids ([#105](https://github.com/commanded/commanded/pull/105)).\n- Initial support for running on a cluster of nodes ([#80](https://github.com/commanded/commanded/pull/80)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Adding a prefix to the aggregate in the router breaks the strong consistency of command dispatch ([#101](https://github.com/commanded/commanded/issues/101)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Upgrading - Changelog","doc":"Please ensure you upgrade the following event store dependencies.\n\nUsing the Elixir [EventStore](https://github.com/commanded/eventstore):\n\n- `eventstore` to [v0.13.0](https://hex.pm/packages/eventstore)\n- `commanded_eventstore_adapter` to [v0.3.0](https://hex.pm/packages/commanded_eventstore_adapter)\n\nUsing [EventStoreDB](https://www.eventstore.com/):\n\n- `commanded_extreme_adapter` to [v0.4.0](https://hex.pm/packages/commanded_extreme_adapter)\n\n---","ref":"changelog.html#upgrading"},{"type":"extras","title":"v0.14.0 - Changelog","doc":"","ref":"changelog.html#v0-14-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Dispatch command with `:eventual` or `:strong` consistency guarantee ([#82](https://github.com/commanded/commanded/issues/82)).\n- Additional stream prefix per aggregate ([#77](https://github.com/commanded/commanded/issues/77)).\n- Include custom metadata during command dispatch ([#61](https://github.com/commanded/commanded/issues/61)).\n- Validate command dispatch registration in router ([59](https://github.com/commanded/commanded/issues/59)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Upgrading - Changelog","doc":"Please ensure you upgrade the following event store dependencies.\n\nUsing the Elixir [EventStore](https://github.com/commanded/eventstore):\n\n- `eventstore` to [v0.11.0](https://hex.pm/packages/eventstore)\n- `commanded_eventstore_adapter` to [v0.2.0](https://hex.pm/packages/commanded_eventstore_adapter)\n\nUsing [EventStoreDB](https://www.eventstore.com/):\n\n- `commanded_extreme_adapter` to [v0.3.0](https://hex.pm/packages/commanded_extreme_adapter)\n\n---","ref":"changelog.html#upgrading"},{"type":"extras","title":"v0.13.0 - Changelog","doc":"","ref":"changelog.html#v0-13-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Command dispatch optionally returns aggregate version, using `include_aggregate_version: true` during dispatch.\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.12.0 - Changelog","doc":"","ref":"changelog.html#v0-12-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- `Commanded.Event.Handler` and `Commanded.ProcessManagers.ProcessManager` macros to simplify defining, and starting, event handlers and process managers. Note the previous approach to defining and starting may still be used, so this is *not* a breaking change.\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.11.0 - Changelog","doc":"","ref":"changelog.html#v0-11-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Shutdown idle aggregate processes ([#43](https://github.com/commanded/commanded/issues/43)).\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.10.0 - Changelog","doc":"","ref":"changelog.html#v0-10-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Extract event store integration to a behaviour (`Commanded.EventStore`). This defines the contract to be implemented by an event store adapter. It allows additional event store databases to be used with Commanded.\n\n  By default, a `GenServer` in-memory event store adapter is used. This should **only be used for testing** as there is no persistence.\n\n  The existing PostgreSQL-based [eventstore](https://github.com/commanded/eventstore) integration has been extracted as a separate package ([commanded_eventstore_adapter](https://github.com/commanded/commanded-eventstore-adapter)). There is also a new adapter for Greg Young's Event Store using the Extreme library ([commanded_extreme_adapter](https://github.com/commanded/commanded-extreme-adapter)).\n\n  You must install the required event store adapter package and update your environment configuration to specify the `:event_store_adapter` module. See the [README](https://github.com/commanded/commanded/blob/master/README.md) for details.\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.9.0 - Changelog","doc":"","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Stream events from event store when rebuilding aggregate state.\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.8.5 - Changelog","doc":"","ref":"changelog.html#v0-8-5"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Upgrade to Elixir 1.4 and remove compiler warnings.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.8.4 - Changelog","doc":"","ref":"changelog.html#v0-8-4"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Event handler and process manager subscriptions should be created from a given stream position ([#14](https://github.com/commanded/commanded/issues/14)).\n- Stop process manager instance after reaching its final state ([#24](https://github.com/commanded/commanded/issues/24)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.8.3 - Changelog","doc":"","ref":"changelog.html#v0-8-3"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Middleware `after_failure` callback is executed even when a middleware halts execution.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.8.2 - Changelog","doc":"","ref":"changelog.html#v0-8-2"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- JsonSerializer should ensure event type atom exists when deserializing ([#28](https://github.com/commanded/commanded/issues/28)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.8.1 - Changelog","doc":"","ref":"changelog.html#v0-8-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Command handlers should be optional by default ([#30](https://github.com/commanded/commanded/issues/30)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.8.0 - Changelog","doc":"","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Simplify aggregates and process managers ([#31](https://github.com/commanded/commanded/issues/31)).\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.7.1 - Changelog","doc":"","ref":"changelog.html#v0-7-1"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Restarting aggregate process should load all events from its stream in batches. The Event Store read stream default limit is 1,000 events.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Command handling middleware allows a command router to define middleware modules that are executed before, and after success or failure of each command dispatch ([#12](https://github.com/commanded/commanded/issues/12)).\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.6.3 - Changelog","doc":"","ref":"changelog.html#v0-6-3"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Process manager instance processes event non-blocking to prevent timeout during event processing and any command dispatching. It persists last seen event id to ensure events are handled only once.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.6.2 - Changelog","doc":"","ref":"changelog.html#v0-6-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Command dispatch timeout. Allow a `timeout` value to be configured during command registration or dispatch. This overrides the default timeout of 5 seconds. The same as the default `GenServer` call timeout.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix pending aggregates restarts: supervisor restarts aggregate process but it cannot accept commands ([#22](https://github.com/commanded/commanded/pull/22)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.6.1 - Changelog","doc":"","ref":"changelog.html#v0-6-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Upgrade `eventstore` mix dependency to v0.6.0 to use support for recorded events created_at as `NaiveDateTime`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Confirm receipt of events in event handler and process manager router ([#19](https://github.com/commanded/commanded/pull/19)).\n- Convert keys to atoms when decoding JSON using Poison decoder.\n- Prefix process manager instance snapshot uuid with process manager name.\n- Multi command dispatch registration in router ([#16](https://github.com/commanded/commanded/issues/16)).\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Include event metadata as second argument to event handlers. An event handler must now implement the `Commanded.Event.Handler` behaviour consisting of a single `handle_event/2` function.\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Macro to assist with building process managers ([README](https://github.com/commanded/commanded/tree/feature/process-manager-macro#process-managers)).\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Include unit test event assertion function: `assert_receive_event/2` ([#13](https://github.com/commanded/commanded/pull/13)).\n- Include top level application in mix config.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Don't persist an aggregate's pending events when executing a command returns an error ([#10](https://github.com/commanded/commanded/pull/10)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Ensure an aggregate's pending events are persisted in the order they were applied.\n\n---","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.2.1 - Changelog","doc":"","ref":"changelog.html#v0-2-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support integer, atom or strings as an aggregate UUID ([#7](https://github.com/commanded/commanded/pull/7)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release.","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"In memory event store","doc":"# In memory event store\n\nCommanded provides an in-memory event store implementation, for **test use only**, in the module `Commanded.EventStore.Adapters.InMemory`. This is a transient event store without persistence.","ref":"inmemoryeventstore.html"},{"type":"extras","title":"Configuration - In memory event store","doc":"Ensure you configure the in-memory adapter in your environment config file, `config/test.exs`:\n\n```elixir\nconfig :my_app, MyApp.App,\n  event_store: [\n    adapter: Commanded.EventStore.Adapters.InMemory,\n    serializer: Commanded.Serialization.JsonSerializer\n  ]\n```\n\nYou may replace or omit the serializer configuration. By including it here we ensure events used by the tests can be successfully serialized and deserialized.","ref":"inmemoryeventstore.html#configuration"},{"type":"extras","title":"Usage - In memory event store","doc":"#### ExUnit case template\n\nYou can use ExUnit's case template feature to restart the in-memory event store for each test run.\n\n```elixir\ndefmodule InMemoryEventStoreCase do\n  use ExUnit.CaseTemplate\n\n  alias Commanded.EventStore.Adapters.InMemory\n\n  setup do\n    {:ok, _apps} = Application.ensure_all_started(:my_app)\n\n    on_exit(fn ->\n      :ok = Application.stop(:my_app)\n    end)\n  end\nend\n```\n\nReplace both occurrences of `:my_app` in the above ExUnit case template with the name of your own application.\n\nThe reason why your app must be stopped and then started between resetting the event store is to ensure all application processes are restarted with their initial state to prevent state from one test affecting another.\n\nUse the `InMemoryEventStoreCase` module within any test files that need to use the event store.\n\n```elixir\ndefmodule ExampleTest do\n  use InMemoryEventStoreCase\n\n  # Define your tests here ...\nend\n```\n\n#### Running your tests\n\nRun `mix test` as usual to execute the tests using the in-memory event store.","ref":"inmemoryeventstore.html#usage"},{"type":"extras","title":"Getting started","doc":"# Getting started\n\nCommanded can be installed from the package manager hex as follows.\n\n1. Add `commanded` to your list of dependencies in `mix.exs`:\n\n    ```elixir\n    def deps do\n      [{:commanded, \"~> 1.4\"}]\n    end\n    ```\n\n2. Optionally add `jason` to support JSON serialization via `Commanded.Serialization.JsonSerializer`:\n\n    ```elixir\n    def deps do\n      [{:jason, \"~> 1.3\"}]\n    end\n    ```\n\n3. Fetch mix dependencies:\n\n    ```console\n    $ mix deps.get\n    ```\n\n4. Define a Commanded application module for your app, see the [Application guide](https://hexdocs.pm/commanded/application.html) for details.\n\n    ```elixir\n    defmodule MyApp.Application do\n      use Commanded.Application, otp_app: :my_app\n    end\n    ```\n\n5. Configure one of the supported event stores by following the [Choosing an Event Store guide](https://hexdocs.pm/commanded/choosing-an-event-store.html).","ref":"getting-started.html"},{"type":"extras","title":"Choosing an event store","doc":"# Choosing an event store\n\nYou must decide which event store to use with Commanded. You have a choice between two existing event stores:\n\n- PostgreSQL-based Elixir [EventStore](https://github.com/commanded/eventstore) ([adapter](https://github.com/commanded/commanded-eventstore-adapter)).\n\n- [EventStoreDB](https://www.eventstore.com/) ([adapter](https://github.com/commanded/commanded-extreme-adapter)).\n\nThere is also an [in-memory event store adapter](https://github.com/commanded/commanded/wiki/In-memory-event-store) for *test use only*.\n\nWant to use a different event store? Then you will need to write your own event store provider as described below.\n\n---","ref":"choosing-an-event-store.html"},{"type":"extras","title":"PostgreSQL-based Elixir EventStore - Choosing an event store","doc":"Use [`:commanded_eventstore_adapter`](https://github.com/commanded/commanded-eventstore-adapter) to persist events to a PostgreSQL database. As the name implies, this is the adapter for [EventStore](https://github.com/commanded/eventstore), which is open-source event store using PostgreSQL for persistence and implemented in Elixir.\n\n---\n\n## [EventStoreDB](https://www.eventstore.com/)\n\nUse [`:commanded_extreme_adapter`](https://github.com/commanded/commanded-extreme-adapter) to persist events to [EventStoreDB](https://www.eventstore.com/): an open-source database, the best data storage solution for event-sourced systems. It can be run as a cluster of nodes containing the same data, which remains available for writes provided at least half the nodes are alive and connected.\n\nThe quickest way to get started with EventStoreDB is by using their official [EventStoreDB Docker container](https://hub.docker.com/r/eventstore/eventstore).\n\nThe Commanded adapter uses the [`:extreme`](https://github.com/exponentially/extreme) Elixir TCP client to connect to EventStoreDB.","ref":"choosing-an-event-store.html#postgresql-based-elixir-eventstore"},{"type":"extras","title":"Running EventStoreDB - Choosing an event store","doc":"You **must** run EventStoreDB with all projections enabled and standard projections started.\n\nUse the `--run-projections=all --start-standard-projections=true` flags when running the EventStoreDB executable.\n\n---","ref":"choosing-an-event-store.html#running-eventstoredb"},{"type":"extras","title":"Writing your own event store provider - Choosing an event store","doc":"To use an alternative event store with Commanded you will need to implement the `Commanded.EventStore.Adapter` behaviour. This defines the contract to be implemented by an adapter module to allow an event store to be used with Commanded. Tests to verify an adapter conforms to the behaviour are provided in `test/event_store_adapter`.\n\nYou can use one of the existing adapters ([commanded_eventstore_adapter](https://github.com/commanded/commanded-eventstore-adapter) or [commanded_extreme_adapter](https://github.com/commanded/commanded-extreme-adapter)) to understand what is required.","ref":"choosing-an-event-store.html#writing-your-own-event-store-provider"},{"type":"extras","title":"Using Commanded","doc":"# Using Commanded\n\nCommanded provides the building blocks for you to create your own Elixir applications following the CQRS/ES pattern.\n\nA separate guide is provided for each of the components you can build:\n\n- [Application](https://hexdocs.pm/commanded/application.html)\n- [Aggregates](https://hexdocs.pm/commanded/aggregates.html)\n- [Commands, registration and dispatch](https://hexdocs.pm/commanded/commands.html)\n- [Events and handlers](https://hexdocs.pm/commanded/events.html)\n- [Process managers](https://hexdocs.pm/commanded/process-managers.html)\n\nCommanded uses strong consistency for command dispatch (write model) and eventual consistency, by default, for the read model. Receiving an `:ok` reply from dispatch indicates the command was successfully handled and any created domain events fully persisted to your chosen event store. You may opt into strong consistency for individual event handlers and command dispatch as required.","ref":"usage.html"},{"type":"extras","title":"Quick overview - Using Commanded","doc":"Here's an example bank account opening feature built using Commanded to demonstrate its usage.\n\n1. Define an `OpenBankAccount` command:\n\n    ```elixir\n    defmodule OpenBankAccount do\n      defstruct [:account_number, :initial_balance]\n    end\n    ```\n\n2. Define a corresponding `BankAccountOpened` domain event:\n\n    ```elixir\n    defmodule BankAccountOpened do\n      @derive Jason.Encoder\n      defstruct [:account_number, :initial_balance]\n    end\n    ```\n\n3. Build a `BankAccount` aggregate to handle the command, protect its business invariants, and return a domain event when successfully handled:\n\n    ```elixir\n    defmodule BankAccount do\n      defstruct [:account_number, :balance]\n\n      # Public command API\n\n      def execute(%BankAccount{account_number: nil}, %OpenBankAccount{account_number: account_number, initial_balance: initial_balance})\n        when initial_balance > 0\n      do\n        %BankAccountOpened{account_number: account_number, initial_balance: initial_balance}\n      end\n\n      # Ensure initial balance is never zero or negative\n      def execute(%BankAccount{}, %OpenBankAccount{initial_balance: initial_balance})\n        when initial_balance <= 0\n      do\n        {:error, :initial_balance_must_be_above_zero}\n      end\n\n      # Ensure account has not already been opened\n      def execute(%BankAccount{}, %OpenBankAccount{}) do\n        {:error, :account_already_opened}\n      end\n\n      # State mutators\n\n      def apply(%BankAccount{} = account, %BankAccountOpened{} = event) do\n        %BankAccountOpened{account_number: account_number, initial_balance: initial_balance} = event\n\n        %BankAccount{account |\n          account_number: account_number,\n          balance: initial_balance\n        }\n      end\n    end\n    ```\n\n4. Define a router module to route the open account command to the bank account aggregate:\n\n    ```elixir\n    defmodule BankRouter do\n      use Commanded.Commands.Router\n\n      dispatch OpenBankAccount, to: BankAccount, identity: :account_number\n    end\n    ```\n\n5. Define an application to host the aggregate and supporting processes:\n\n    ```elixir\n    defmodule BankApp do\n      use Commanded.Application,\n        otp_app: :bank,\n        event_store: [adapter: Commanded.EventStore.Adapters.InMemory]\n\n      router BankRouter\n    end\n    ```\n\n    This application is configured to use in-memory event store included with Commanded for testing.\n\n6. Create an event handler module that updates a bank account balance:\n\n    ```elixir\n    defmodule AccountBalanceHandler do\n      use Commanded.Event.Handler,\n        application: BankApp,\n        name: __MODULE__\n\n      def after_start(_state) do\n        with {:ok, _pid} <- Agent.start_link(fn -> 0 end, name: __MODULE__) do\n          :ok\n        end\n      end\n\n      def handle(%BankAccountOpened{initial_balance: initial_balance}, _metadata) do\n        Agent.update(__MODULE__, fn _ -> initial_balance end)\n      end\n\n      def current_balance do\n        Agent.get(__MODULE__, fn balance -> balance end)\n      end\n    end\n    ```\n\n7. Start the application and event handler processes:\n\n    ```elixir\n    {:ok, _pid} = BankApp.start_link()\n    {:ok, _pid} = AccountBalanceHandler.start_link()\n    ```\n\n    In a real application you would use a supervisor to start these processes.\n\nFinally, we can dispatch a command to open a new bank account:\n\n```elixir\n:ok = BankApp.dispatch(%OpenBankAccount{account_number: \"ACC123456\", initial_balance: 1_000})\n```","ref":"usage.html#quick-overview"},{"type":"extras","title":"Application","doc":"# Application\n\nCommanded allows you to define, supervise, and start your own application module. To use Commanded you must create at least one application. You can create multiple Commanded applications which will run independently, each using its own separately configured event store.\n\nThe application expects at least an `:otp_app` option to be specified. It should point to an OTP application containing the application's configuration.\n\nFor example, the application:\n\n```elixir\ndefmodule MyApp.Application do\n  use Commanded.Application, otp_app: :my_app\n\n  router MyApp.Router\nend\n```\n\nCould be configured with:\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.Application,\n  event_store: [\n    adapter: Commanded.EventStore.Adapters.EventStore,\n    event_store: MyApp.EventStore\n  ],\n  pubsub: :local,\n  registry: :local\n```\n\nAlternatively, you can include the event store, pubsub, and registry config when defining the application:\n\n```elixir\ndefmodule MyApp.Application do\n  use Commanded.Application,\n    otp_app: :my_app,\n    event_store: [\n      adapter: Commanded.EventStore.Adapters.EventStore,\n      event_store: MyApp.EventStore\n    ],\n    pubsub: :local,\n    registry: :local\n\n  router(MyApp.Router)\nend\n```\n\nFinally, you can provide an optional `init/1` function to provide runtime configuration.\n\n```elixir\ndefmodule MyApp.Application do\n  use Commanded.Application, otp_app: :my_app\n\n  def init(config) do\n    {:ok, config}\n  end\nend\n```","ref":"application.html"},{"type":"extras","title":"Routing commands - Application","doc":"A Commanded application is also a composite router. This provides the `router` macro allowing you to include one or more router modules for command dispatch.\n\n```elixir\ndefmodule MyApp.Application do\n  use Commanded.Application, otp_app: :my_app\n\n  router(MyApp.Billing.Router)\n  router(MyApp.Customers.Router)\n  router(MyApp.Notifications.Router)\nend\n```\n\nOnce you have defined a router you can dispatch a command using the application module:\n\n```elixir\n:ok = MyApp.Application.dispatch(%RegisterCustomer{id: Commanded.UUID.uuid4(), name: \"Ben\"})\n```","ref":"application.html#routing-commands"},{"type":"extras","title":"Aggregates","doc":"# Aggregates\n\nBuild your aggregates using standard Elixir modules and functions, with structs to hold state. There is no external dependency requirement.\n\nAn aggregate is comprised of its state, public command functions, and state mutators.\n\nIn a CQRS application all state must be derived from the published domain events. This prevents tight coupling between aggregate instances, such as querying for their state, and ensures their state isn't exposed.\n\nIf an aggregate needs data owned by another aggregate, then you should lookup the data from a projection and include it in the command before dispatch. Usually this would be a projection into a read model built for querying. You can use Commanded Ecto projections to project events into a SQL database.","ref":"aggregates.html"},{"type":"extras","title":"Aggregate state - Aggregates","doc":"Use the `defstruct` keyword to define the aggregate state and fields.\n\n```elixir\ndefmodule ExampleAggregate do\n  defstruct [:uuid, :name]\nend\n```","ref":"aggregates.html#aggregate-state"},{"type":"extras","title":"Command functions - Aggregates","doc":"A command function receives the aggregate's state and the command to execute. It must return the resultant domain events, which may be one event or multiple events. You can return a single event or a list of events: `%Event{}`, `[%Event{}]`, `{:ok, %Event{}}`, or `{:ok, [%Event{}]}`.\n\nTo respond without returning an event you can return `:ok`, `nil` or an empty list as either `[]` or `{:ok, []}`.\n\nFor business rule violations and errors you may return an `{:error, error}` tagged tuple or raise an exception.\n\nName your public command functions `execute/2` to dispatch commands directly to the aggregate without requiring an intermediate command handler.\n\n```elixir\ndefmodule ExampleAggregate do\n  def execute(%ExampleAggregate{uuid: nil}, %Create{} = command) do\n    %Create{uuid: uuid, name: name} = command\n\n    event = %Created{uuid: uuid, name: name}\n\n    {:ok, event}\n  end\n\n  def execute(%ExampleAggregate{}, %Create{}),\n    do: {:error, :already_created}\nend\n```","ref":"aggregates.html#command-functions"},{"type":"extras","title":"State mutators - Aggregates","doc":"The state of an aggregate can only be mutated by applying a domain event to its state. This is achieved by an `apply/2` function that receives the state and the domain event. It returns the modified state.\n\nPattern matching is used to invoke the respective `apply/2` function for an event. These functions **MUST NOT** fail as they are used when rebuilding the aggregate state from its history of domain events. You cannot reject the event once it has occurred.\n\n```elixir\ndefmodule ExampleAggregate do\n  def apply(%ExampleAggregate{}, %Created{} = event) do\n    %Created{uuid: uuid, name: name} = event\n\n    %ExampleAggregate{uuid: uuid, name: name}\n  end\nend\n```","ref":"aggregates.html#state-mutators"},{"type":"extras","title":"Example aggregate - Aggregates","doc":"You can define your aggregate with public API functions using the language of your domain. In this bank account example, the public function to open a new account is `open_account/3`:\n\n```elixir\ndefmodule BankAccount do\n  defstruct [:account_number, :balance]\n\n  # Public API\n\n  def open_account(%BankAccount{account_number: nil}, account_number, initial_balance)\n    when initial_balance > 0\n  do\n    event = %BankAccountOpened{account_number: account_number, initial_balance: initial_balance}\n\n    {:ok, event}\n  end\n\n  def open_account(%BankAccount{}, _account_number, initial_balance)\n    when initial_balance <= 0\n  do\n    {:error, :initial_balance_must_be_above_zero}\n  end\n\n  def open_account(%BankAccount{}, _account_number, _initial_balance) do\n    {:error, :account_already_opened}\n  end\n\n  # State mutators\n\n  def apply(%BankAccount{} = account, %BankAccountOpened{} = event) do\n    %BankAccountOpened{account_number: account_number, initial_balance: initial_balance} = event\n\n    %BankAccount{account |\n      account_number: account_number,\n      balance: initial_balance\n    }\n  end\nend\n```\n\nWith this approach you must dispatch the command to a command handler:\n\n```elixir\ndefmodule Commanded.ExampleDomain.OpenAccountHandler do\n  alias BankAccount\n  alias Commands.OpenAccount\n\n  @behaviour Commanded.Commands.Handler\n\n  def handle(%BankAccount{} = aggregate, %OpenAccount{} = command) do\n    %OpenAccount{account_number: account_number, initial_balance: initial_balance} = command\n\n    BankAccount.open_account(aggregate, account_number, initial_balance)\n  end\nend\n```\n\nAn alternative approach is to expose one or more public command functions, `execute/2`, and use pattern matching on the command argument. With this approach you can route your commands directly to the aggregate.\n\nIn this example the `execute/2` function pattern matches on the `OpenAccount` command module:\n\n```elixir\ndefmodule BankAccount do\n  defstruct [:account_number, :balance]\n\n  # Public API\n\n  def execute(\n    %BankAccount{account_number: nil},\n    %OpenAccount{initial_balance: initial_balance} = command\n    ) when initial_balance > 0\n  do\n    %OpenAccount{account_number: account_number} = command\n\n    %BankAccountOpened{account_number: account_number, initial_balance: initial_balance}\n  end\n\n  def execute(%BankAccount{}, %OpenAccount{initial_balance: initial_balance})\n    when initial_balance <= 0\n  do\n    {:error, :initial_balance_must_be_above_zero}\n  end\n\n  def execute(%BankAccount{}, %OpenAccount{}) do\n    {:error, :account_already_opened}\n  end\n\n  # State mutators\n\n  def apply(%BankAccount{} = account, %BankAccountOpened{} = event) do\n    %BankAccountOpened{account_number: account_number, initial_balance: initial_balance} = event\n\n    %BankAccount{account |\n      account_number: account_number,\n      balance: initial_balance\n    }\n  end\nend\n```\n\n> #### Effective Aggregate State {: .tip}\n>\n> The aggregate state should be carefully designed to maintain only the data required to:\n>\n> - Enforce business rules and invariants (e.g. preventing withdrawals that would overdraw an account)\n> - Provide context for command handling based on previous events (e.g. checking an order's current status)\n> - Make decisions that depend on historical events (e.g. verifying a refund doesn't exceed original payment)\n>\n> A common anti-pattern is blindly copying all event data into aggregate state without considering whether that data\n> is actually needed for command handling. If a piece of state is never referenced in any command handling logic,\n> it should be removed from the aggregate. This keeps the aggregate focused and maintainable.\n>\n> State that is only needed for querying/reporting should be maintained in read models rather than aggregate state.","ref":"aggregates.html#example-aggregate"},{"type":"extras","title":"Using `Commanded.Aggregate.Multi` to return multiple events - Aggregates","doc":"Sometimes you need to create multiple events from a single command. You can use `Commanded.Aggregate.Multi` to help track the events and update the aggregate state. This can be useful when you want to emit multiple events that depend upon the aggregate state being updated.\n\nAny errors encountered will be returned to the caller, the modified aggregate state and any pending events are discarded.","ref":"aggregates.html#using-commanded-aggregate-multi-to-return-multiple-events"},{"type":"extras","title":"Example - Aggregates","doc":"In the example below, money is withdrawn from the bank account and the updated balance is used to check whether the account is overdrawn.\n\n```elixir\ndefmodule BankAccount do\n  defstruct [\n    account_number: nil,\n    balance: 0,\n    state: nil,\n  ]\n\n  alias Commanded.Aggregate.Multi\n\n  # Public API\n\n  def execute(\n    %BankAccount{state: :active} = account,\n    %WithdrawMoney{amount: amount})\n    when is_number(amount) and amount > 0\n  do\n    account\n    |> Multi.new()\n    |> Multi.execute(&withdraw_money(&1, amount))\n    |> Multi.execute(&check_balance/1)\n  end\n\n  # State mutators\n\n  def apply(%BankAccount{} = state, %MoneyWithdrawn{balance: balance}),\n    do: %BankAccount{state | balance: balance}\n\n  def apply(%BankAccount{} = state, %AccountOverdrawn{}),\n    do: %BankAccount{state | state: :overdrawn}\n\n  # Private helpers\n\n  defp withdraw_money(%BankAccount{} = account, amount) do\n    %BankAccount{account_number: account_number, balance: balance} = account\n\n    event = %MoneyWithdrawn{\n      account_number: account_number,\n      amount: amount,\n      balance: balance - amount\n    }\n\n    {:ok, event}\n  end\n\n  defp check_balance(%BankAccount{balance: balance} = account)\n    when balance < 0\n  do\n    %BankAccount{account_number: account_number} = account\n\n    event = %AccountOverdrawn{account_number: account_number, balance: balance}\n\n    {:ok, event}\n  end\n\n  defp check_balance(%BankAccount{}), do: :ok\nend\n```","ref":"aggregates.html#example"},{"type":"extras","title":"Aggregate state snapshots - Aggregates","doc":"A snapshot represents the aggregate state when all events to that point in time have been replayed. You can optionally configure state snapshotting for individual aggregates in your app configuration. Instead of loading every event for an aggregate when rebuilding its state, only the snapshot and any events appended since its creation are read. By default snapshotting is disabled for all aggregates.\n\nAs an example, assume a snapshot was taken after persisting an event for the aggregate at version 100. When the aggregate process is restarted we load and deserialize the snapshot data as the aggregate's initial state. Then we fetch and replay the aggregate's events after version 100.\n\nThis is a performance optimisation for aggregates that have a long lifetime or raise a large number of events. It limits the worst case scenario when rebuilding the aggregate state: it will need to read the snapshot and at most this many events from storage.\n\nUse the following options to configure snapshots for an aggregate:\n\n- `snapshot_every` - snapshot aggregate state every so many events. Use `nil` to disable snapshotting, or exclude the configuration entirely.\n\n- `snapshot_version` - a non-negative integer indicating the version of the aggregate state snapshot. Incrementing this version forces any earlier recorded snapshots to be ignored when rebuilding aggregate state.","ref":"aggregates.html#aggregate-state-snapshots"},{"type":"extras","title":"Example - Aggregates","doc":"In `config/config.exs` enable snapshots for `ExampleAggregate` after every ten events:\n\n```elixir\nconfig :my_app, MyApp.Application,\n  snapshotting: %{\n    MyApp.ExampleAggregate => [\n      snapshot_every: 10,\n      snapshot_version: 1\n    ]\n  }\n```","ref":"aggregates.html#example"},{"type":"extras","title":"Snapshot serialization - Aggregates","doc":"Aggregate state will be serialized using the configured event store serializer, by default this stores the data as JSON. Remember to derive the `Jason.Encoder` protocol for the aggregate state to ensure JSON serialization is supported, as shown below.\n\n```elixir\ndefmodule ExampleAggregate do\n  @derive Jason.Encoder\n  defstruct [:name, :datetime]\nend\n```\n\nYou can use the `Commanded.Serialization.JsonDecoder` protocol to decode the parsed JSON data into the expected types:\n\n```elixir\ndefimpl Commanded.Serialization.JsonDecoder, for: ExampleAggregate do\n  @doc \"\"\"\n  Parse the datetime included in the aggregate state\n  \"\"\"\n  def decode(%ExampleAggregate{} = state) do\n    %ExampleAggregate{datetime: datetime} = state\n\n    {:ok, dt, _} = DateTime.from_iso8601(datetime)\n\n    %ExampleAggregate{state | datetime: dt}\n  end\nend\n```\n\nNote: The default JSON encoding of a `DateTime` struct uses the `to_iso8601/1` function which is why we must decode it using the `from_iso8601/1` function.","ref":"aggregates.html#snapshot-serialization"},{"type":"extras","title":"Rebuilding an aggregate snapshot - Aggregates","doc":"Whenever you change the structure of an aggregate's state you **MUST** increment the `snapshot_version` number. The aggregate state will be rebuilt from its events, ignoring any existing snapshots. They will be overwritten when the next snapshot is taken.","ref":"aggregates.html#rebuilding-an-aggregate-snapshot"},{"type":"extras","title":"Commands","doc":"# Commands\n\nYou need to create a module per command and define the fields using `defstruct`:\n\n```elixir\ndefmodule OpenAccount do\n  @enforce_keys [:account_number]\n  defstruct [:account_number, :initial_balance]\nend\n```\n\nA command **must contain** a field to uniquely identify the aggregate instance (e.g. `account_number`). Use `@enforce_keys` to force the identity field to be specified when creating the command struct.\n\nSince commands are just plain Elixir structs you can use a library such as [`typed_struct`](https://hex.pm/packages/typed_struct) for defining structs, fields with their types, and enforcing mandatory keys without writing too much boilerplate code.\n\nCommanded logs commands during execution when using debug log level. To prevent sensitive data from being exposed in\nlogs you can use the built-in Elixir `Inspect` module to exclude fields.\n\n```elixir\ndefmodule RegisterUser do\n  @derive {Inspect, only: [:id, :name]}\n  defstruct [:id, :name, :password]\nend\n```","ref":"commands.html"},{"type":"extras","title":"Command dispatch and routing - Commands","doc":"A router module is used to route commands to their registered command handler and aggregate module.\n\nYou create a router module using `Commanded.Commands.Router` and register each command with its associated handler:\n\n```elixir\ndefmodule BankRouter do\n  use Commanded.Commands.Router\n\n  dispatch OpenAccount, to: OpenAccountHandler, aggregate: BankAccount, identity: :account_number\n  dispatch DepositMoney, to: DepositMoneyHandler, aggregate: BankAccount, identity: :account_number\nend\n```\n\nThis can be more succinctly configured by excluding command handlers and [dispatching directly to the aggregate](#dispatch-directly-to-aggregate), using [multi-command registration](#multi-command-registration), and with the [`identify`](#define-aggregate-identity) helper macro:\n\n```elixir\ndefmodule BankRouter do\n  use Commanded.Commands.Router\n\n  identify BankAccount, by: :account_number\n  dispatch [OpenAccount, DepositMoney], to: BankAccount\nend\n```\n\nA router must be registered with its associated application using the `router` macro, as shown below:\n\n```elixir\ndefmodule BankApp do\n  use Commanded.Application, otp_app: :bank_app\n\n  router BankRouter\nend\n```","ref":"commands.html#command-dispatch-and-routing"},{"type":"extras","title":"Command handlers - Commands","doc":"A command handler receives the aggregate and the command being executed. It allows you to validate, authorize, and/or enrich the command with additional data before executing the appropriate aggregate module function.\n\nThe command handler must implement the `Commanded.Commands.Handler` behaviour consisting of a single `handle/2` function. It receives the aggregate state and the command to be handled. It must return the raised domain events from the aggregate. It may return an `{:error, reason}` tuple on failure. The `handle/2` function is executed in the same process as the aggregate itself and is therefore within the consistency boundary of the aggregate. In effect, there is no essential difference between using the `handle/2` function in a command handler module vs. the `execute/2` function in the aggregate module aside from organization of your source code.\n\n```elixir\ndefmodule OpenAccountHandler do\n  @behaviour Commanded.Commands.Handler\n\n  def handle(%BankAccount{} = aggregate, %OpenAccount{} = command) do\n    %OpenAccount{account_number: account_number, initial_balance: initial_balance} = command\n\n    BankAccount.open_account(aggregate, account_number, initial_balance)\n  end\nend\n```\n\nCommand handlers execute in the context of the dispatch call, as such they are limited to the timeout period specified. The default timeout is five seconds, the same as a `GenServer` call. You can increase the timeout value for individual commands as required - see the section on [Timeouts](#timeouts) below.","ref":"commands.html#command-handlers"},{"type":"extras","title":"Dispatch directly to aggregate - Commands","doc":"It is also possible to route a command directly to an aggregate, without requiring an intermediate command handler.\n\n```elixir\ndefmodule BankRouter do\n  use Commanded.Commands.Router\n\n  # will route to `BankAccount.execute/2`\n  dispatch OpenAccount, to: BankAccount, identity: :account_number\nend\n```\n\nBy default, the aggregate module's `execute/2` function will be called with the aggregate's state and the command to execute. Using this approach, you will create an `execute/2` clause that pattern-matches on each command that the aggregate should handle.\n\nAlternatively, you may specify the name of a function (also receiving both the aggregate state and the command) on your aggregate module to which the command will be dispatched:\n\n```elixir\ndefmodule BankRouter do\n  use Commanded.Commands.Router\n\n  # Will route to `BankAccount.open_account/2`\n  dispatch OpenAccount, to: BankAccount, function: :open_account, identity: :account_number\nend\n```","ref":"commands.html#dispatch-directly-to-aggregate"},{"type":"extras","title":"Dispatching commands - Commands","doc":"You dispatch a command to its registered aggregate using the application:\n\n```elixir\n:ok = BankApp.dispatch(%OpenAccount{account_number: \"ACC123\", initial_balance: 1_000})\n```\n\nOptionally, you can dispatch a command using the router by providing the application as an option:\n\n```elixir\ncommand = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n:ok = BankRouter.dispatch(command, application: BankApp)\n```","ref":"commands.html#dispatching-commands"},{"type":"extras","title":"Define aggregate identity - Commands","doc":"You can define the identity field for an aggregate once using the `identify` macro. The configured identity will be used for all commands registered to the aggregate, unless overridden by a command registration.\n\n#### Example\n\n```elixir\ndefmodule BankRouter do\n  use Commanded.Commands.Router\n\n  identify BankAccount, by: :account_number\n\n  dispatch OpenAccount, to: BankAccount\nend\n```\n\nThe above configuration requires that all commands for the `BankAccount` aggregate must contain an `account_number` field used to identity a unique instance.\n\n#### Identity prefix\n\nAn optional identity prefix can be used to distinguish between different aggregates that would otherwise share the same identity. As an example you might have a `User` and a `UserPreferences` aggregate that you wish to share the same identity. In this scenario you should specify a `prefix` for each aggregate (e.g. \"user-\" and \"user-preference-\").\n\n```elixir\ndefmodule BankRouter do\n  use Commanded.Commands.Router\n\n  identify BankAccount,\n    by: :account_number,\n    prefix: \"bank-account-\"\n\n  dispatch OpenAccount, to: BankAccount\nend\n```\n\nThe prefix is used as the stream identity when appending, and reading, the aggregate's events (e.g. `  `). Note you **must not** change the stream prefix once you have events persisted in your event store, otherwise the aggregate's events cannot be read from the event store and its state cannot be rebuilt since the stream name will be different.\n\n#### Custom aggregate identity\n\nAny module that implements the `String.Chars` protocol can be used for an aggregate's identity. By default this includes the following Elixir built-in types: strings, integers, floats, atoms, and lists.\n\nYou can define your own custom identity modules and implement the `String.Chars` protocol for them:\n\n```elixir\ndefmodule AccountNumber do\n  defstruct [:branch, :account_number]\n\n  defimpl String.Chars do\n    def to_string(%AccountNumber{branch: branch, account_number: account_number}),\n      do: branch <> \":\" <> account_number\n  end\nend\n```\n\nThe custom identity will be converted to a string during command dispatch. This is used as the aggregate's identity and determines the stream to append its events in the event store.\n\n```elixir\nopen_account = %OpenAccount{\n  account_number: %AccountNumber{branch: \"B1\", account_number: \"ACC123\"},\n  initial_balance: 1_000\n}\n\n:ok = BankApp.dispatch(open_account)\n```","ref":"commands.html#define-aggregate-identity"},{"type":"extras","title":"Timeouts - Commands","doc":"A command handler has a default timeout of 5 seconds. The same default as a `GenServer.call/3` process call. It must handle the command in this period, otherwise the call fails and the caller process exits.\n\nYou can configure a different timeout value during command registration by providing a `timeout` option, defined in milliseconds:\n\n```elixir\ndefmodule BankRouter do\n  use Commanded.Commands.Router\n\n  # Configure a timeout of 1 second for the open account command handler\n  dispatch OpenAccount,\n    to: OpenAccountHandler,\n    aggregate: BankAccount,\n    identity: :account_number,\n    timeout: 1_000\nend\n```\n\nYou can override the timeout value during command dispatch. This example is dispatching the open account command with a timeout of 2 seconds:\n\n```elixir\nopen_account = %OpenAccount{account_number: \"ACC123\", initial_balance: 1_000}\n\n:ok = BankApp.dispatch(open_account, timeout: 2_000)\n```","ref":"commands.html#timeouts"},{"type":"extras","title":"Multi-command registration - Commands","doc":"Command routers support multi command registration so you can group related command handlers into the same module:\n\n```elixir\ndefmodule BankRouter do\n  use Commanded.Commands.Router\n\n  dispatch [OpenAccount,CloseAccount],\n    to: BankAccountHandler,\n    aggregate: BankAccount,\n    identity: :account_number\nend\n```","ref":"commands.html#multi-command-registration"},{"type":"extras","title":"Command dispatch consistency guarantee - Commands","doc":"You can choose the consistency guarantee when dispatching a command.\n\n- **Strong consistency** offers up-to-date data but at the cost of high latency.\n- **Eventual consistency** offers low latency but read model queries may reply with stale data since they may not have processed the persisted events.\n\nIn Commanded, the available options during command dispatch are:\n\n- `:eventual` (default) - don't block command dispatch and don't wait for any event handlers, regardless of their own consistency configuration.\n\n  ```elixir\n  :ok = BankApp.dispatch(command)\n  :ok = BankApp.dispatch(command, consistency: :eventual)\n  ```\n\n- `:strong` - block command dispatch until all strongly consistent event handlers and process managers have successfully processed all events created by the command.\n\n  ```elixir\n  :ok = BankApp.dispatch(command, consistency: :strong)\n  ```\n\n  Dispatching a command using `:strong` consistency but without any strongly consistent event handlers configured will have no effect.\n\n- Provide an explicit list of event handler and process manager modules (or their configured names), containing only those handlers you'd like to wait for. No other handlers will be awaited on, regardless of their own configured consistency setting.\n\n  ```elixir\n  :ok = BankApp.dispatch(command, consistency: [ExampleHandler, AnotherHandler])\n  :ok = BankApp.dispatch(command, consistency: [\"ExampleHandler\", \"AnotherHandler\"])\n  ```\n\n  Note you cannot opt-in to strong consistency for a handler that has been configured as eventually consistent.\n\n#### Which consistency guarantee should I use?\n\nWhen dispatching a command using `consistency: :strong` the dispatch will block until all of the strongly consistent event handlers and process managers have handled all events created by the command. This guarantees that when you receive the `:ok` response from dispatch, your strongly consistent read models will have been updated and can safely be queried.\n\nStrong consistency helps to alleviate problems and workarounds you would otherwise encounter when dealing with eventual consistency in your own application. Use `:strong` consistency when you want to query a read model immediately after dispatching a command. You **must** also configure the event handler to use `:strong` consistency.\n\nUsing `:eventual` consistency, or omitting the `consistency` option, will cause the command dispatch to immediately return without waiting for any event handlers or process managers. The handlers run independently, and asynchronously, in the background, therefore you will need to deal with potentially stale read model data.\n\n#### Configure default consistency\n\nYou may override the default consistency (`:eventual`) by setting `default_consistency` in your environment config (e.g. `config/config.exs`):\n\n```elixir\nconfig :commanded, default_consistency: :strong\n```\n\nThis will effect command dispatch, event handlers, and process managers where a consistency is not explicitly defined.\n\n#### Consistency failures\n\nBy opting-in to strong consistency you may encounter an additional error reply from command dispatch:\n\n```elixir\ncase BankApp.dispatch(command, consistency: :strong) do\n  :ok -> # ... all ok\n  {:error, :consistency_timeout} -> # command ok, handlers have not yet executed\nend\n```\n\nReceiving an `{:error, :consistency_timeout}` error indicates the command successfully dispatched, but some or all of the strongly consistent event handlers have not yet executed.\n\nThe default timeout is configured as five seconds; this determines how long the dispatch will block waiting for the handlers. You can override the default value in your environment config file (e.g. `config/config.exs`):\n\n```elixir\nconfig :commanded,\n  dispatch_consistency_timeout: 10_000  # ten seconds\n```","ref":"commands.html#command-dispatch-consistency-guarantee"},{"type":"extras","title":"Dispatch returning execution result - Commands","doc":"You can choose to include the execution result as part of the dispatch result by setting `include_execution_result` true:\n\n```elixir\n{\n  :ok,\n  %Commanded.Commands.ExecutionResult{\n    aggregate_uuid: aggregate_uuid,\n    aggregate_version: aggregate_version,\n    events: events,\n    metadata: metadata\n  }\n} = BankApp.dispatch(command, include_execution_result: true)\n```\n\nThis is useful if you need to get information from the events produced by the aggregate.","ref":"commands.html#dispatch-returning-execution-result"},{"type":"extras","title":"Dispatch returning aggregate version - Commands","doc":"You can optionally choose to include the aggregate's version as part of the dispatch result by setting the `include_aggregate_version` option to true:\n\n```elixir\n{:ok, aggregate_version} = BankApp.dispatch(command, include_aggregate_version: true)\n```\n\nThe returned `aggregate_version` can be used as an ETAG, allowing you to synchronize operations across the read-side and write-side of your application. For example, if need to wait for an event handler, such as a read model projection, to be up-to-date before continuing execution or querying its data.\n\n```elixir\ndefmodule BankAccountProjector do\n  use Commanded.Projections.Ecto,\n    application: BankApp,\n    name: \"BankAccountProjector\",\n    repo: BankApp.Repo\n\n  project(%BankAccountOpened{} = event, metadata, fn multi ->\n    multi\n    |> Ecto.Multi.run(:bank_account, &find_bank_account(&1, &2, event.id))\n    |> Ecto.Multi.update(\n      :updated_bank_account,\n      &BankAccount.changeset(&1.bank_account, %{\n        # Notice that I am using the stream version from the metadata\n        # to update the auction's stream version.\n        stream_version: metadata.stream_version,\n      })\n    )\n  end)\nend\n```\n\nThen you can use the aggregate version to wait for the event handler to be up-to-date:\n\n```elixir\ndefmodule BankAccounts do\n  def get_bank_account(id, stream_version) do\n    query =\n      from b in BankAccount,\n        where: b.id == ^id,\n        where: b.stream_version >= ^stream_version,\n        order_by: [asc: b.stream_version]\n\n    case BankApp.one(query) do\n      nil ->\n        # not ready yet means the projection is not yet up-to-date\n        # so you can retry the query\n        {:error, :not_ready}\n\n      bank_account ->\n        {:ok, bank_account}\n    end\n  end\nend\n```","ref":"commands.html#dispatch-returning-aggregate-version"},{"type":"extras","title":"Aggregate version as an ETAG - Commands","doc":"The aggregate version can be thought of as an ETAG (Entity Tag) for a given resource. ETAGs are commonly used in HTTP caching mechanisms to determine if a resource has changed. Similarly, the aggregate version serves as a unique identifier for a specific state of the aggregate.\nWhen building a REST API, you can use the aggregate version as the ETAG in your HTTP responses.","ref":"commands.html#aggregate-version-as-an-etag"},{"type":"extras","title":"Causation and correlation ids - Commands","doc":"To assist with monitoring and debugging your deployed application it is useful to track the causation and correlation ids for your commands and events.\n\n- `causation_id` - the UUID of the command causing an event, or the event causing a command dispatch.\n- `correlation_id` - a UUID used to correlate related commands/events.\n\nYou can set causation and correlation ids when dispatching a command:\n\n```elixir\n:ok = BankApp.dispatch(command, causation_id: Commanded.UUID.uuid4(), correlation_id: Commanded.UUID.uuid4())\n```\n\nWhen dispatching a command in an event handler, you should copy these values from the metadata (second) argument associated with the event you are handling:\n\n```elixir\ndefmodule ExampleHandler do\n  use Commanded.Event.Handler,\n    application: ExampleApp,\n    name: \"ExampleHandler\"\n\n  def handle(%AnEvent{..}, %{event_id: causation_id, correlation_id: correlation_id}) do\n    command = %ExampleCommand{..}\n\n    BankApp.dispatch(command,\n      causation_id: causation_id,\n      correlation_id: correlation_id,\n    )\n  end\nend\n```\n\nCommands dispatched by a process manager will be automatically assigned the appropriate causation and correlation ids from the source domain event.\n\nYou can use [Commanded audit middleware](https://hex.pm/packages/commanded_audit_middleware) to record every dispatched command. This allows you to follow the chain of commands and events by using the causation id. The correlation id can be used to find all related commands and events.","ref":"commands.html#causation-and-correlation-ids"},{"type":"extras","title":"Event metadata - Commands","doc":"It's helpful for debugging to have additional metadata associated with events issued by a command. You can set it when dispatching a command:\n\n```elixir\n:ok = BankApp.dispatch(command, metadata: %{\"issuer_id\" => issuer_id, \"user_id\" => \"user@example.com\"})\n```\n\nNote, due to metadata serialization you should expect that only: strings, numbers, and boolean values are preserved; any other value will be converted to a string.\n\nYou should always use string keys in your metadata map; atom keys will be converted to strings.","ref":"commands.html#event-metadata"},{"type":"extras","title":"Aggregate lifespan - Commands","doc":"By default an aggregate instance process will run indefinitely once started. You can control this by implementing the `Commanded.Aggregates.AggregateLifespan` behaviour in a module.\n\nDefine a module that implements the `Commanded.Aggregates.AggregateLifespan` behaviour:\n\n```elixir\ndefmodule BankAccountLifespan do\n  @behaviour Commanded.Aggregates.AggregateLifespan\n\n  def after_event(%MoneyDeposited{}), do: :timer.hours(1)\n  def after_event(%BankAccountClosed{}), do: :stop\n  def after_event(_event), do: :infinity\n\n  def after_command(%CloseAccount{}), do: :stop\n  def after_command(_command), do: :infinity\n\n  def after_error(:invalid_initial_balance), do: :timer.minutes(5)\n  def after_error(_error), do: :stop\nend\n```\n\nThen specify the module as the `lifespan` option when registering the command in the router.\n\n```elixir\ndefmodule BankRouter do\n  use Commanded.Commands.Router\n\n  dispatch [OpenAccount,CloseAccount],\n    to: BankAccount,\n    lifespan: BankAccountLifespan,\n    identity: :account_number\nend\n```\n\nThe inactivity timeout is specified in milliseconds, after which time the aggregate process will be stopped if no other messages are received by it.\n\nReturn `:stop` or `{:stop, reason}` to immediately shutdown the aggregate process. Return `:infinity` to prevent the aggregate instance from shutting down.\n\nYou can also return `:hibernate` and the process is hibernated, it will continue its loop once a message is in its message queue. Hibernating an aggregate causes garbage collection and minimises the memory used by the process. Hibernating should not be used aggressively as too much time could be spent garbage collecting.","ref":"commands.html#aggregate-lifespan"},{"type":"extras","title":"Middleware - Commands","doc":"Allows a command router to define middleware modules that are executed before and after success or failure of each command dispatch.\n\nThis provides an extension point to add in command validation, authorization, logging, and other cross-cutting behaviour that you want to be called for every command the router dispatches.\n\n```elixir\ndefmodule BankingRouter do\n  use Commanded.Commands.Router\n\n  middleware CommandLogger\n  middleware MyCommandValidator\n  middleware AuthorizeCommand\n\n  identify BankAccount, by: :account_number\n\n  dispatch OpenAccount, to: OpenAccountHandler, aggregate: BankAccount\n  dispatch DepositMoney, to: DepositMoneyHandler, aggregate: BankAccount\nend\n```\n\nThe middleware modules are executed in the order they’ve been defined. They will receive a `Commanded.Middleware.Pipeline` struct containing the command being dispatched, in the `:command` field.","ref":"commands.html#middleware"},{"type":"extras","title":"Example middleware - Commands","doc":"Implement the `Commanded.Middleware` behaviour in your module and define the `before_dispatch`, `after_dispatch`, and `after_failure` callback functions.\n\n```elixir\ndefmodule NoOpMiddleware do\n  @behaviour Commanded.Middleware\n\n  alias Commanded.Middleware.Pipeline\n  import Pipeline\n\n  def before_dispatch(%Pipeline{} = pipeline) do\n    pipeline\n  end\n\n  def after_dispatch(%Pipeline{} = pipeline) do\n    pipeline\n  end\n\n  def after_failure(%Pipeline{} = pipeline) do\n    pipeline\n  end\nend\n```\n\nCommanded provides a `Commanded.Middleware.Logger` middleware for logging the name of each dispatched command and its execution duration.","ref":"commands.html#example-middleware"},{"type":"extras","title":"Composite command routers - Commands","doc":"You can use the `Commanded.Commands.CompositeRouter` macro to define a router comprised of other router modules. This approach is useful if you prefer to construct a router per context and then combine them together to form a top level application router.\n\nBy using `Commanded.Commands.CompositeRouter` in your own module you can include other routers via the `router` macro:\n\n```elixir\ndefmodule ApplicationRouter do\n  use Commanded.Commands.CompositeRouter\n\n  router BankAccountRouter\n  router MoneyTransferRouter\nend\n```\n\n```elixir\ndefmodule BankApp do\n  use Commanded.Application, otp_app: :bank_app\n\n  router ApplicationRouter\nend\n```\n\nCommand dispatch works the same as any other router:\n\n```elixir\n:ok = BankApp.dispatch(%OpenAccount{account_number: \"ACC123\", initial_balance: 1_000})\n```","ref":"commands.html#composite-command-routers"},{"type":"extras","title":"Events","doc":"# Events","ref":"events.html"},{"type":"extras","title":"Domain events - Events","doc":"Domain events indicate that something of importance has occurred, within the context of an aggregate. They **should** be named in the past tense: account registered; funds transferred; fraudulent activity detected etc.\n\nCreate a module per domain event and define the fields with `defstruct`. An event **should contain** a field to uniquely identify the aggregate instance (e.g. `account_number`).\n\nRemember to derive the `Jason.Encoder` protocol for the event struct to ensure JSON serialization is supported, as shown below.\n\n```elixir\ndefmodule BankAccountOpened do\n  @derive Jason.Encoder\n  defstruct [:account_number, :initial_balance]\nend\n```\n\nNote, due to event serialization you should expect that only: strings, numbers and boolean values defined in an event are preserved; any other value will be converted to a string. You can control this behaviour as described in the [Serialization guide](https://hexdocs.pm/commanded/serialization.html).","ref":"events.html#domain-events"},{"type":"extras","title":"Event handlers - Events","doc":"Event handlers allow you to execute code that reacts to domain events: to build read model projections; dispatch commands to other aggregates; and to interact with third-party systems such as sending emails.\n\nCommanded guarantees only one instance of an event handler will run, regardless of how many nodes are running (even when not using distributed Erlang). This is enforced by the event store subscription (PostgreSQL advisory locks in Elixir Event Store).\n\nUse the `Commanded.Event.Handler` macro within your event handler module to implement the defined behaviour. This consists of a single `handle/2` function that receives each published domain event and its metadata, including the event's unique event number. It should return `:ok` on success or `{:error, :reason}` on failure. You can return `{:error, :already_seen_event}` to skip events that have already been handled, due to the at-least-once event delivery of the supported event stores.\n\nUse pattern matching to match on each type of event you are interested in. A catch-all `handle/2` function is included, so all other events will be ignored by default.\n\n```elixir\ndefmodule ExampleHandler do\n  use Commanded.Event.Handler,\n    application: ExampleApp,\n    name: \"ExampleHandler\"\n\n  @impl Commanded.Event.Handler\n  def handle(%AnEvent{..}, _metadata) do\n    # ... process the event\n    :ok\n  end\nend\n```\n\nThe name given to the event handler **must be** unique and remain unchanged between releases. It is used when subscribing to the event store to track which events the handler has seen during restarts.\n\n```elixir\n{:ok, _handler} = ExampleHandler.start_link()\n```","ref":"events.html#event-handlers"},{"type":"extras","title":"Subscription options - Events","doc":"You can choose to start the event handler's event store subscription from the `:origin`, `:current` position, or an exact event number using the `start_from` option. The default is to use the origin so your handler will receive all events.\n\n```elixir\ndefmodule ExampleHandler do\n  # Define `start_from` as one of :origin, :current, or an explicit event number (e.g. 1234)\n  use Commanded.Event.Handler,\n    application: ExampleApp,\n    name: \"ExampleHandler\",\n    start_from: :origin\nend\n```\n\nYou can optionally override `:start_from` by passing it as param:\n\n```elixir\n{:ok, _handler} = ExampleHandler.start_link(start_from: :current)\n```\n\nUse the `:current` position when you don't want newly created event handlers to go through all previous events. An example would be adding an event handler to send transactional emails to an already deployed system containing many historical events.\n\nYou should start your event handlers using an OTP `Supervisor` to ensure they are restarted on error. See the [Supervision guide](https://hexdocs.pm/commanded/supervision.html) for more details.","ref":"events.html#subscription-options"},{"type":"extras","title":"Configuration options - Events","doc":"You can choose the default error behaviour for *all* of your event handlers in each Application's configuration:\n\n```ellxir\nconfig :example, ExampleApp,\n  on_event_handler_error: :stop # or :backoff or MyCustomErrorHandler\n```\n\nThe default behaviour is to stop the event handler process when any error is encountered. As event handlers are supervised either by a custom supervisor or by the application itself, the handlers are usually restarted right away. If the error is permanent, due to a logic or data bug, then the process will likely crash again right away. This can lead the supervisor itself to give up, crash and this will continue up your supervision tree until it stops your application.\n\nThe `:backoff` option, introduced in v1.4.7, cause the even handler to retry after an exponentially increasing backoff period (up to a maximum of 24 hours). The event handler will still not be able to make forward progress until you address the issue, but it won't take your supervisors or applications down with it.\n\nIf you want to provide your own strategy, you can pass in a module that implements the `c:Commanded.Event.Handler.error/3` callback.\n\nIt's important to note that if your event handler overrides the `error/3` callback, then that will be called instead of the application-wide strategy.","ref":"events.html#configuration-options"},{"type":"extras","title":"Subscribing to an individual stream - Events","doc":"By default event handlers will subscribe to all events appended to any stream. Provide a `subscribe_to` option to subscribe to a single stream.\n\n```elixir\ndefmodule ExampleHandler do\n  use Commanded.Event.Handler,\n    application: ExampleApp,\n    name: __MODULE__,\n    subscribe_to: \"stream1234\"\nend\n```\n\nThis will ensure the handler only receives events appended to that stream.","ref":"events.html#subscribing-to-an-individual-stream"},{"type":"extras","title":"Handling errors - Events","doc":"It is important to consider how errors in the handling of events will affect your application. By default any errors encountered are configured to stop the event handler immediately. For this reason it is vital to have handlers under supervision. Errors in handlers occur in two categories:\n\n#### Transient errors\n\nTransient errors occur due temporary issues such as network connectivity, flakey downstream services, memory, or disk issues. Retrying here will often solve the problem.\n\n#### Permanent errors\n\nPermanent errors occure due to bugs in code, and as such have no hope of resolving themselves. The handler will not be able to make any progress, and will repeatedly crash.\n\nRepeated crashes become a problem when the handler crashes more than it's supervisor is configured to tolerate. When that happens, the supervisor itself will crash and this process will continue upwards until finally the application itself crashes. This is obviously bad.\n\nYou can opt to have your event handlers backoff when crashing to avoid this:\n\n```elixir\nconfig :my_app, MyApp.CommandedApp,\n  on_event_handler_error: :backoff\n```\n\nThis will cause all event handlers from `MyApp.CommandedApp` to back off exponentially when they encounter an error. Errors will backoff to maximum of 24 hours, and will continue at that rate until the issue is resolved. The event handler still can't make any forward progress, but it will at least not bring down your application.\n\nIt is also possible to configure the error handler for the default behaviour `:stop`.\n\nYou can bring your own Commanded application-level behaviour by specifiying a module which implements `c:Commanded.Event.Handler.error/3`\n\n```elixir\ndefmodule YoloErrorHandler do\n  def error(error, failing_event, failure_context) do\n    # Ignore all errors!\n    :skip\n  end\nend\n\nconfig :my_app, MyApp.CommandedApp,\n  on_event_handler_error: YoloErrorHandler\n```","ref":"events.html#handling-errors"},{"type":"extras","title":"Event handler callbacks - Events","doc":"- `c:Commanded.Event.Handler.init/1` - (optional) used to configure the handler before it starts.\n- `c:Commanded.Event.Handler.after_start/1` - (optional) initialisation callback function called in the process of the started handler.\n- `c:Commanded.Event.Handler.error/3` - (optional) called when an event handle/2 callback returns an error.\n\nError event handlers configured on a per handler basis like this will override the application level error handling.","ref":"events.html#event-handler-callbacks"},{"type":"extras","title":"Metadata - Events","doc":"The `handle/2` function in your handler receives the domain event and a map of metadata associated with that event. You can provide the metadata key/value pairs when dispatching a command:\n\n```elixir\n:ok = ExampleApp.dispatch(command, metadata: %{\"issuer_id\" => issuer_id, \"user_id\" => \"user@example.com\"})\n```\n\nIn addition to the metadata key/values you provide, the following system values will be included in the metadata passed to an event handler:\n\n- `application` - the `Commanded.Application` associated with the event handler.\n- `handler_name` - the name of the event handler.\n- `state` - optional event handler state.\n- `event_id` - a globally unique UUID to identify the event.\n- `event_number` - a globally unique, monotonically incrementing and gapless integer used to order the event amongst all events.\n- `stream_id` - the stream identity for the event.\n- `stream_version` - the version of the stream for the event.\n- `causation_id` - an optional UUID identifier used to identify which command caused the event.\n- `correlation_id` - an optional UUID identifier used to correlate related commands/events.\n- `created_at` - the datetime, in UTC, indicating when the event was created.\n\nThese key/value metadata pairs will use atom keys to differentiate them from the user provided metadata:\n\n```elixir\ndefmodule ExampleHandler do\n  use Commanded.Event.Handler,\n    application: ExampleApp,\n    name: \"ExampleHandler\"\n\n  @impl Commanded.Event.Handler\n  def handle(event, metadata) do\n    IO.inspect(metadata)\n    # %{\n    #   :causation_id => \"db1ebd30-7d3c-40f7-87cd-12cd9966df32\",\n    #   :correlation_id => \"1599630b-9c38-433c-9548-0dd793108ba0\",\n    #   :created_at => #DateTime<2017-10-30 11:19:56.178901Z>,\n    #   :event_id => \"5e4a0f38-385b-4d57-823b-a1bcf705b7bb\",\n    #   :event_number => 12345,\n    #   :stream_id => \"e42a588d-2cda-4314-a471-5d008cce01fc\",\n    #   :stream_version => 1,\n    #   \"issuer_id\" => \"0768d69a-d2b7-48f4-d0e9-083a97f7ebe0\",\n    #   \"user_id\" => \"user@example.com\"\n    # }\n\n    :ok\n  end\nend\n```","ref":"events.html#metadata"},{"type":"extras","title":"Consistency guarantee - Events","doc":"You can specify an event handler's consistency guarantee using the `consistency` option:\n\n```elixir\ndefmodule ExampleHandler do\n  use Commanded.Event.Handler,\n    application: ExampleApp,\n    name: \"ExampleHandler\",\n    consistency: :eventual\n```\n\nThe available options are `:eventual` (default) and `:strong`:\n\n- *Strong consistency* offers up-to-date data but at the cost of high latency.\n- *Eventual consistency* offers low latency but read model queries may reply with stale data since they may not have processed the persisted events.\n\nYou request the consistency guarantee, either `:strong` or `:eventual`, when dispatching a command. Strong consistency will block the command dispatch and wait for all strongly consistent event handlers to successfully process all events created by the command. Whereas eventual consistency will immediately return after command dispatch, without waiting for any event handlers, even those configured for strong consistency.","ref":"events.html#consistency-guarantee"},{"type":"extras","title":"How does it work? - Events","doc":"An event handler is a `GenServer` process that subscribes to the configured event store. For each event persisted to the store the `handle/2` callback is called, passing the domain event and its metadata.","ref":"events.html#how-does-it-work"},{"type":"extras","title":"Upcasting events - Events","doc":"Commanded supports upcasting of events at runtime using the `Commanded.Event.Upcaster` protocol.\n\nBy implementing the upcaster protocol you can transform an event before it is used by a consumer. This might be an aggregate, an event handler, or a process manager. Because the upcaster changes the event at runtime, handlers only need to support the latest version. You can also use upcasting to change the type of event.","ref":"events.html#upcasting-events"},{"type":"extras","title":"Examples - Events","doc":"Change the shape of an event by renaming a field:\n\n```elixir\ndefimpl Commanded.Event.Upcaster, for: AnEvent do\n  def upcast(%AnEvent{} = event, _metadata) do\n    %AnEvent{name: name} = event\n\n    %AnEvent{event | first_name: name}\n  end\nend\n```\n\nChange the type of event by replacing a historical event with a new event:\n\n```elixir\ndefimpl Commanded.Event.Upcaster, for: HistoricalEvent do\n  def upcast(%HistoricalEvent{} = event, _metadata) do\n    %HistoricalEvent{id: id, name: name} = event\n\n    %NewEvent{id: id, name: name}\n  end\nend\n```","ref":"events.html#examples"},{"type":"extras","title":"Reset an EventHandler - Events","doc":"An event handler can be reset (using a mix task), it will restart the event store subscription from the configured\n`start_from`. This allow an individual handler to be restart while the app is still running.\n\nYou can implement the `before_reset/0` callback that will be called before resetting the event handler.\n\n```elixir\ndefmodule ExampleHandler do\n  use Commanded.Event.Handler,\n    application: ExampleApp,\n    name: __MODULE__\n\n  require Logger\n\n  alias Commanded.Event.FailureContext\n\n  @impl Commanded.Event.Handler\n  def before_reset do\n    # Do something\n    :ok\n  end\nend\n```","ref":"events.html#reset-an-eventhandler"},{"type":"extras","title":"Process managers","doc":"# Process managers\n\nA process manager is responsible for coordinating one or more aggregates. It handles events and dispatches commands in response. You can think of a process manager as the opposite of an aggregate: aggregates handle commands and create events; process managers handle events and create commands. Process managers have state that can be used to track which aggregates are being orchestrated.\n\nUse the `Commanded.ProcessManagers.ProcessManager` macro in your process manager module and implement the callback functions defined in the behaviour: `c:Commanded.ProcessManagers.ProcessManager.interested?/1`, `c:Commanded.ProcessManagers.ProcessManager.handle/2`, `c:Commanded.ProcessManagers.ProcessManager.apply/2`, and `c:Commanded.ProcessManagers.ProcessManager.error/3`.\n\n## `interested?/1`\n\nThe `c:Commanded.ProcessManagers.ProcessManager.interested?/1` function is used to indicate which events the process manager handles. The response is used to route the event to an existing instance or start a new process instance:\n\n- `{:start, process_uuid}` - create a new instance of the process manager.\n- `{:start!, process_uuid}` - create a new instance of the process manager (strict).\n- `{:continue, process_uuid}` - continue execution of an existing process manager.\n- `{:continue!, process_uuid}` - continue execution of an existing process manager (strict).\n- `{:stop, process_uuid}` - stop an existing process manager, shutdown its\n  process, and delete its persisted state.\n- `false` - ignore the event.\n\nYou can return a list of process identifiers when a single domain event is to be handled by multiple process instances.","ref":"process-managers.html"},{"type":"extras","title":"Strict process routing - Process managers","doc":"Using strict routing, with `:start!` or `:continue!`, enforces the following validation checks:\n\n- `{:start!, process_uuid}` - validate process does not already exist.\n- `{:continue!, process_uuid}` - validate process already exists.\n\nIf the check fails an error will be passed to the `error/3` callback function:\n\n- `{:error, {:start!, :process_already_started}}`\n- `{:error, {:continue!, :process_not_started}}`\n\nThe `c:Commanded.ProcessManagers.ProcessManager.error/3` function can choose to `:stop` the process or `:skip` the problematic event.\n\n## `handle/2`\n\nA `handle/2` function can be defined for each `:start` and `:continue` tagged event previously specified. It receives the process manager's state and the event to be handled. It must return the commands to be dispatched. This may be none, a single command, or many commands.\n\nThe `handle/2` function can be omitted if you do not need to dispatch a command and are only mutating the process manager's state.\n\n## `apply/2`\n\nThe `c:Commanded.ProcessManagers.ProcessManager.apply/2` function is used to mutate the process manager's state. It receives the current state and the domain event, and must return the modified state.\n\nThis callback function is optional, the default behaviour is to retain the process manager's current state.\n\n## `error/3`\n\nYou can define an `c:Commanded.ProcessManagers.ProcessManager.error/3` callback function to handle any errors or exceptions during event handling or returned by commands dispatched from your process manager. The function is passed the error (e.g. `{:error, :failure}`), the failed event or command, and a failure context. See `Commanded.ProcessManagers.FailureContext` for details.\n\nUse pattern matching on the error and/or failed event/command to explicitly handle certain errors, events, or commands. You can choose to retry, skip, ignore, or stop the process manager after a command dispatch error.\n\nThe default behaviour, if you don't provide an `c:Commanded.ProcessManagers.ProcessManager.error/3` callback, is to stop the process manager using the exact error reason returned from the event handler function or command dispatch.\n\nThe `c:Commanded.ProcessManagers.ProcessManager.error/3` callback function must return one of the following responses depending upon the severity of error and how you choose to handle it:\n\n- `{:retry, context}` - retry the failed command, provide a context map containing any state passed to subsequent failures. This could be used to count the number of retries, failing after too many attempts.\n\n- `{:retry, delay, context}` - retry the failed command, after sleeping for the requested delay (in milliseconds). Context is a map as described in `{:retry, context}` above.\n\n- `{:stop, reason}` - stop the process manager with the given reason.\n\nFor event handling failures, when failure source is an event, you can also return:\n\n- `:skip` - to skip the problematic event. No commands will be dispatched.\n\nFor command dispatch failures, when failure source is a command, you can also return:\n\n- `{:continue, commands, context}` - continue dispatching the given commands. This allows you to retry the failed command, modify it and retry, drop it, or drop all pending commands by passing an empty list `[]`.\n\n- `{:skip, :discard_pending}` - discard the failed command and any pending commands.\n\n- `{:skip, :continue_pending}` - skip the failed command, but continue dispatching any pending commands.","ref":"process-managers.html#strict-process-routing"},{"type":"extras","title":"Supervision - Process managers","doc":"Supervise your process managers to ensure they are restarted on error.\n\n```elixir\ndefmodule Bank.Payments.Supervisor do\n  use Supervisor\n\n  def start_link(_arg) do\n    Supervisor.start_link(__MODULE__, [], name: __MODULE__)\n  end\n\n  def init(_arg) do\n    Supervisor.init(\n      [\n        Bank.Payments.TransferMoneyProcessManager\n      ],\n      strategy: :one_for_one\n    )\n  end\nend\n```","ref":"process-managers.html#supervision"},{"type":"extras","title":"Supervision caveats - Process managers","doc":"The default error handling strategy is to stop the process manager. When supervised, the process will be restarted and will attempt to handle the same event again, which will likely result in the same error. This could lead to too many restarts of the supervisor, which may eventually cause the application to stop, depending upon your supervision tree and its strategy.\n\nTo prevent this you can choose to define the default error handling strategy as `:skip` to skip over any problematic events.\n\n```elixir\ndefmodule ExampleProcessManager do\n  use Commanded.ProcessManagers.ProcessManager,\n    application: ExampleApp,\n    name: __MODULE__\n\n  require Logger\n\n  # By default skip any problematic events\n  def error(error, _command_or_event, _failure_context) do\n    Logger.error(fn -> \"#{__MODULE__} encountered an error: \" <> inspect(error) end)\n\n    :skip\n  end\nend\n```\n\nAlternatively you can define the restart strategy of your process manager as `:temporary` to prevent it from being restarted on termination. This approach will require manual intervention to fix the stopped process manager, but ensures that it won't miss any events nor crash the application.","ref":"process-managers.html#supervision-caveats"},{"type":"extras","title":"Error handling example - Process managers","doc":"Define an `error/3` callback function to determine how to handle errors during event handling and command dispatch.\n\n```elixir\ndefmodule ExampleProcessManager do\n  use Commanded.ProcessManagers.ProcessManager,\n    application: ExampleApp,\n    name: \"ExampleProcessManager\"\n\n  # Stop process manager after three failures\n  def error({:error, _failure}, _failed_message, %{context: %{failures: failures}})\n    when failures >= 2\n  do\n    {:stop, :too_many_failures}\n  end\n\n  # Retry command, record failure count in context map\n  def error({:error, _failure}, _failed_message, %{context: context}) do\n    context = Map.update(context, :failures, 1, fn failures -> failures + 1 end)\n    {:retry, context}\n  end\nend\n```\n\nThe default behaviour if you don't provide an `error/3` callback is to stop the process manager using the same error reason returned from the failed command dispatch.","ref":"process-managers.html#error-handling-example"},{"type":"extras","title":"Example process manager - Process managers","doc":"```elixir\ndefmodule TransferMoneyProcessManager do\n  use Commanded.ProcessManagers.ProcessManager,\n    application: ExampleApp,\n    name: \"TransferMoneyProcessManager\"\n\n  @derive Jason.Encoder\n  defstruct [\n    :transfer_uuid,\n    :debit_account,\n    :credit_account,\n    :amount,\n    :status\n  ]\n\n  # Process routing\n\n  def interested?(%MoneyTransferRequested{transfer_uuid: transfer_uuid}), do: {:start, transfer_uuid}\n  def interested?(%MoneyWithdrawn{transfer_uuid: transfer_uuid}), do: {:continue, transfer_uuid}\n  def interested?(%MoneyDeposited{transfer_uuid: transfer_uuid}), do: {:stop, transfer_uuid}\n  def interested?(_event), do: false\n\n  # Command dispatch\n\n  def handle(%TransferMoneyProcessManager{}, %MoneyTransferRequested{} = event) do\n    %MoneyTransferRequested{transfer_uuid: transfer_uuid, debit_account: debit_account, amount: amount} = event\n\n    %WithdrawMoney{account_number: debit_account, transfer_uuid: transfer_uuid, amount: amount}\n  end\n\n  def handle(%TransferMoneyProcessManager{} = pm, %MoneyWithdrawn{}) do\n    %TransferMoneyProcessManager{transfer_uuid: transfer_uuid, credit_account: credit_account, amount: amount} = pm\n\n    %DepositMoney{account_number: credit_account, transfer_uuid: transfer_uuid, amount: amount}\n  end\n\n  # State mutators\n\n  def apply(%TransferMoneyProcessManager{} = transfer, %MoneyTransferRequested{} = event) do\n    %MoneyTransferRequested{transfer_uuid: transfer_uuid, debit_account: debit_account, credit_account: credit_account, amount: amount} = event\n\n    %TransferMoneyProcessManager{transfer |\n      transfer_uuid: transfer_uuid,\n      debit_account: debit_account,\n      credit_account: credit_account,\n      amount: amount,\n      status: :withdraw_money_from_debit_account\n    }\n  end\n\n  def apply(%TransferMoneyProcessManager{} = transfer, %MoneyWithdrawn{}) do\n    %TransferMoneyProcessManager{transfer |\n      status: :deposit_money_in_credit_account\n    }\n  end\nend\n```\n\nThe name given to the process manager *must* be unique. This is used when subscribing to events from the event store to track the last seen event and ensure they are only received once.\n\n```elixir\n{:ok, _pid} = TransferMoneyProcessManager.start_link(start_from: :current)\n```\n\nYou can choose to start the process router's event store subscription from the `:origin`, `:current` position or an exact event number using the `start_from` option. The default is to use `:origin` so it will receive all events. You typically use `:current` when adding a new process manager to an already deployed system containing historical events.\n\nProcess manager instance state is persisted to storage after each handled event. This allows the process manager to resume should the host process terminate.","ref":"process-managers.html#example-process-manager"},{"type":"extras","title":"Configuration options - Process managers","doc":"- `consistency` - defined as one of either `:strong` or `:eventual` (default) for event handling.\n- `event_timeout` - a timeout for event handling to ensure that events are processed in a timely manner without getting stuck.\n- `idle_timeout` - to reduce memory usage you can configure an idle timeout, in milliseconds, after which an inactive process instance will be shutdown.\n\nRefer to the `Commanded.ProcessManagers.ProcessManager` module docs for more details.","ref":"process-managers.html#configuration-options"},{"type":"extras","title":"Supervision","doc":"# Supervision\n\nUse an OTP supervisor to host your Commanded application, process managers, event handlers, and read model projectors.\n\n```elixir\ndefmodule Bank.Supervisor do\n  use Supervisor\n\n  def start_link(arg) do\n    Supervisor.start_link(__MODULE__, arg, name: __MODULE__)\n  end\n\n  @impl true\n  def init(_arg) do\n    children = [\n      # Application\n      BankApp,\n\n      # Event handler\n      AccountBalanceHandler,\n\n      # Process manager\n      TransferMoneyProcessManager,\n\n      # Read model projector\n      AccountsProjector,\n\n      # Optionally, provide runtime configuration\n      {WelcomeEmailHandler, start_from: :current},\n    ]\n\n    Supervisor.init(children, strategy: :one_for_one)\n  end\nend\n```\n\nYour application should start the supervisor:\n\n```elixir\ndefmodule Bank do\n  use Application\n\n  def start(_type, _args) do\n    Bank.Supervisor.start_link()\n  end\nend\n```","ref":"supervision.html"},{"type":"extras","title":"Serialization","doc":"# Serialization","ref":"serialization.html"},{"type":"extras","title":"Default JSON serializer - Serialization","doc":"JSON serialization can be used for event data & metadata, and aggregate and process manager snapshots.\n\nTo enable JSON serialization with the included `Commanded.Serialization.JsonSerializer` module add the `jason` library to your deps:\n\n```elixir\ndef deps do\n  [{:jason, \"~> 1.2\"}]\nend\n```\n\n[Jason](https://hex.pm/packages/jason), a pure Elixir JSON library, is used for the actual serialization.\n\nYou must derive the `Jason.Encoder` protocol for all structs you plan on encoding.\n\n```elixir\ndefmodule ExampleEvent do\n  @derive Jason.Encoder\n  defstruct [:name, :datetime]\nend\n```\n\nJason provides an extension point if you need to manually encode your event by using the `Jason.Encoder` protocol:\n\n```elixir\ndefimpl Jason.Encoder, for: Person do\n  def encode(%{name: name, age: age}, opts) do\n    Jason.Encode.string(\"#{name} (#{age})\", options)\n  end\nend\n```","ref":"serialization.html#default-json-serializer"},{"type":"extras","title":"Configuring JSON serialization - Serialization","doc":"JSON serialization configuration depends upon which event store you are using with Commanded. Configure the serializer in `config/config.exs`, or per environment, as follows.\n\n- Postgres EventStore:\n\n  ```elixir\n  config :my_app, MyApp.EventStore,\n    serializer: Commanded.Serialization.JsonSerializer\n  ```\n\n  You can also use the `EventStore.JsonSerializer` (`bytea` column type) and `EventStore.JsonbSerializer` (`jsonb` column type) serializers which are included in the EventStore library.\n\n  ```elixir\n  config :my_app, MyApp.EventStore,\n    serializer: EventStore.JsonSerializer\n  ```\n\n  ```elixir\n  config :my_app, MyApp.EventStore,\n    serializer: EventStore.JsonbSerializer\n  ```\n\n  Note that the two EventStore serializers do not implement the `Commanded.Serialization.JsonSerializer` decoding protocol.\n\n- EventStoreDB:\n\n  ```elixir\n  config :commanded_extreme_adapter,\n    serializer: Commanded.Serialization.JsonSerializer\n  ```\n\n- In-memory event store:\n\n  ```elixir\n  config :commanded, Commanded.EventStore.Adapters.InMemory,\n    serializer: Commanded.Serialization.JsonSerializer\n  ```","ref":"serialization.html#configuring-json-serialization"},{"type":"extras","title":"Decoding event structs - Serialization","doc":"The `Commanded.Serialization.JsonSerializer` module provides an extension point to allow additional decoding of the deserialized data. This can be used for parsing data into valid types, such as datetime parsing from a string.\n\nThe example event below has an implementation of the `Commanded.Serialization.JsonDecoder` protocol to parse the datetime into a `DateTime` struct.\n\n```elixir\ndefmodule ExampleEvent do\n  @derive Jason.Encoder\n  defstruct [:name, :datetime]\nend\n\ndefimpl Commanded.Serialization.JsonDecoder, for: ExampleEvent do\n  @doc \"\"\"\n  Parse the datetime included in the event.\n  \"\"\"\n  def decode(%ExampleEvent{datetime: datetime} = event) do\n    {:ok, dt, _} = DateTime.from_iso8601(datetime)\n\n    %ExampleEvent{event | datetime: dt}\n  end\nend\n```\n\nThe above protocol is *only* supported by the `Commanded.Serialization.JsonSerializer` serializer.","ref":"serialization.html#decoding-event-structs"},{"type":"extras","title":"Using an alternative serialization format - Serialization","doc":"You can implement your own serializer module to use an alternative serialization format if preferred.\n\nConfigure your own serializer in `config/config.exs` for the event store you are using.\n\n- PostgreSQL EventStore:\n\n  ```elixir\n  config :my_app, MyApp.EventStore, serializer: MyApp.MessagePackSerializer\n  ```\n\n- EventStoreDB:\n\n  ```elixir\n  config :my_app, MyApp, serializer: MyApp.MessagePackSerializer\n  ```\n\n- In-memory event store:\n\n  ```elixir\n  config :my_app, MyApp,\n    event_store: [\n      adapter: Commanded.EventStore.Adapters.InMemory,\n      serializer: MyApp.MessagePackSerializer\n    ]\n  ```\n\nYou *should not* change serialization format once your app has been deployed to production since Commanded will not be able to deserialize any existing events or snapshot data. In this scenario, to change serialization format you would need to also migrate your event store to the new format.","ref":"serialization.html#using-an-alternative-serialization-format"},{"type":"extras","title":"Customising serialization - Serialization","doc":"To implement custom serialization you could implement your own serializer which extends the behaviour of the existing `Commanded.Serialization.JsonSerializer` module.\n\nThe example below shows how you might compress the serialized data before storage and decompress during deserialization.\n\n```elixir\ndefmodule CompressedJsonSerializer do\n  def serialize(term) do\n    term\n    |> Commanded.Serialization.JsonSerializer.serialize()\n    |> compress()\n  end\n\n  def deserialize(binary, config \\\\ []) do\n    binary\n    |> decompress()\n    |> Commanded.Serialization.JsonSerializer.deserialize(config)    \n  end\n\n  defp compress(term), do: ...\n  defp decompress(binary), do: ...\nend\n```","ref":"serialization.html#customising-serialization"},{"type":"extras","title":"Read model projections","doc":"# Read model projections\n\nYour read model can be built using a Commanded event handler and whichever storage provider you prefer. You can choose to use a SQL or NoSQL database, document store, the filesystem, a full text search index, or any other storage mechanism. You may even use multiple storage providers, optimised for the querying they must support.","ref":"read-model-projections.html"},{"type":"extras","title":"Ecto projections - Read model projections","doc":"You can use the [Commanded Ecto projections](https://github.com/commanded/commanded-ecto-projections) library to build a read model using one of the databases supported by Ecto (PostgreSQL, MySQL, et al).","ref":"read-model-projections.html#ecto-projections"},{"type":"extras","title":"Example - Read model projections","doc":"```elixir\ndefmodule MyApp.ExampleProjector do\n  use Commanded.Projections.Ecto,\n    application: MyApp.ExampleApp,\n    name: \"ExampleProjector\"\n\n  project %AnEvent{name: name}, _metadata do\n    Ecto.Multi.insert(multi, :example_projection, %ExampleProjection{name: name})\n  end\n\n  project %AnotherEvent{name: name} do\n    Ecto.Multi.insert(multi, :example_projection, %ExampleProjection{name: name})\n  end\nend\n```","ref":"read-model-projections.html#example"},{"type":"extras","title":"Consistency guarantee - Read model projections","doc":"You will often choose to use `:strong` consistency for read model projections to ensure that you can query data affected by a dispatched command. In a typical web request using the [POST/Redirect/GET](https://en.wikipedia.org/wiki/Post/Redirect/Get) pattern you want to ensure the read model is up-to-date before redirecting the user to the modified resource.\n\nBy opting in to strong consistency you are guaranteed that an `:ok` reply from command dispatch indicates all strongly consistent read models will have been updated.\n\nConfigure the `consistency` option in your projector:\n\n```elixir\ndefmodule MyApp.ExampleProjector do\n  use Commanded.Projections.Ecto,\n    application: MyApp.ExampleApp,\n    name: \"ExampleProjector\",\n    consistency: :strong\nend\n```","ref":"read-model-projections.html#consistency-guarantee"},{"type":"extras","title":"Testing","doc":"# Testing\n\nWhen using ES/CQRS, events are first-class citizens. It's critical to be able to assert that specific events are being emitted. Commanded provides test helpers to simplify your life.","ref":"testing.html"},{"type":"extras","title":"Setting up your test environment / your tests - Testing","doc":"Please refer to the [Testing your application](https://github.com/commanded/commanded/wiki/Testing-your-application) page on the Wiki for help with configuring your test environment.","ref":"testing.html#setting-up-your-test-environment-your-tests"},{"type":"extras","title":"Remember `projection_versions` when truncating tables - Testing","doc":"If you rely on your read projections in your tests, remember to truncate the `projection_versions` table in your `truncate_readstore_tables/0` function. Otherwise, your projector will ignore everything but the first projection.\n\n```elixir\ndefp truncate_readstore_tables do\n    \"\"\"\n    TRUNCATE TABLE\n      table1,\n      table2,\n      table3,\n      projection_versions\n    RESTART IDENTITY\n    CASCADE;\n    \"\"\"\n  end\n```","ref":"testing.html#remember-projection_versions-when-truncating-tables"},{"type":"extras","title":"Asserting that an event is published - Testing","doc":"Often you'll want to make sure a given event is published. Commanded provides `assert_receive_event/3` and `assert_receive_event/4` functions in the `Commanded.Assertions.EventAssertions` module to help with this.\n\nIn the first case, we check that any event is received and use the argument as an assertion.\n\nIn the second case, we also provide a predicate function that we can use to narrow our search down to a specific event.\n\n```elixir\nimport Commanded.Assertions.EventAssertions\n\ntest \"ensure any event of this type is published\" do\n  :ok = MyApp.dispatch(%Command{id: 4, date: Date.today})\n\n  assert_receive_event(MyApp, Event, fn event ->\n    assert event.id == 4\n  end)\nend\n\ntest \"ensure an event is published matching the given predicate\" do\n  :ok = MyApp.dispatch(%Command{id: 4, date: Date.today})\n\n  assert_receive_event(\n    MyApp,\n    Event,\n    fn event -> event.id == 4 end,\n    fn event ->\n      assert event.date == Date.today\n    end\n  )\nend\n```","ref":"testing.html#asserting-that-an-event-is-published"},{"type":"extras","title":"Waiting for an event - Testing","doc":"Use the `wait_for_event/2` and `wait_for_event/3` functions to pause until a specific type of event, or event type matching a given predicate, is received. This can help you deal with eventual consistency in your tests.\n\n```elixir\nimport Commanded.Assertions.EventAssertions\n\ntest \"pause until specific event is published\" do\n  :ok = BankApp.dispatch(%OpenBankAccount{account_number: \"ACC123\", initial_balance: 1_000})\n\n  wait_for_event(BankApp, BankAccountOpened, fn opened -> opened.account_number == \"ACC123\" end)\nend\n```","ref":"testing.html#waiting-for-an-event"},{"type":"extras","title":"Correlation - Testing","doc":"It's a given that when going through CQRS, sometimes many events are part of the same action, either because they are returned from the aggregate together, or because event handlers trigger new commands which generate new events, etc. We will usually want to know, for audit trail purposes, that these events belong together.\n\nFor this purpose, Commanded provides `assert_correlated/4` which can be used to ensure that specific events have the same `correlation_id`:\n\n```elixir\nimport Commanded.Assertions.EventAssertions\n\ntest \"make sure two events are correlated\" do\n  :ok = BankApp.dispatch(%OpenBankAccount{account_number: \"ACC123\", initial_balance: 1_000})\n\n  assert_correlated(\n    BankApp,\n    BankAccountOpened, fn opened -> opened.account_number == \"ACC123\" end,\n    InitialAmountDeposited, fn deposited -> deposited.account_number == \"ACC123\" end\n  )\nend\n```","ref":"testing.html#correlation"},{"type":"extras","title":"Aggregate state testing - Testing","doc":"Sometimes it's useful to compare an expected aggregate's state with the previous one. This kind of function should be used\nonly for testing.\n\nFor this purpose, Commanded provides an `aggregate_state` function which returns the current aggregate state.\n\n```elixir\nimport Commanded.Assertions.EventAssertions\n\nalias Commanded.Aggregates.Aggregate\n\ntest \"make sure aggregate state are what we wanted\" do\n  account_number = \"ACC123\"\n\n  :ok = BankApp.dispatch(%OpenBankAccount{account_number: account_number, initial_balance: 1_000})\n  :ok = BankApp.dispatch(%WithdrawnMoney{account_number: account_number, amount: 200})\n\n  wait_for_event(BankApp, BankAccountOpened, fn opened -> opened.account_number == \"ACC123\" end)\n  wait_for_event(BankApp, MoneyWithdrawn, fn withdrawn -> withdrawn.balance == 800 end)\n\n  assert Aggregate.aggregate_state(BankApp, BankAccount, account_number) == %BankAccount{\n    account_number: account_number,\n    balance: 800,\n    state: :active\n  }\nend\n```","ref":"testing.html#aggregate-state-testing"},{"type":"extras","title":"Tests using the event store and read store - Testing","doc":"To test your application using an event store and read model projection you can take advantage of ExUnit's case template feature to have the databases reset between each test execution. This guarantees that each test starts from a known good state and one test won't affect any other.\n\nFirst, define a `DataCase` module which is used to reset the event store and read store databases after each test run using the `on_exit/0` callback:\n\n```elixir\n# test/support/data_case.ex\ndefmodule MyApp.DataCase do\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      import Ecto\n      import Ecto.Changeset\n      import Ecto.Query\n      import Commanded.Assertions.EventAssertions\n    end\n  end\n\n  setup do\n    {:ok, _} = Application.ensure_all_started(:my_app)\n\n    on_exit(fn ->\n      :ok = Application.stop(:my_app)\n\n      MyApp.Storage.reset!()\n    end)\n\n    :ok\n  end\nend\n```\n\nThe `DataCase` module uses the following `Storage.reset!/0` function to:\n\n1. Reset the Postgres EventStore database.\n2. Truncate the listed tables in the read store database.\n\nRename `table1`, `table2`, and `table3` to your own table names and remember to include any new tables when added to your app.\n\n```elixir\n# test/support/storage.ex\ndefmodule MyApp.Storage do\n  @doc \"\"\"\n  Clear the event store and read store databases\n  \"\"\"\n  def reset! do\n    reset_eventstore()\n    reset_readstore()\n  end\n\n  defp reset_eventstore do\n    config = MyEventStore.config()\n\n    {:ok, conn} = Postgrex.start_link(config)\n\n    EventStore.Storage.Initializer.reset!(conn, config)\n  end\n\n  defp reset_readstore do\n    config = Application.get_env(:my_app, MyApp.Repo)\n\n    {:ok, conn} = Postgrex.start_link(config)\n\n    Postgrex.query!(conn, truncate_readstore_tables(), [])\n  end\n\n  defp truncate_readstore_tables do\n    \"\"\"\n    TRUNCATE TABLE\n      table1,\n      table2,\n      table3\n    RESTART IDENTITY\n    CASCADE;\n    \"\"\"\n  end\nend\n```\n\nYou need to include the `test/support` files in the test environment Elixir paths by adding the following `elixirc_paths/1` function to your app's `mix.exs` file:\n\n```elixir\n# mix.exs\ndefmodule MyApp.Mixfile do\n  use Mix.Project\n\n  # Include `test/support` files in test environment\n  defp elixirc_paths(:test), do: [\"lib\", \"test/support\"]\n  defp elixirc_paths(_), do: [\"lib\"]\nend\n```\n\nFinally, you can use the `MyApp.DataCase` case template within any test modules that require access to the databases:\n\n```elixir\n# test/example_test.exs\ndefmodule MyApp.ExampleTest do\n  use MyApp.DataCase\n\n  # Each test will be run against clean read and write databases.\n  # After test execution (regardless of success or failure) the databases will be reset.\nend\n```\n\nRun your tests using: `mix test`\n\nWhen these tests run they will execute against empty event store and read store databases. The caveat with the approach is that the databases will be reset after your tests run; it won't be possible to look at data contained within database after a test fails to debug the failure. The workaround is to temporarily disable the reset operation, run a single failing test, and then you will be able to look at the data. Note that the next time you run any test it will fail because the databases won't have been reset. Subsequent tests will behave normally, assuming you reinstate the reset behaviour.\n\n---\n\n#### Using the in-memory event store for testing\n\nYou can use the [[InMemoryEventStore]] to test your application. You can also use an [ExUnit `CaseTemplate` to have the in-memory event store restarted between each test run](https://github.com/commanded/commanded/wiki/In-memory-event-store#exunit-case-template).\n\n---\n\n#### Using `strong` consistency for tests only\n\nYou can configure environment specific consistency setting for Commanded event handlers:\n\n```elixir\n# config/config.exs\nuse Mix.Config\n\nconfig :my_app, consistency: :eventual\n```\n\n```elixir\n# config/test.exs\nuse Mix.Config\n\nconfig :my_app, consistency: :strong\n```\n\nThen read the setting when defining your event handlers and process managers:\n\n```elixir\ndefmodule ExampleEventHandler do\n  use Commanded.Event.Handler,\n    name: __MODULE__,\n    consistency: Application.get_env(:my_app, :consistency, :eventual)\nend\n```\n\n---\n\n#### Given *events* / When *command* / Then *assert* tests\n\nIn your test you can append events to the aggregate's event stream to setup its *given* state. Use `Commanded.EventStore.append_to_stream/4` to append events directly to the event store you've configured to use with Commanded. This allows you to configure a different event store for each environment (e.g. in-memory event store for test env).\n\nYou need to map your app's domain events to `Commanded.EventStore.EventData` structs as follows:\n\n```elixir\ncausation_id = UUID.uuid4()\ncorrelation_id = UUID.uuid4()\n\nevent_data =\n  Enum.map(events, fn -> event\n    %Commanded.EventStore.EventData{\n      causation_id: causation_id,\n      correlation_id: correlation_id,\n      event_type: Commanded.EventStore.TypeProvider.to_string(event),\n      data: event,\n      metadata: %{},\n    }\n  )\n\n{:ok, _} = Commanded.EventStore.append_to_stream(application, stream_uuid, expected_version, event_data)\n```\n\nThe `stream_uuid` will be your aggregate's identity and `expected_version` is the aggregate version (count of events already appended to its stream, use `0` when creating a new aggregate).\n\nOnce you've appended the events, you can dispatch the command via your router. The aggregate process will be started, it'll fetch its events, including those you just appended, and then handle the command.","ref":"testing.html#tests-using-the-event-store-and-read-store"},{"type":"extras","title":"Deployment","doc":"# Deployment\n\nCommanded supports running on a single node, or multiple nodes run as either a [distributed Erlang](http://erlang.org/doc/reference_manual/distributed.html) cluster or as multiple single instance nodes.","ref":"deployment.html"},{"type":"extras","title":"Single node deployment - Deployment","doc":"Running your app using Commanded on a single node requires no configuration as local is the default setting.","ref":"deployment.html#single-node-deployment"},{"type":"extras","title":"Multi node distributed Erlang deployment - Deployment","doc":"To support deployment to a cluster of nodes and distributed Erlang you must configure:\n\n1. A registry which supports distributed Erlang. The `:global` registry provided by Commanded or the [Commanded Swarm registry](https://github.com/commanded/commanded-swarm-registry) library.\n2. [Phoenix's distributed pub/sub and presence platform](https://hex.pm/packages/phoenix_pubsub) to allow process distribution and communication amongst all nodes in the cluster.\n\n### `:global` registry\n\nUse Erlang's [`:global`](http://erlang.org/doc/man/global.html) name registration facility with distributed Erlang. The global name server starts automatically when a node is started. The registered names are stored in replicated global name tables on every node. Thus, the translation of a name to a pid is fast, as it is always done locally.\n\nDefine the `:global` registry for your application:\n\n```elixir\ndefmodule MyApp.Application do\n  use Commanded.Application,\n    otp_app: :my_app,\n    event_store: [\n      adapter: Commanded.EventStore.Adapters.EventStore,\n      event_store: MyApp.EventStore\n    ],\n    registry: :global\nend\n```\n\nOr configure your application to use the `:global` registry in config:\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.Application, registry: :global\n```\n\nNote that when clusters are formed dynamically (e.g. using [libcluster](https://hex.pm/packages/libcluster)]), the typical sequence of events is that first all nodes will start all processes, then the cluster is formed and `:global` will kill off duplicate names. This is ugly in the logs but expected; it also means that if your supervisor's `:max_restarts` is too low - lower than the number of event handlers/projectors you start - it will immediately exit and if that was your\napplication supervisor, your app gets shutdown. The solution is simple: keep `:max_restarts` above the number of event handlers you start under your supervisor and the transition from no cluster to cluster will be clean.","ref":"deployment.html#multi-node-distributed-erlang-deployment"},{"type":"extras","title":"Commanded Swarm registry - Deployment","doc":"Add `commanded_swarm_registry` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:commanded_swarm_registry, \"~> 1.0\"}]\nend\n```\n\nFetch mix dependencies:\n\n```console\n$ mix deps.get\n```\n\nConfigure your application to use the Swarm registry:\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.Application, registry: Commanded.Registration.SwarmRegistry\n```\n\nThis uses the [Swarm](https://hex.pm/packages/swarm) to distribute processes amongst the available nodes in the cluster.","ref":"deployment.html#commanded-swarm-registry"},{"type":"extras","title":"Phoenix pub/sub - Deployment","doc":"First, add it as a dependency to your project's `mix.exs` file:\n\n```elixir\ndefp deps do\n  [{:phoenix_pubsub, \"~> 2.0\"}]\nend\n```\n\nFetch mix dependencies and configure the pubsub settings for your application in the environment config file:\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.Application,\n  pubsub: [\n    phoenix_pubsub: [\n      adapter: Phoenix.PubSub.PG2,\n      pool_size: 1\n    ]\n  ]\n```\n\nThe PG2 adapter is preferable for cluster deployment since it is provided by Erlang and requires no further dependencies.","ref":"deployment.html#phoenix-pub-sub"},{"type":"extras","title":"EventStore - Deployment","doc":"If using PostgreSQL-based Elixir EventStore please also refer to its documentation about [running on a clustering of nodes](https://hexdocs.pm/eventstore/cluster.html).","ref":"deployment.html#eventstore"},{"type":"extras","title":"Multi-node, but not distributed Erlang deployment - Deployment","doc":"Running multiple nodes, but choosing not to connect the nodes together to form a distributed Erlang cluster, requires that you use the local registry and Phoenix's pub/sub library with its Redis adapter.\n\nYou must install and use Phoenix's pub/sub library, [as described above](#phoenix-pub-sub).\n\nSince the nodes aren't connected, you are required to use the Redis adapter as a way of communicating between the nodes. Therefore you will need to host a Redis instance for use by your app.\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.Application,\n  registry: :local,\n  pubsub: [\n    phoenix_pubsub: [\n      adapter: Phoenix.PubSub.Redis,\n      host: \"localhost\",\n      port: 6379,\n      node_name: \"localhost\"\n    ]\n  ]\n```","ref":"deployment.html#multi-node-but-not-distributed-erlang-deployment"},{"type":"extras","title":"Upgrade guide v0.19.x to v1.0","doc":"# Upgrade guide v0.19.x to v1.0\n\nIn v0.19.x and earlier a singleton Commanded application was used to host aggregates and support event handlers, process managers, and other infrastructure processes. For v1.0, [support for multiple Commanded apps](https://github.com/commanded/commanded/pull/298) was added allowing you to define and use more than one Commanded app and to control its lifecycle. Follow the following upgrade advice to migrate your application to Commanded v1.0.","ref":"0.19-1.0.html"},{"type":"extras","title":"Commanded application - Upgrade guide v0.19.x to v1.0","doc":"First you must define a Commanded application:\n\n```elixir\ndefmodule MyApp.Application do\n  use Commanded.Application, otp_app: :my_app\n\n  router(MyApp.Router)\nend\n```\n\nThe application needs to be configured. You can do so in application config:\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.Application,\n  event_store: [\n    adapter: Commanded.EventStore.Adapters.EventStore,\n    event_store: MyApp.EventStore\n  ],\n  pubsub: :local,\n  registry: :local\n```\n\nAlternatively, you can include the event store, pubsub, and registry config when defining the application:\n\n```elixir\ndefmodule MyApp.Application do\n  use Commanded.Application,\n    otp_app: :my_app,\n    event_store: [\n      adapter: Commanded.EventStore.Adapters.EventStore,\n      event_store: MyApp.EventStore\n    ],\n    pubsub: :local,\n    registry: :local\n\n  router(MyApp.Router)\nend\n```","ref":"0.19-1.0.html#commanded-application"},{"type":"extras","title":"Event store configuration - Upgrade guide v0.19.x to v1.0","doc":"To use Commanded v1.0 you must also upgrade the adapter for the event store you are using.\n\n#### Postgres EventStore\n\nUse the [commanded_eventstore_adapter](https://hex.pm/packages/commanded_eventstore_adapter/1.0.0) and follow the [Getting started guide](https://hexdocs.pm/commanded_eventstore_adapter/1.0.0/getting-started.html).\n\nIn brief, the upgrade requires you to:\n\n1. Define an event store module for your Commanded application:\n\n    ```elixir\n    defmodule MyApp.EventStore do\n      use EventStore, otp_app: :my_app\n    end\n    ```\n\n2. Add a config entry containing the PostgreSQL database connection details for your event store module to each environment's mix config file (e.g. `config/dev.exs`):\n\n    ```elixir\n    config :my_app, MyApp.EventStore,\n      serializer: Commanded.Serialization.JsonSerializer,\n      username: \"postgres\",\n      password: \"postgres\",\n      database: \"eventstore\",\n      hostname: \"localhost\"\n    ```\n\n3. Configure the Commanded application to use the event store:\n\n    ```elixir\n    defmodule MyApp.Application do\n      use Commanded.Application,\n        otp_app: :my_app,\n        event_store: [\n          adapter: Commanded.EventStore.Adapters.EventStore,\n          event_store: MyApp.EventStore\n        ]\n    end\n    ```\n\n#### Event Store\n\nUse the [commanded_extreme_adapter](https://hex.pm/packages/commanded_extreme_adapter/1.0.0) and follow the [Getting started guide](https://hexdocs.pm/commanded_extreme_adapter/1.0.0/getting-started.html).","ref":"0.19-1.0.html#event-store-configuration"},{"type":"extras","title":"Supervision - Upgrade guide v0.19.x to v1.0","doc":"The Commanded application must be included and started somewhere in your app's supervision tree. You can include it within the top level application supervisor:\n\n```elixir\ndefmodule MyApp do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      MyApp.Application\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end  \nend\n```","ref":"0.19-1.0.html#supervision"},{"type":"extras","title":"Command dispatch - Upgrade guide v0.19.x to v1.0","doc":"Previously commands were always dispatched via a router module. Now, you dispatch commands using your Commanded application:\n\n```elixir\n:ok = MyApp.Application.dispatch(%DoSomething{..})\n```\n\nThe above should be a straightforward find within your source code to replace the router module (e.g. `MyApp.Router`) with the new application module (e.g. `MyApp.Application`).\n\nOptionally, you can dispatch a command using an existing router by configuring the router with the Commanded application:\n\n```elixir\ndefmodule MyApp.Router do\n  use Commanded.Commands.Router, application: MyApp.Application\nend\n\n:ok = MyApp.Router.dispatch(%DoSomething{..})\n```\n\nIt is also possible to specify the application during command dispatch:\n\n```elixir\n:ok = MyApp.Router.dispatch(%DoSomething{..}, application: MyApp.Application)\n```","ref":"0.19-1.0.html#command-dispatch"},{"type":"extras","title":"Event handlers and process managers - Upgrade guide v0.19.x to v1.0","doc":"Each handler, read model projections, and process manager needs to be provided with the additional `application` option:\n\n```elixir\ndefmodule MyApp.ExampleHandler do\n  use Commanded.Event.Handler,\n    application: MyApp.Application,\n    name: \"MyApp.ExampleHandler\"\nend\n```","ref":"0.19-1.0.html#event-handlers-and-process-managers"},{"type":"extras","title":"Upgrade Commanded supporting libraries - Upgrade guide v0.19.x to v1.0","doc":"You will need to upgrade to the v1.0 versions of any Commanded related library used by your application, such as the event store adapter and Ecto read model projections.","ref":"0.19-1.0.html#upgrade-commanded-supporting-libraries"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}